! *************************************************************************************************
! sue_sed.f90
! Integral sediment system routines
! *************************************************************************************************



MODULE sue_sed


  USE sue_config
  USE sue_lib
  USE sue_co2calc
  USE sue_sed_lib
  IMPLICIT NONE
  SAVE
  
  
CONTAINS
  

! ******************************************************************
! *** initialize sediment sub-system: sedimentary layers content ***
! ******************************************************************
  SUBROUTINE init_sed_layers_default(sed,sedtop)
    IMPLICIT NONE
! dummy variables
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_tot,n_sed_com_par)::sed
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop
! local variables
    INTEGER::m,c,o
! start of water column loop
    DO c = 1,n_col
! start of sediment depth level loop
      DO m = 1,n_sed_depth
! set default sediment stack values
! NOTE: sediment component volumes are in the units of 
!       actual volume of solid matter per cm2 area of sub-layer
! NOTE: initialize total sediment solids age tied to refractory volume
!       (as it comprises 100% of sediment on initialization)
	sedtop(c,m,:)               = 0.0
	sedtop(c,m,ised_refrac)     = (1.0 - par_sed_poros_top) * par_sed_top_th
	DO o = 1,n_sed_tot_init
	  sed(c,m,o,:)               = 0.0
	  sed(c,m,o,ised_refrac)     = (1.0 - par_sed_poros)
	END DO
	DO o = (n_sed_tot_init + 1),n_sed_tot
	  sed(c,m,o,:) = 0.0
	END DO
      END DO
    END DO
  END SUBROUTINE init_sed_layers_default
! ******************************************************************


! *************************************************
! *** initialize sediment sub-system parameters ***
! *************************************************
  SUBROUTINE init_sed_par(sedpar)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
! local variables
    INTEGER::m,c
! start of water column loop
    DO c = 1,n_col
! set height of top layer of old sediment
      sedpar(c,:,ised_par_top) = REAL(n_sed_tot_init,KIND=fp_1)
! set water column number
      sedpar(c,:,ised_par_colnum) = c
! set default sediment depths
      sedpar(c,:,ised_par_D_top) = sedpar(c,:,ised_par_D_PIG_top)
      sedpar(c,:,ised_par_D_bot) = sedpar(c,:,ised_par_D_PIG_bot)
      sedpar(c,:,ised_par_D_mid) = sedpar(c,:,ised_par_D_PIG_mid)
    END DO
  END SUBROUTINE init_sed_par
! *************************************************


! ***********************************************************************************
! *** calculate required bathymetry data for sedimentary layers from raw data-set ***
! ***********************************************************************************
  SUBROUTINE calc_bathdata_sed(bathdata,sedpar,PIGphys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bath_depth,n_bath_par)::bathdata
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::PIGphys
! local variables
    INTEGER::c=0,m=0,o=0
    REAL(KIND=fp_1)::areafrac=0.0,sumareafrac=0.0

! *** read in raw area fraction data-set and integrate between model depth intervals ***
! NOTE: use PIG sediment depths, rather than 'current' values,
! because the ocean bathymetry data-set is tied to a PIG depth scheme
! NOTE: the loop direction is from the bottom of the bathymetry data-set to the top
    DO  c = 1,n_col
      areafrac = 0.0
      m = 1
      o = 1
      DO
	IF (bathdata(c,o,ibath_D) <= sedpar(c,m,ised_par_D_PIG_bot) .and. o <= n_bath_depth) THEN
! increment area fraction of sediment layer
! (if the bathymetry depth is less than the lower depth of the sediment layer)
	  areafrac = areafrac + bathdata(c,o,ibath_Afrac)
	ELSE
	  sedpar(c,m,ised_par_Afrac) = areafrac ! store area fraction value
	  m = m + 1
	  IF (m > n_sed_depth) THEN
	    EXIT ! exit the current DO loop: the top sediment column has been exceeded
	  ENDIF
	  areafrac = 0.0
	  areafrac = areafrac + bathdata(c,o,ibath_Afrac)
	ENDIF
	o = o + 1
	IF (o > n_bath_depth) THEN
	  sedpar(c,m,ised_par_Afrac) = areafrac ! store area fraction value
	  EXIT ! exit the current DO loop: the top of the bathymetry has been exceeded
	ENDIF
      END DO
    END DO
! check that area fraction in each column adds up to about 1.0!
    DO c = 1,n_col
      sumareafrac = SUM(sedpar(c,:,ised_par_Afrac))
      IF (sumareafrac < 0.999999 .OR. sumareafrac > 1.000001) THEN
	PRINT*,"FATAL ERROR [sue_sed.f90, calc_bathdata_sed]:"
	PRINT*,"ocean-floor area fraction = ",sumareafrac
	PRINT*,"ocean column              = ",c
	PRINT*,""
	STOP
      ENDIF
    END DO
    
! *** calculate corresponding area data-set ***
! NOTE: keep in units of m2
    DO c = 1,n_col
      sedpar(c,:,ised_par_A) =  sedpar(c,:,ised_par_Afrac) * PIGphys(c,1,ip_A_top)
    END DO

  END SUBROUTINE calc_bathdata_sed
! ***********************************************************************************


! ******************************************************************************
! *** calculate area fraction of sediments below mixed ocean (surface) layer ***
! ******************************************************************************
  SUBROUTINE calc_Afrac_np(sedpar,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,m
    REAL(KIND=fp_1),DIMENSION(n_col)::sumareafrac = 0.0
! calculate the area fraction of each sedimentary layer,
! w.r.t. the area of the base of the mixed (surface) ocean layer
! NOTE: sediment layers which lie wholly within the mixed (surface) layer
!       are assigned an area fraction of 0.0
! NOTE: sediment layers which lie partially within the mixed (surface) layer
!       are assigned an area fraction in proportion to the depth fraction of the sediment layer 
!       lying below the mixed layer
! NOTE: rescale all the fractional areas to sum to unity
    DO c = 1,n_col
      sedpar(c,:,ised_par_Afrac_np) = 0.0
      DO m = n_sed_depth,1,-1
	IF (sedpar(c,m,ised_par_D_top) > phys(c,1,ip_D_bot)) THEN
	  sedpar(c,m,ised_par_Afrac_np) = sedpar(c,m,ised_par_A) / phys(c,1,ip_A_bot)
	ELSEIF (sedpar(c,m,ised_par_D_bot) <= phys(c,1,ip_D_bot)) THEN
	  sedpar(c,m,ised_par_Afrac_np) = 0.0
	ELSE
	  sedpar(c,m,ised_par_Afrac_np) = (sedpar(c,m,ised_par_A) / phys(c,1,ip_A_bot)) * &
	    & (sedpar(c,m,ised_par_D_bot) - phys(c,1,ip_D_bot)) / &
	    & (sedpar(c,m,ised_par_D_bot) - sedpar(c,m,ised_par_D_top))
	ENDIF
      ENDDO
      sedpar(c,:,ised_par_Afrac_np) = sedpar(c,:,ised_par_Afrac_np) / SUM(sedpar(c,:,ised_par_Afrac_np))
    ENDDO
! check that the fractional areas sum to unity
    DO c = 1,n_col
      sumareafrac(c) = SUM(sedpar(c,:,ised_par_Afrac_np))
      IF (sumareafrac(c) < 0.999999 .OR. sumareafrac(c) > 1.000001) THEN
	PRINT*,"FATAL ERROR [sue_sed.f90, calc_Afrac_np]:"
	PRINT*,"sediment area as fraction of base of phototic zone = ",sumareafrac(c)
	PRINT*,"ocean column number                                = ",c
	PRINT*,""
	STOP
      ENDIF
    END DO
  END SUBROUTINE calc_Afrac_np
! ******************************************************************************


! ******************************************************************
! *** update sediment sub-system bathymetry from sealevel change ***
! ******************************************************************
  SUBROUTINE update_sed_bath(sedpar,sealevel)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in)::sealevel
! calculate new depths taking into account sea-level changes
! NOTE: sea-level falls are represented by a NEGATIVE value
! NOTE: the depth below sealevel of the top of the uppermost sediment layer can go negative!
    sedpar(:,:,ised_par_D_top) = sedpar(:,:,ised_par_D_PIG_top) + sealevel
    sedpar(:,:,ised_par_D_bot) = sedpar(:,:,ised_par_D_PIG_bot) + sealevel
    sedpar(:,:,ised_par_D_mid) = (sedpar(:,:,ised_par_D_top) + sedpar(:,:,ised_par_D_bot)) / 2.0
  END SUBROUTINE update_sed_bath
! ******************************************************************


! ****************************************************
! *** CALCULATE REMINERALIZATION FUNCTION PROFILES ***
! ****************************************************
  SUBROUTINE calc_remin_frac_sed(tracer,sedtracer,phys,sedpar)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_tracer)::tracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_tracer)::sedtracer
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
! local variables
    INTEGER::c,l,m,p
    INTEGER::sed_lay_mid           ! mid-sediment/ocean adjacent ocean layer number
    REAL(KIND=fp_1),DIMENSION(n_lay)::sed_lay_frac ! sediment/ocean layer fractional adjacencies
    REAL(KIND=fp_1)::sed_lay_Dtop  ! sediment/ocean adjacent top layer depth
    REAL(KIND=fp_1)::sed_lay_Dbot  ! sediment/ocean adjacent bottom layer depth
    REAL(KIND=fp_1)::sed_D_calsat  ! ocean calcite saturation depth w.r.t. sedimentary layer
    REAL(KIND=fp_1)::sed_D_argsat  ! ocean aragonite saturation depth w.r.t. sedimentary layer
    REAL(KIND=fp_1)::delta_Dargsat ! depth below Dargsat (m)
    REAL(KIND=fp_1)::av_Dargsat    ! mean (mass-weighted) aragonite saturation depth (m)
    REAL(KIND=fp_1)::arg_remin_top ! value of the arg remin function at ocean layer top depth
    REAL(KIND=fp_1)::arg_remin_bot ! value of the arg remin function at ocean layer bottom depth
    REAL(KIND=fp_1)::arg_remin_sed ! value of the arg remin function at sediment layer (mid) depth
    REAL(KIND=fp_1)::et_opal_Ganan ! opal remin e-folding time using the Gnanadesikan [1999] relationship
    REAL(KIND=fp_1)::lay_res       ! residence time within ocean layer (d)
    REAL(KIND=fp_1)::lay_Si_eq     ! equilibrium [H4SiO4] within ocean layer (umol kg-1)
    REAL(KIND=fp_1)::lay_u         ! degree of undersaturation (w.r.t. opal) within ocean layer
    

! *** calculate water column remineralization above integral sediment sub-system sediments ***

! initialize variable default values
    phys(:,:,ip_reminf_Corg)         = 1.0
    phys(:,:,ip_reminf_opal)         = 1.0
    phys(:,:,ip_reminf_cal)          = 1.0
    phys(:,:,ip_reminf_arg)          = 1.0
    sedpar(:,:,ised_par_reminf_Corg) = 1.0
    sedpar(:,:,ised_par_reminf_opal) = 1.0
    sedpar(:,:,ised_par_reminf_cal)  = 1.0
    sedpar(:,:,ised_par_reminf_arg)  = 1.0

    DO c = 1,n_col
      
! calculate remineralization function values for each ocean layer
! NOTE: this is calculated as the fraction of new production NOT remineralized
! at depths corresponding to the bottom of each ocean layer
! NOTE: the remineralization depth is initialized to minus the depth of the surface layer,
! so that the depth will be reset to zero at the base of the surface layer in the loop calculation
! NOTE: in calculating the total opal remineralization at the depth of the sediments,
!       first calculate the total opal remineralization within the ocean layers lying 
!       wholly above the mid-point of the sediment layer,
!       then assume that the mean sediment layer temperature is representative of the remaining distance
!       from the top of the first ocean layer to have lay_D_bot < sed_D_mid
! (1) Corg
      SELECT CASE (bioopt(bopt_remin_Corg_alt))
      CASE ("1")
!     scheme #1 : exponential scheme
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    phys(c,l,ip_reminf_Corg) = par_remin_fracrecal_Corg + &
	      & (1.0 - par_remin_fracrecal_Corg) * &
	      & EXP(-(phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_eL_Corg)
	  ELSE
	    phys(c,l,ip_reminf_Corg) = 1.0
	  END IF
	END DO
      CASE ("2")
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    phys(c,l,ip_reminf_Corg) = par_remin_fracrecal_Corg +                    &
	      & (1.0 - par_remin_fracrecal_Corg) *                                   &
	      & (                                                                    &
	      &   (par_remin_pL_Corg + phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) &
	      &   /                                                                  &
	      &   par_remin_pL_Corg                                                  &
	      & )**par_remin_pP_Corg 
	  ELSE
	    phys(c,l,ip_reminf_Corg) = 1.0
	  END IF
	END DO
      END SELECT
! (2) opal
      SELECT CASE (bioopt(bopt_remin_opal_alt))
      CASE ("1")
!     scheme #1 : exponential scheme
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    phys(c,l,ip_reminf_opal) = &
	      & EXP(-(phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_eL_opal)
	  ELSE
	    phys(c,l,ip_reminf_opal) = 1.0
	  END IF
	ENDDO
      CASE ("2")
!     scheme #2 : 
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
	      lay_res = (phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
	    ELSE
	      lay_res = (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) / par_remin_opal_vel
	    END IF
	    et_opal_Ganan = -1.0 / (LOG(1.0 - 1.32E+16 * EXP(-11481.0 / phys(c,l,ip_T))))
	    phys(c,l,ip_reminf_opal) = phys(c,l - 1,ip_reminf_opal) * &
	      & EXP(-lay_res / et_opal_Ganan)
	    IF (phys(c,l,ip_reminf_opal) < 0.0) phys(c,l,ip_reminf_opal) = 0.0
	  ELSE
	    phys(c,l,ip_reminf_opal) = 1.0
	  END IF
	ENDDO
      CASE ("3")
!     scheme #3 :
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    lay_Si_eq = conv_umol_mol * 10.0**(6.44 - 968.0 / tracer(c,l,it_T))
	    lay_u     = (lay_Si_eq - tracer(c,l,it_Si)) / lay_Si_eq
	    IF (lay_u > 1.0) lay_u = 1.0
	    IF (lay_u < 0.0) lay_u = 0.0
	    IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
	      lay_res = (phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
	    ELSE
	      lay_res = (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) / par_remin_opal_vel
	    END IF
	    phys(c,l,ip_reminf_opal) = phys(c,l - 1,ip_reminf_opal) *                              &
	      & (1.0 -                                                                             &
	      &   lay_res * par_remin_opal_K *                                                     &
	      &   (1.0 / 0.71) *                                                                   &
	      &   (                                                                                &
	      &     (0.16 * (1.0 + (tracer(c,l,it_T) - const_zeroC) / 15.0) * lay_u) +             &
	      &     (0.55 * ((1.0 + (tracer(c,l,it_T) - const_zeroC) / 400.0)**4.0 * lay_u)**9.25) &
	      &   )                                                                                &
	      & )
	    IF (phys(c,l,ip_reminf_opal) < 0.0) phys(c,l,ip_reminf_opal) = 0.0
	  ELSE
	    phys(c,l,ip_reminf_opal) = 1.0
	  END IF
	ENDDO
      END SELECT
! (3) calcite
      SELECT CASE (bioopt(bopt_remin_cal_alt))
      CASE ("1")
!     scheme #1 : none
	phys(c,:,ip_reminf_cal) = 1.0
      CASE ("2")
!     scheme #2 : exponential scheme
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    phys(c,l,ip_reminf_cal) = par_remin_fracrecal_cal + &
	      & (1.0 - par_remin_fracrecal_cal) *               &
	      & EXP(-(phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_eL_cal)
	  ELSE
	    phys(c,l,ip_reminf_cal) = 1.0
	  END IF
	END DO
      END SELECT
! (4) aragonite
      SELECT CASE (bioopt(bopt_remin_arg_alt))
      CASE ("1")
!     scheme #1 : none
	phys(c,:,ip_reminf_arg) = 1.0
      CASE ("2")
!     scheme #2 : exponential scheme
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    delta_Dargsat = phys(c,l,ip_D_bot) - phys(c,l,ip_D_argsat)
	    IF (delta_Dargsat < 0.0) THEN
	      phys(c,l,ip_reminf_arg) = 1.0
	    ELSE
	      phys(c,l,ip_reminf_arg)  = &
		& EXP(-delta_Dargsat / par_remin_eL_arg)
	    ENDIF
	  ELSE
	    phys(c,l,ip_reminf_arg) = 1.0
	  END IF
	END DO
      CASE ("3")
!     scheme #3 : 
	av_Dargsat =                                           &
	  & SUM                                                &
	  & (                                                  &
	  &   phys(c,par_remin_nlbase + 1:n_lay,ip_D_argsat) * &
	  &   phys(c,par_remin_nlbase + 1:n_lay,ip_M)          &
	  & )                                                  &
	  & /                                                  &
	  & SUM(phys(c,par_remin_nlbase + 1:n_lay,ip_M))
	DO l = 1,n_lay
	  IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
	    delta_Dargsat = phys(c,l,ip_D_bot) - av_Dargsat
	    IF (delta_Dargsat < 0.0) THEN
	      phys(c,l,ip_reminf_arg) = 1.0
	    ELSE
	      phys(c,l,ip_reminf_arg) = 1.0 - &
		& (delta_Dargsat / 11000.0)**0.25 * (1.0 - EXP(-delta_Dargsat / 1400.0))**4.5
	    ENDIF
	    IF (phys(c,l,ip_reminf_arg) < 0.0) phys(c,l,ip_reminf_arg) = 0.0
	  END IF
	END DO
      END SELECT

! calculate remineralization function value at sediment layer mid depth
! (1) Corg
      SELECT CASE (bioopt(bopt_remin_Corg_alt))
      CASE ("1")
!     scheme #1 : exponential scheme
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedpar(c,m,ised_par_reminf_Corg) = par_remin_fracrecal_Corg + &
	      & (1.0 - par_remin_fracrecal_Corg) * &
	      & EXP(-(sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_eL_Corg)
	  ELSE
	    sedpar(c,m,ised_par_reminf_Corg) = 1.0
	  ENDIF
	ENDDO
      CASE ("2")
!     scheme #2 : 
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedpar(c,m,ised_par_reminf_Corg) = par_remin_fracrecal_Corg +                    &
	      & (1.0 - par_remin_fracrecal_Corg) *                                           &
	      & (                                                                            &
	      &   (par_remin_pL_Corg + sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)) &
	      &   /                                                                          &
	      &   par_remin_pL_Corg                                                          &
	      & )**par_remin_pP_Corg
	  ELSE
	    sedpar(c,m,ised_par_reminf_Corg) = 1.0
	  ENDIF
	ENDDO
      END SELECT
! (2) opal
      SELECT CASE (bioopt(bopt_remin_opal_alt))
      CASE ("1")
!     scheme #1 : exponential scheme
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedpar(c,m,ised_par_reminf_opal) = &
	      & EXP(-(sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_eL_opal)
	  ELSE
	    sedpar(c,m,ised_par_reminf_opal) = 1.0
	  ENDIF
	ENDDO
      CASE ("2")
!     scheme #2 : 
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedpar(c,m,ised_par_reminf_opal) = 1.0
	    DO l = 1,n_lay
	      IF (sedpar(c,m,ised_par_D_mid) > phys(c,l,ip_D_bot)) THEN
		IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
		  IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
		    lay_res = (phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
		  ELSE
		    lay_res = (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		  END IF
		  et_opal_Ganan = -1.0 / (LOG(1.0 - 1.32E+16 * EXP(-11481.0 / tracer(c,l,it_T))))
		  sedpar(c,m,ised_par_reminf_opal) = sedpar(c,m,ised_par_reminf_opal) * &
		    & EXP(-lay_res / et_opal_Ganan)
		  IF (sedpar(c,m,ised_par_reminf_opal) < 0.0) sedpar(c,m,ised_par_reminf_opal) = 0.0
		ELSE
		  sedpar(c,m,ised_par_reminf_opal) = 1.0
		END IF
	      ELSE
		IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
		  lay_res = (sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
		ELSE
		  lay_res = (sedpar(c,m,ised_par_D_mid) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		END IF
		et_opal_Ganan = -1.0 / (LOG(1.0 - 1.32E+16 * EXP(-11481.0 / sedtracer(c,m,it_T))))
		sedpar(c,m,ised_par_reminf_opal) = sedpar(c,m,ised_par_reminf_opal) * &
		  & EXP(-lay_res / et_opal_Ganan)
		IF (sedpar(c,m,ised_par_reminf_opal) < 0.0) sedpar(c,m,ised_par_reminf_opal) = 0.0
		EXIT
	      ENDIF
	    END DO
	  ELSE
	    sedpar(c,m,ised_par_reminf_opal) = 1.0
	  ENDIF
	ENDDO
      CASE ("3")
!     scheme #3 :
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedpar(c,m,ised_par_reminf_opal) = 1.0
	    DO l = 1,n_lay
	      IF (sedpar(c,m,ised_par_D_mid) > phys(c,l,ip_D_bot)) THEN
		IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
		  lay_Si_eq = conv_umol_mol * 10.0**(6.44 - 968.0 / tracer(c,l,it_T))
		  lay_u     = (lay_Si_eq - tracer(c,l,it_Si)) / lay_Si_eq
		  IF (lay_u > 1.0) lay_u = 1.0
		  IF (lay_u < 0.0) lay_u = 0.0
		  IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
		    lay_res = (phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
		  ELSE
		    lay_res = (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		  END IF
		  sedpar(c,m,ised_par_reminf_opal) = sedpar(c,m,ised_par_reminf_opal) *                  &
		    & (1.0 -                                                                             &
		    &   lay_res * par_remin_opal_K *                                                     &
		    &   (1.0 / 0.71) *                                                                   &
		    &   (                                                                                &
		    &     (0.16 * (1.0 + (tracer(c,l,it_T) - const_zeroC) / 15.0) * lay_u) +             &
		    &     (0.55 * ((1.0 + (tracer(c,l,it_T) - const_zeroC) / 400.0)**4.0 * lay_u)**9.25) &
		    &   )                                                                                &
		    & )
		  IF (sedpar(c,m,ised_par_reminf_opal) < 0.0) sedpar(c,m,ised_par_reminf_opal) = 0.0
		ELSE
		  sedpar(c,m,ised_par_reminf_opal) = 1.0
		END IF
	      ELSE
		lay_Si_eq = conv_umol_mol * 10.0**(6.44 - 968.0 / sedtracer(c,m,it_T))
		lay_u     = (lay_Si_eq - sedtracer(c,m,it_Si)) / lay_Si_eq
		IF (lay_u > 1.0) lay_u = 1.0
		IF (lay_u < 0.0) lay_u = 0.0
		IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
		  lay_res = (sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
		ELSE
		  lay_res = (sedpar(c,m,ised_par_D_mid) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		END IF
		sedpar(c,m,ised_par_reminf_opal) = sedpar(c,m,ised_par_reminf_opal) *                     &
		  & (1.0 -                                                                                &
		  &   lay_res * par_remin_opal_K *                                                        &
		  &   (1.0 / 0.71) *                                                                      &
		  &   (                                                                                   &
		  &     (0.16 * (1.0 + (sedtracer(c,m,it_T) - const_zeroC) / 15.0) * lay_u) +             &
		  &     (0.55 * ((1.0 + (sedtracer(c,m,it_T) - const_zeroC) / 400.0)**4.0 * lay_u)**9.25) &
		  &   )                                                                                   &
		  & )
		IF (sedpar(c,m,ised_par_reminf_opal) < 0.0) sedpar(c,m,ised_par_reminf_opal) = 0.0
		EXIT
	      ENDIF
	    END DO
	  ELSE
	    sedpar(c,m,ised_par_reminf_opal) = 1.0
	  ENDIF
	ENDDO
      END SELECT
! (3) calcite
      SELECT CASE (bioopt(bopt_remin_cal_alt))
      CASE ("1")
!     scheme #1 : none
	sedpar(c,:,ised_par_reminf_cal) = 1.0
      CASE ("2")
!     scheme #2 : 
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedpar(c,m,ised_par_reminf_cal) = par_remin_fracrecal_cal + &
	      & (1.0 - par_remin_fracrecal_cal) *                       &
	      & EXP(-(sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_eL_cal)
	  ELSE
	    sedpar(c,m,ised_par_reminf_cal) = 1.0
	  ENDIF
	ENDDO
      END SELECT
! (4) aragonite
!     NOTE: calculate sediment / ocean layer fractional connectivity and aragonite saturation depths
      SELECT CASE (bioopt(bopt_remin_arg_alt))
      CASE ("1")
!     scheme #1 : none
	sedpar(c,:,ised_par_reminf_arg) = 1.0
      CASE ("2")
!     scheme #2 : exponential scheme
	DO m = 1,n_sed_depth
	  sed_lay_frac(:) = calc_sed_lay_frac(sedpar(c,m,:),phys(c,:,:))
	  sed_D_argsat = SUM(sed_lay_frac(:) * phys(c,:,ip_D_argsat))
	  sed_lay_Dbot = SUM(sed_lay_frac(:) * phys(c,:,ip_D_bot))
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    delta_Dargsat = sedpar(c,m,ised_par_D_mid) - sed_D_argsat
	    IF (delta_Dargsat < 0.0) THEN
	      sedpar(c,m,ised_par_reminf_arg) = 1.0
	    ELSE
	      sedpar(c,m,ised_par_reminf_arg)  = &
		& EXP(-delta_Dargsat / par_remin_eL_arg)
	    ENDIF
	  ELSE
	    sedpar(c,m,ised_par_reminf_arg) = 1.0
	  ENDIF
	ENDDO
      CASE ("3")
!     scheme #3 : 
	DO m = 1,n_sed_depth
	  IF (sedpar(c,m,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    av_Dargsat =                                           &
	      & SUM                                                &
	      & (                                                  &
	      &   phys(c,par_remin_nlbase + 1:n_lay,ip_D_argsat) * &
	      &   phys(c,par_remin_nlbase + 1:n_lay,ip_M)          &
	      & )                                                  &
	      & /                                                  &
	      & SUM(phys(c,par_remin_nlbase + 1:n_lay,ip_M))
	    delta_Dargsat = sedpar(c,m,ised_par_D_mid) - av_Dargsat
	    IF (delta_Dargsat < 0.0) THEN
	      sedpar(c,m,ised_par_reminf_arg) = 1.0
	    ELSE
	      sedpar(c,m,ised_par_reminf_arg) = 1.0 - &
		& (delta_Dargsat / 11000.0)**0.25 * (1.0 - EXP(-delta_Dargsat / 1400.0))**4.5
	    ENDIF
	    IF (sedpar(c,m,ised_par_reminf_arg) < 0.0) sedpar(c,m,ised_par_reminf_arg) = 0.0
	  ELSE
	    sedpar(c,m,ised_par_reminf_arg) = 1.0
	  ENDIF
	ENDDO
      END SELECT
      
    ENDDO
    
    
  END SUBROUTINE calc_remin_frac_sed
! ****************************************************


! **********************************************************************
! *** calculate sediment sub-system sediment/ocean layer adjacencies ***
! **********************************************************************
  SUBROUTINE calc_sed_lay(sedpar,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,l,m

! *** find ocean layers adjactent to each sediment layer ***
! NOTE: the top and bottom ocean layer numbers in the sediment parameter array are real numbers,
!       but are used for storing an integer value
    DO c = 1,n_col
      DO m = 1,n_sed_depth
! (1) zero ocean layer elements
	sedpar(c,m,ised_par_lay_top) = 0.0
	sedpar(c,m,ised_par_lay_bot) = 0.0
! (2) work up through water column:
!     the first ocean layer that has a D_top value <= the sediment D_top value,
!     must be the uppermost ocean layer that can affect that sediment layer
!     NOTE: because the depth below sealevel of the top of the first sediment layer is allowed to go negative,
!           if no top ocean layer is found, then the index must be "1"
	DO l = n_lay,1,-1
	  IF (phys(c,l,ip_D_top) <= sedpar(c,m,ised_par_D_top)) THEN
	    sedpar(c,m,ised_par_lay_top) = REAL(l,KIND=fp_1)
	    EXIT
	  END IF
	END DO
	IF (sedpar(c,m,ised_par_lay_top) == 0.0) sedpar(c,m,ised_par_lay_top) = 1.0
! (3) work down through water column:
!     the first ocean layer that has a D_bot value >= the sediment D_bot value,
!     must be the lowermost ocean layer that can affect that sediment layer
!     NOTE: because sedimentary layers are allowed to fall (either in part or fully) below 
!           the bottom depth the lower most ocean layer
!           (such as might occur if a fixed depth ("flat bottom") ocean is being considered),
!           if no bottom ocean layer is found, then the index must be "n_lay"
	DO l = 1,n_lay
	  IF (phys(c,l,ip_D_bot) >= sedpar(c,m,ised_par_D_bot)) THEN
	    sedpar(c,m,ised_par_lay_bot) = REAL(l,KIND=fp_1)
	    EXIT
	  END IF
	END DO
	IF (sedpar(c,m,ised_par_lay_bot) == 0.0) sedpar(c,m,ised_par_lay_bot) = REAL(n_lay,KIND=fp_1)
      END DO
    END DO

  END SUBROUTINE calc_sed_lay
! **********************************************************************


! ********************************************************************
! *** average tracer and phys values at sediment sub-system depths ***
! ********************************************************************
  SUBROUTINE calc_sed_tracer(sedpar,sedtracer,tracer,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_tracer)::sedtracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_tracer)::tracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,l,m,it
    REAL(KIND=fp_1),DIMENSION(n_lay)::sed_lay_frac ! fractional sediment/ocean layer adjacencies

! *** calculate average tracer concentrations (and water temperature) for each sediment depth ***
! loop through all water columns, and sediment depths
    DO c = 1,n_col
      DO m = 1,n_sed_depth
! (1) calculate sediment/ocean layer adjacencies
	sed_lay_frac(:) = calc_sed_lay_frac(sedpar(c,m,:),phys(c,:,:))
! (2) calculate the average tracer concentration across all the ocean layers spanning the sediment layer
	DO it = 1,n_tracer
	  sedtracer(c,m,it) = SUM(sed_lay_frac(:) * tracer(c,:,it))
	ENDDO
! (3) calculate the average physical characteristics across all the ocean layers spanning the sediment layer
	sedpar(c,m,ised_par_T) = SUM(sed_lay_frac(:) * phys(c,:,ip_T))
      ENDDO
    ENDDO

  END SUBROUTINE calc_sed_tracer
! ********************************************************************


! *********************************************************************
! *** carbonate equilibrium constants at sediment sub-system depths ***
! *********************************************************************
  SUBROUTINE calc_sed_disconst(seddis,sedpar,sedtracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_dis)::seddis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_tracer)::sedtracer
! local variables
    INTEGER::c,m

! *** calculate carbonate equilibrium constants for each sediment depth ***
! loop through all ocean column and sediment layers,
! calculate carbonate equilibrium constants at mid sediment depths and return function value
    DO c = 1,n_col
      DO m = 1,n_sed_depth
	CALL co2const_ex(sedpar(c,m,ised_par_T),sedtracer(c,m,it_S),sedpar(c,m,ised_par_D_mid), &
	  & seddis(c,m,:))
      ENDDO
    ENDDO

  END SUBROUTINE calc_sed_disconst
! *********************************************************************


! ********************************************************
! *** calculate sediment surface carbonate equilibrium ***
! ********************************************************
  SUBROUTINE calc_sed_carbchem(sedpar,seddis,sedtracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_dis)::seddis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_tracer)::sedtracer
! local variables
    INTEGER::c,m
    REAL(KIND=fp_1)::fug_CO2,conc_CO2,conc_HCO3

! *** calculate carbonate equilibria for each sediment depth ***
! loop through all water columns and sediment depths,
! and calculate 'ohm_cal', 'ohm_arg', 'conc_Ca', 'conc_CO3' and pH values
    DO c = 1,n_col
      DO m = 1,n_sed_depth
	CALL calccarb_ex(seddis(c,m,:), &
	  & sedtracer(c,m,it_S), &
	  & sedtracer(c,m,it_alk), &
	  & sedtracer(c,m,it_CO2), &
	  & sedtracer(c,m,it_PO4), &
	  & sedtracer(c,m,it_Si), &
	  & sedpar(c,m,ised_par_Ca), &
	  & fug_CO2, &
	  & conc_CO2, &
	  & sedpar(c,m,ised_par_CO3), &
	  & conc_HCO3, &
	  & sedpar(c,m,ised_par_ohmcal), &
	  & sedpar(c,m,ised_par_ohmarg), &
	  & sedpar(c,m,ised_par_dCO3cal), &
	  & sedpar(c,m,ised_par_dCO3arg), &
	  & sedpar(c,m,ised_par_pH))
      END DO
    END DO

  END SUBROUTINE calc_sed_carbchem
! ********************************************************


! ********************************************************************
! *** couple in sediment sub-system to ocean-atmosphere sub-system ***
! ********************************************************************
  SUBROUTINE couple_sed(R_dt,I_tice,I_tloop,tracer,sedtracer,sedfnp,frefrac,phys, &
    & sed,sedtop,sedpar,sedmixk,dtot,sedfdis,sscc)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::R_dt ! primary model time step as a REAL (in yrs)
    INTEGER,INTENT(in)::I_tice
    INTEGER,INTENT(in)::I_tloop
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_tracer)::tracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_tracer)::sedtracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_fnp)::sedfnp
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::frefrac
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_tot,n_sed_com_par)::sed
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_sed_mix)::sedmixk
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_tracer)::dtot
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedfdis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sscc)::sscc
! local variables
    INTEGER::c,m,o
    INTEGER::it
    INTEGER::n_sed_stack_top          ! sediment stack top (incomplete) layer number
    REAL(KIND=fp_1)::sed_A_cm2        ! sedimentary layer area in cm2
    REAL(KIND=fp_1)::R_tice           ! simulation year as a REAL (kyrs)
    REAL(KIND=fp_1)::R_tloop          ! spin-up year as a REAL (kyrs)
    REAL(KIND=fp_1)::new_sed_vol      ! new sediment volume (as SOLID material)
    REAL(KIND=fp_1)::exe_sed_vol      ! exchangable sediment volume (as SOLID material)
    REAL(KIND=fp_1)::top_sed_vol      ! top sediment volume (as SOLID material)
    REAL(KIND=fp_1)::exe_sed_th       ! exchangable sediment thickness (as stack material)
    REAL(KIND=fp_1)::sed_top_th       ! sediment top ('well-mixed') layer thickness
    REAL(KIND=fp_1)::dsed_top_th      ! change in sediment top ('well-mixed') layer thickness
    REAL(KIND=fp_1)::sed_stack_top_th ! sediment stack top (incomplete) sub-layer thickness
    REAL(KIND=fp_1),DIMENSION(n_lay)::sed_lay_frac            ! sediment/ocean layer fractional adjacencies
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth)::r13C_CO2    ! 13C/C ratio
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth)::r18O_O2     ! 18O/O ratio
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::new_sed         ! new (sedimenting) top layer material
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::dis_sed         ! remineralized top layer material
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::exe_sed         ! top layer material to be exchanged with stack
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_tracer)::sed_fdis ! sediment dissolution flux

! NOTE: the top ('well-mixed') sediment layer is par_sed_top_th cm in thickness,
!       the sediment composition components sum to par_sed_top_th (cm3 cm-2) in this layer, and
!       the porosity is par_sed_poros_top (i.e., solids actually occupy 1.0 - par_sed_poros_top)

! *** initialize local variables ***
! zero 'fsed_dis'
! NOTE: there is no good reason for re-initializing 'fsed_dis' 
! (other than the program refuses to work otherwise!!!)
    sed_fdis = 0.0
! zero sediment arrays
    new_sed(:) = 0.0
    dis_sed(:) = 0.0
    exe_sed(:) = 0.0
! set local constants
    R_tice           = REAL(I_tice,KIND=fp_1)
    R_tloop          = REAL(I_tloop,KIND=fp_1)
! calculate ratios of 13C:C and 18O:O
    r13C_CO2(:,:) = sedtracer(:,:,it_13C_CO2) / sedtracer(:,:,it_CO2)
    r18O_O2(:,:)  = sedtracer(:,:,it_18O_O2)  / sedtracer(:,:,it_O2)

! *** interact ocean boxes and sediment sub-system ***
! NOTE: the overall scheme is to loop through each ocean water column and each sediment layer sub-system, and
!       (a) calculate local constants
!       (b) calculate new sedimenting material to be added to the sediment top ('well mixed') layer, and
!           calculate the thickness of material that this represents
!       (c) estimate dissolution from sediment top layer
!       (d) update sediment top and sediment stack:
!           - add new sedimenting material to sediment top layer
!           - remove remineralized material
!           calculate potential thickness of sediment top layer including new sedimenting material:
!           - remove material to the sediment stack below if the thickness is >= par_sed_top_th cm, or
!           - add material from the sediment stack below if the thickness is < par_sed_top_th cm, then
!           update sediment stack height
!       (e) mix the sediment stack if this is required
!       (f) check the thickness of sediment stack:
!           if it is within 1.0 cm of the maximum thickness, then remove the bottom n_sedcor_tot cm 
!           entirely, and move the remaining sedimentary layers down to start at the bottom of the stack
!       (g) calculate sedimment fluxes to ocean


! *** ocean water column loop ***

    DO c = 1,n_col

! sediment depth level loop
      DO m = 1,n_sed_depth

!print*,"(a)"
! ***** (a) calculate local constants
! calculate sediment / ocean layer fractional connectivity
	sed_lay_frac(:) = calc_sed_lay_frac(sedpar(c,m,:),phys(c,:,:))
! calculate local loop constants
	sed_A_cm2       = sedpar(c,m,ised_par_A) * conv_m2_cm2

!print*,"(b)"
! ***** (b) calculate new sedimenting material to be added to the sediment top layer
!           NOTE: sedimentary material is represented as SOILD matter (i.e., zero porosity)
!           NOTE: convert new particulate matter added to sediments (in mol yr-1) to (cm3 cm-2 yr-1)
!           NOTE: sedimenting FeO is assumed to be made up of Fe in POM, Fe scavenged by falling POM, 
!                 and the fraction of Fe in terrigeneous dust which didn't dissolved in the surface ocean
!           NOTE: convert refractory (terregeneous dust) flux from (g cm-2 yr-1) to (cm3 cm-2 yr-1)
!           NOTE: the additional refractory flux must be converted from (g cm-1 kyr-1) to (cm3 cm-2 yr-1)
!           NOTE: the initial fraction of Fe in the terrigeneous dust flux is first deducted from 
!                 the total flux, to give the refractory flux
!           calculate 'solid' components
	new_sed(ised_Corg)       = sedfnp(c,m,ised_fnp_Corg)     * conv_Corg_mol_cm3  / sed_A_cm2
	new_sed(ised_FeO)        = sedfnp(c,m,ised_fnp_Feorg)    * conv_FeO_mol_cm3   / sed_A_cm2
	new_sed(ised_cal)        = sedfnp(c,m,ised_fnp_cal)      * conv_cal_mol_cm3   / sed_A_cm2
	new_sed(ised_arg)        = sedfnp(c,m,ised_fnp_arg)      * conv_arg_mol_cm3   / sed_A_cm2
	new_sed(ised_opal)       = sedfnp(c,m,ised_fnp_opal)     * conv_opal_mol_cm3  / sed_A_cm2
	new_sed(ised_refrac)     = (frefrac(c) + par_frefrac)      * conv_refrac_g_cm3
!           calculate new sediment (bulk material) stable isotopes
	new_sed(ised_13C_Corg)   = sedfnp(c,m,ised_fnp_13C_Corg) * conv_Corg_mol_cm3  / sed_A_cm2
	new_sed(ised_13C_cal)    = sedfnp(c,m,ised_fnp_13C_cal)  * conv_cal_mol_cm3   / sed_A_cm2
	new_sed(ised_13C_arg)    = sedfnp(c,m,ised_fnp_13C_arg)  * conv_arg_mol_cm3   / sed_A_cm2
!           calculate new sediment (planktonic proxy) stable isotopes
!           NOTE: foraminiferal calcite d18O is assumed to simply follow the oceanic d18O signal
	new_sed(ised_18O_calfp) = new_sed(ised_cal) * &
	  & calc_frac_18O(calc_d18O(tracer(c,1,it_O2),tracer(c,1,it_18O_O2)))
	SELECT CASE (bioopt(bopt_d13CaCO3_fp))
	CASE ("1")
!           scheme #1 : after Mook et al. [1986]
!           NOTE: the Mook [1986] fractionation is approximated by assuming that 
!                 the d13C of HCO3- is the same as that of total DIC 
!                 (this is in order to be consistent with the calculation of benthic fractionation where 
!                 the fractionation within the aqueous carbonate system is not explicitly calculated)
	  new_sed(ised_13C_calfp) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(tracer(c,1,it_CO2),tracer(c,1,it_13C_CO2)) + &
	    & 15.10 - 4232.0 / tracer(c,1,it_T))
	CASE ("2")
!           scheme #2 : after Spero et al. [1997]
!           NOTE: G. bulloides relationship is taken to be the average of the 12th and 13th chamber 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfp) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(tracer(c,1,it_CO2),tracer(c,1,it_13C_CO2)) + &
	    & -0.135 - 13000.0 * sscc(c,isscc_cCO3))
	CASE ("3")
!           scheme #3 : after Spero et al. [1997]
!           NOTE: O. universa relationship is taken to be the average of the dark and high-light 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfp) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(tracer(c,1,it_CO2),tracer(c,1,it_13C_CO2)) + &
	    & +1.795 -  6000.0 * sscc(c,isscc_cCO3))
	CASE default
	  PRINT*,"FATAL ERROR [sue_sed.f90, couple_sed]:"
	  PRINT*,"option 'bioopt(bopt_d13CaCO3_fp)' is out of range;"
	  PRINT*,"selected number = ",bioopt(bopt_d13CaCO3_fp)
	  PRINT*,""
	  STOP
	END SELECT
!           calculate new sediment (benthic proxy) stable isotopes
!           NOTE: foraminiferal calcite d18O is assumed to simply follow the oceanic d18O signal
	new_sed(ised_18O_calfb) = new_sed(ised_cal) * &
	  & calc_frac_18O(calc_d18O(sedtracer(c,m,it_O2),sedtracer(c,m,it_18O_O2)))
	SELECT CASE (bioopt(bopt_d13CaCO3_fb))
	CASE ("1")
!           scheme #1 : after Mook et al. [1986]
!           NOTE: the Mook [1986] fractionation is approximated by assuming that 
!                 the d13C of HCO3- is the same as that of total DIC 
!                 (this is in order to be consistent with the calculation of benthic fractionation where 
!                 the fractionation within the aqueous carbonate system is not explicitly calculated)
	  new_sed(ised_13C_calfb) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(sedtracer(c,m,it_CO2),sedtracer(c,m,it_13C_CO2)) + &
	    & 15.10 - 4232.0 / sedpar(c,m,ised_par_T))
	CASE ("2")
!           scheme #2 : after Spero et al. [1997]
!           NOTE: G. bulloides relationship is taken to be the average of the 12th and 13th chamber 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfb) = new_sed(ised_cal) * calc_frac_13C(&
	    & calc_d13C(sedtracer(c,m,it_CO2),sedtracer(c,m,it_13C_CO2)) + &
	    & -0.135 - 13000.0 * sedpar(c,m,ised_par_CO3))
	CASE ("3")
!           scheme #3 : after Spero et al. [1997]
!           NOTE: O. universa relationship is taken to be the average of the dark and high-light 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfb) = new_sed(ised_cal) * calc_frac_13C(&
	    & calc_d13C(sedtracer(c,m,it_CO2),sedtracer(c,m,it_13C_CO2)) + &
	    & +1.795 -  6000.0 * sedpar(c,m,ised_par_CO3))
	CASE default
	  PRINT*,"FATAL ERROR [sue_sed.f90, couple_sed]:"
	  PRINT*,"option 'bioopt(bopt_d13CaCO3_fp)' is out of range;"
	  PRINT*,"selected number = ",bioopt(bopt_d13CaCO3_fp)
	  PRINT*,""
	  STOP
	END SELECT
!           calculate new sediment radioisotopes
!           NOTE: account for the initial 14C age of sedimenting material,
!                 in addition to the inital 14C/12C ratio
	new_sed(ised_14C_cal) = const_fracdecay_14C_ip * standard_14C_12C * new_sed(ised_cal)
!           calculate new sediment ages
	new_sed(ised_age_cal) = (R_tice + (1000.0 - R_tloop) / 1000.0) * new_sed(ised_cal)
!           determine whether the ash tracer should be added
!           NOTE: it is assumed that the ash occupies a negliable volume in the sediments
	IF ((I_tice == (t_ice_init - 1)) .AND. (I_tloop == 1)) THEN
	  new_sed(ised_ash)   = 1.0
	ELSE
	  new_sed(ised_ash)   = 0.0
	ENDIF
!           calculate volume of added material (as SOILD matter. i.e., zero porosity), in units of cm3 (cm-2)
!           NOTE: Porg, Norg, and Feorg, have only a 'virtual volume' and so are not counted
!                 this goes for the isotope and pH components as well
	new_sed_vol =  new_sed(ised_cal) + new_sed(ised_arg) + new_sed(ised_opal) + &
	  & new_sed(ised_refrac) + new_sed(ised_FeO)
!           check that the thickness of new sedimenting material is >= 0.0 cm yr-1
	IF (new_sed_vol < 0.0) THEN
	  PRINT*,"FATAL ERROR [sue_sed.f90, couple_sed]:"
	  PRINT*,"accumulating sediment < 0.0 cm3 yr-1"
	  PRINT*,"c,m,new_sed_th : ",c,m,new_sed_vol
	  PRINT*,"new_sed(ised_cal)     = ",new_sed(ised_cal)
	  PRINT*,"new_sed(ised_arg)     = ",new_sed(ised_arg)
	  PRINT*,"new_sed(ised_opal)    = ",new_sed(ised_opal)
	  PRINT*,"new_sed(ised_refrac)  = ",new_sed(ised_refrac)
	  PRINT*,"new_sed(ised_FeO)     = ",new_sed(ised_FeO)
	  PRINT*,"new_sed(ised_Corg)    = ",new_sed(ised_Corg)
	  PRINT*,""
	  STOP
	ENDIF
!           check that the thickness of new sedimenting material is < 1.0 cm yr-1
!           NOTE: this limit is imposed because of the way in which excess sedimentary material 
!                 is removed from the top layer and added to the sediment stack, 
!                 which has layers of thickness 1.0 cm
!           NOTE: take into account the porosity of the sediment stack layers
	IF ((new_sed_vol / (1.0 - par_sed_poros)) >= 1.0) THEN
	  PRINT*,"accumulating sediment thickness >= 1.0 cm yr-1"
	  PRINT*,"c,m,new_sed_th : ",c,m,(new_sed_vol / (1.0 - par_sed_poros))
	  PRINT*,"new_sed(ised_cal) th     = ",new_sed(ised_cal)    / (1.0 - par_sed_poros)
	  PRINT*,"new_sed(ised_arg) th     = ",new_sed(ised_arg)    / (1.0 - par_sed_poros)
	  PRINT*,"new_sed(ised_opal) th    = ",new_sed(ised_opal)   / (1.0 - par_sed_poros)
	  PRINT*,"new_sed(ised_refrac) th  = ",new_sed(ised_refrac) / (1.0 - par_sed_poros)
	  PRINT*,"new_sed(ised_FeO) th     = ",new_sed(ised_FeO)    / (1.0 - par_sed_poros)
	  PRINT*,"new_sed(ised_Corg) th    = ",new_sed(ised_Corg)   / (1.0 - par_sed_poros)
	  PRINT*,""
	  STOP
	ENDIF

!print*,"(c)"
! ***** (c) estimate dissolution/remineralization from sediment top ('well mixed') sedimentary layer
!           NOTE: sedimentary material is represented as SOILD matter (i.e., zero porosity)
!           NOTE: return all PO4 to the ocean to avoid having to balance the global PO4 budget
!           NOTE: return all NO3- to the ocean to avoid having to balance the global alkalinity budget
!           NOTE: retain all Feorg in the sediments
!           NOTE: allow no dissolution of refractory or ash tracer material
!           NOTE: remove isotopic components in proportion to their isotopic fraction in the sediment,
!                 but add small constant value to the demoninator to ensure that no divide-by-zero can occur
!           NOTE: because a sedimentary layer may be connected to two (or more) different ocean layers,
!                 the weighted average calcite and aragonite saturation depths must be used 
!                 in the estimation of interface calcite and aragonite dissolution
!           NOTE: when calculating the dissolution fluxes derived from top sediment Corg, cal, and arg 
!                 components, add a threshold factor to the denominator to ensure that 
!                 'divide-by-zero' problems are avoided
!           NOTE: the remineralization of Corg (and associated stable isotopes) 
!                 are not calculated at this stage
!           calculate Corg remineralization
!           NOTE: the Corg fluxes are left in units of (mol cm-2 yr)
!           NOTE: remineralize all Corg and 13Corg,
!                 so that the Corg and 13Corg budgets over long timescales balance 
!                 without the need for additional inputs to the ocean/atmosphere from external sources
!           NOTE: 'create' an extra fraction of Corg and 13Corg and add it to the new sedimentary material,
!                 so that changes in the Corg rain rate and in d13C can be kept track of
!           NOTE: do not bother to update the value of 'new_sed_vol', as it is not used again, 
!                 and its value will cannot go over 1.0 cm total from the creating of 0.1% new Corg volume
!           NOTE: the precise value of this fraction is unimportant, but should be small enough so that 
!                 Corg never attains a signficiant wt% of the sediment
!           NOTE: Corg remineralization must be calculated after the above check for sediment layers 
!                 being wholly or partly within the ocean mixed layer, as otherwise Corg tends 
!                 to accumulate in near-surface sediments, causing a gradual draw-down of C from the ocean
	dis_sed(ised_Corg)     = (1.0 - par_sed_presfrac_Corg) * new_sed(ised_Corg)
	dis_sed(ised_13C_Corg) = (1.0 - par_sed_presfrac_Corg) * new_sed(ised_13C_Corg)
!           calculate CaCO3 diagenesis
	IF (sedopt(sopt_sed_diagen_lookup_CaCO3) == yes) THEN
	  CALL calc_sed_dis_CaCO3_lookup(dis_sed(:),new_sed(:),sedtop(c,m,:),sedpar(c,m,:),sedtracer(c,m,:))
	ELSE
	  dis_sed(ised_cal)       = new_sed(ised_cal)
	  dis_sed(ised_arg)       = new_sed(ised_arg)
	  dis_sed(ised_13C_cal)   = new_sed(ised_13C_cal)
	  dis_sed(ised_13C_calfp) = new_sed(ised_13C_calfp)
	  dis_sed(ised_13C_calfb) = new_sed(ised_13C_calfb)
	  dis_sed(ised_18O_calfp) = new_sed(ised_18O_calfp)
	  dis_sed(ised_18O_calfb) = new_sed(ised_18O_calfb)
	  dis_sed(ised_13C_arg)   = new_sed(ised_13C_arg)
	ENDIF
!           calculate opal diagenesis
	IF (sedopt(sopt_sed_diagen_lookup_opal) == yes) THEN
	  CALL calc_sed_dis_opal_lookup(dis_sed(:),new_sed(:),sedtop(c,m,:),sedpar(c,m,:),sedtracer(c,m,:))
	ELSE
	  dis_sed(ised_opal) = new_sed(ised_opal)
	ENDIF
!           calculate diagenesis of scavenged Fe and Fe in POM
	dis_sed(ised_FeO) = (1.0 - par_sed_presfrac_FeO) * new_sed(ised_FeO)
!           set dissolution of refractory components to zero
	dis_sed(ised_refrac)   = 0.0
	dis_sed(ised_ash)      = 0.0
!           check whether the sediment layer is wholly or partly within the mixed ocean layer:
!           (1) if the sediment layer falls wholly within the mixed ocean layer set dissolution fluxes = 0.0
!           (2) if the sediment layer falls partly within the mixed ocean layer, 
!               set the dissolution flux proportional to the sediment area fraction lying below the 
!               base of the mixed layer and reset the sediment / ocean layer fractional connectivity,
!               so that there is no dissolution flux into the mixed ocean layer - 
!               re-normalize the remaining connectivity
!           NOTE: for sediment layers partially connecting with the mixed ocean layer,
!                 ensure that sediment fluxes intended to be virtually completely remineralized, are so
	IF (sedpar(c,m,ised_par_D_top) < phys(c,1,ip_D_bot)) THEN
	  IF (sedpar(c,m,ised_par_D_bot) <= phys(c,1,ip_D_bot)) THEN
	    dis_sed(:) = 0.0
	  ELSE
	    dis_sed(:) = dis_sed(:) * (sedpar(c,m,ised_par_D_bot) - phys(c,1,ip_D_bot)) / &
	      & (sedpar(c,m,ised_par_D_bot) - sedpar(c,m,ised_par_D_top))
	    sed_lay_frac(1) = 0.0
	    sed_lay_frac(:) = sed_lay_frac(:) / SUM(sed_lay_frac(:))
	  ENDIF
	ENDIF

!print*,"(d)"
! ***** (d) update sediment stack
!           add the new sediment to the top sediment layer,  and deduct the calculated dissolved material
!           NOTE: all sediment quantities are in volume (cm3) of solid material
	sedtop(c,m,:) = sedtop(c,m,:) + new_sed(:) - dis_sed(:)
!           calculate temporary (i.e., beofre exchange with underlying sediments) top sediment composition
	top_sed_vol = sedtop(c,m,ised_cal) + sedtop(c,m,ised_arg) + sedtop(c,m,ised_opal) + &
	  & sedtop(c,m,ised_refrac) + sedtop(c,m,ised_FeO) + sedtop(c,m,ised_Corg)
!           calculate the volume of material to be exchanged between the sediment stack and the top layer,
!           together with the equivalent sediment stack thicckness that the represents
!           NOTE: sedimentary material volume is as SOILD matter (i.e., zero porosity)
!           NOTE: use the ABSOLUTE value
!           NOTE: the thickness of sediment material to be exchanged between the top sediment and 
!                 sediment stack, is expressed using the sediment stack porosity
!           NOTE: the volume to be exchanged is calculated in this way, 
!                 using the difference between the updated (temporary) top sediment volume and the pre-
!                 determined volume (based on the defined thickness of the top layer and it's porosity), 
!                 rather than by the difference between new and dis volume, 
!                 to ensure that the top sediment thickness remains exactly as orignially set 
!                 (otherwise it tends to drift down very slightly)
	exe_sed_vol = ABS(top_sed_vol - ((1.0 - par_sed_poros_top) * par_sed_top_th))
	exe_sed_th  = ABS(top_sed_vol - ((1.0 - par_sed_poros_top) * par_sed_top_th)) / &
	  & (1.0 - par_sed_poros)
!           calculate potential change in thickness of sediment top layer including new sedimenting material
!           NOTE: take into account the top layer porosity
!           NOTE: retain sign
	dsed_top_th = (top_sed_vol - ((1.0 - par_sed_poros_top) * par_sed_top_th)) &
	  & / (1.0 - par_sed_poros_top)
!           calculate sub-layer number and thickness of top (incomplete) sub-layer of sediment stack
	n_sed_stack_top  = INT(sedpar(c,m,ised_par_top)) + 1
	sed_stack_top_th = sedpar(c,m,ised_par_top) - REAL((n_sed_stack_top - 1),KIND=fp_1)
!           keep thickness of top layer = par_sed_top_th by transfer to/from sediment stack,
!           (calculating a volume of sediment to be exchanged exchanged (exe_sed(:)):
!           - remove material to the sediment stack below if dsed_top_th >= 0.0 cm, or
!           - add material from the sediment stack below if dsed_top_th < 0.0 cm
!           NOTE: if dsed_top_th is zero then skip this section to potential 'avoid divide-by-zero' problems
	IF (dsed_top_th == 0.0) THEN
!          do nothing
          exe_sed(:) = 0.0
	ELSE
	  IF (dsed_top_th > 0.0) THEN
!           add material to the sediment stack
!           test thickness of top (incomplete) sub-layer compared with thickness of material to be added:
!           - if exchange th <= remaining (unfilled) thickness of top sub-layer of sediment stack, then
!             add required material to top sub-layer only 
!           - if exchange th > remaining (unfilled) thickness of top sub-layer of sediment stack, then
!             add sufficient material to top sub-layer to fill it plus additional material to next layer up
!           NOTE: the porosity of the sediment stack must be taken into account
	    IF (exe_sed_th <= (1.0 - sed_stack_top_th)) THEN
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter)
!           NOTE: the sediment material composition in 'sedtop' must be unit normalized,
!                 to give a compositional fraction
	      exe_sed(:) = exe_sed_vol * &
		& (sedtop(c,m,:) / par_sed_top_th) * (1.0 / (1.0 - par_sed_poros_top))
!           update sediment stack
	      sed(c,m,n_sed_stack_top,:) = sed(c,m,n_sed_stack_top,:) + exe_sed(:)
	    ELSE
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter)
!           NOTE: the sediment material composition in 'sedtop' must be unit normalized,
!                 to give a compositional fraction
	      exe_sed(:) = exe_sed_vol * &
		& (sedtop(c,m,:) / par_sed_top_th) * (1.0 / (1.0 - par_sed_poros_top))
!           update sediment stack
!           NOTE: add material to the top (incomplete) sediment stack sub-layer in proportion to 
!                 the fraction of unfilled thickness over the equivalent thickness of total material to add
!                 add the remaining material to the next (completely unfilled) sub-layer up
	      sed(c,m,n_sed_stack_top,:) = sed(c,m,n_sed_stack_top,:) + &
		& ((1.0 - sed_stack_top_th) / exe_sed_th) * exe_sed(:)
	      sed(c,m,(n_sed_stack_top + 1),:) = sed(c,m,(n_sed_stack_top + 1),:) + &
		& (1.0 - ((1.0 - sed_stack_top_th) / exe_sed_th)) * exe_sed(:)
	    ENDIF
!           deduct exchange sediment material from the sediment top
	    sedtop(c,m,:) = sedtop(c,m,:) - exe_sed(:)
!           update sediment height and top sub-layer number
	    sedpar(c,m,ised_par_top) = sedpar(c,m,ised_par_top) + exe_sed_th
	    n_sed_stack_top = INT(sedpar(c,m,ised_par_top)) + 1
	  ELSE
!           remove material from the sediment stack
!           test thickness of top (incomplete) sub-layer compared with thickness of material to be removed:
!           - if exchange vol <= thickness of top sub-layer of sediment stack, then
!             remove required material from the top stack sub-layer only 
!           - if exchange vol > thickness of top sub-layer of sediment stack, then
!             remove all material in the top stack sub-layer, plus additional material from next layer down
!           NOTE: the porosity of the sediment stack must be taken into account
	    IF (exe_sed_th <= sed_stack_top_th) THEN
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter)
!           NOTE: the composition of the sediment in the top stack sub-layer must be unit normalized,
	      exe_sed(:) = exe_sed_vol * &
		& (sed(c,m,n_sed_stack_top,:) / sed_stack_top_th) * (1.0 / (1.0 - par_sed_poros))
!           update sediment stack
	      sed(c,m,n_sed_stack_top,:) = sed(c,m,n_sed_stack_top,:) - exe_sed(:)
	    ELSE
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter),
!           equal to ALL the material in the top (incomplete) sediment stack sub-layer,
!           plus a proportion of the material in the sub-layer immediately below
!           update sediment stack at the same time
	      exe_sed(:) = ((exe_sed_th - sed_stack_top_th) / exe_sed_th) * exe_sed_vol * &
		& sed(c,m,(n_sed_stack_top - 1),:) * (1.0 / (1.0 - par_sed_poros))
	      sed(c,m,(n_sed_stack_top - 1),:) = sed(c,m,(n_sed_stack_top - 1),:) - exe_sed(:)
	      exe_sed(:) = exe_sed(:) + sed(c,m,n_sed_stack_top,:)
	      sed(c,m,n_sed_stack_top,:) = 0.0
	    ENDIF
!           add exchange sediment material to the sediment top
	    sedtop(c,m,:) = sedtop(c,m,:) + exe_sed(:)
!           update sediment height and top sub-layer number
	    sedpar(c,m,ised_par_top) = sedpar(c,m,ised_par_top) - exe_sed_th
	    n_sed_stack_top = INT(sedpar(c,m,ised_par_top)) + 1
	  ENDIF
	ENDIF

! ***** DEBUG
	IF (runopt(ropt_debug_sed) == yes) THEN
	  PRINT*,"---"
	  PRINT*,R_tice,R_tloop,(R_tice + (1000.0 - R_tloop) / 1000.0)
	  PRINT*,c,m
	  PRINT*,"new_sed(:)"
	  PRINT*,new_sed(:)
	  PRINT*,"dis_sed(:)"
	  PRINT*,dis_sed(:)
	  PRINT*,"sedtop(c,m,:)"
	  PRINT*,sedtop(c,m,:)
	  PRINT*,"new sed age = ",new_sed(ised_age_cal) / (new_sed(ised_cal) + 1.0E-14)
	  PRINT*,"new sed thickness = ",new_sed_vol / (1.0 - par_sed_poros_top)
	  PRINT*,"actual sedtop thickness = ",(sedtop(c,m,ised_cal) + sedtop(c,m,ised_arg) + &
	    & sedtop(c,m,ised_opal) + sedtop(c,m,ised_FeO) + sedtop(c,m,ised_refrac) + &
	    & sedtop(c,m,ised_Corg)) / (1.0 - par_sed_poros_top)
	  PRINT*,"n_sed_stack_top  = ",INT(sedpar(c,m,ised_par_top)) + 1
	  PRINT*,"sed_stack_top_th = ",sedpar(c,m,ised_par_top) - &
	    & REAL(((INT(sedpar(c,m,ised_par_top)) + 1) - 1),KIND=fp_1)
	  PRINT*,"======="
	  IF ((MINVAL(new_sed(:)) < 0.0) .OR. (MINVAL(dis_sed(:)) < 0.0)) STOP
	ENDIF

!print*,"(e)"
! ***** (e) mix the sediment stack
	IF (sedopt(sopt_sed_mix) == yes) THEN
	  CALL sed_mix(n_sed_com_min, &
	    & sed(c,m,(n_sed_stack_top - n_sed_mix):n_sed_stack_top,1:n_sed_com_min), &
	    & sedtop(c,m,1:n_sed_com_min), &
	    & sedpar(c,m,:),sedmixk(:), &
	    & par_sed_top_th,sed_stack_top_th,par_sed_poros,par_sed_poros_top)
	ENDIF

!print*,"(f)"
! ***** (f) check the thickness of sediment stack
!           NOTE: if the sediment stack has reached the last available sub-layer, 
!                 then remove a number of sub-layer equal to 'n_sed_tot_init' from the bottom, 
!                 and shift the remaining sublayer down
	IF (n_sed_stack_top == n_sed_tot) THEN
!           shift sediment down the stack
	  sed(c,m,1:(n_sed_tot - n_sed_tot_init),:) = sed(c,m,(n_sed_tot_init + 1):n_sed_tot,:)
	  sed(c,m,(n_sed_tot - n_sed_tot_init + 1):n_sed_tot,:) = 0.0
!           update sediment height and top sub-layer number
	  sedpar(c,m,ised_par_top) = sedpar(c,m,ised_par_top) - REAL(n_sed_tot_init,KIND=fp_1)
	  n_sed_stack_top = INT(sedpar(c,m,ised_par_top)) + 1
	ENDIF

!print*,"(g)"
! ***** (g) calculate dissolved sedimment return tracer fluxes to the ocean
!           NOTE: partition fluxes between ocean layer when a sediment layer adjoins more than 1 ocean layer
!           NOTE: take into account respitation of Corg when calculateing CO2 fluxes
!           NOTE: assume that all P and N is remineralized
	sed_fdis(c,:,it_CO2) = sed_fdis(c,:,it_CO2) + &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   conv_cal_cm3_mol * dis_sed(ised_cal) + &
	  &   conv_arg_cm3_mol * dis_sed(ised_arg) + &
	  &   conv_Corg_cm3_mol * dis_sed(ised_Corg) &
	  & )
	sed_fdis(c,:,it_PO4) = sed_fdis(c,:,it_PO4) + &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   red_Corg_Porg * conv_Corg_cm3_mol * dis_sed(ised_Corg) &
	  & )
	sed_fdis(c,:,it_Fe) = sed_fdis(c,:,it_Fe) + &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   conv_FeO_cm3_mol * dis_sed(ised_FeO) &
	  & )
	sed_fdis(c,:,it_Si) = sed_fdis(c,:,it_Si) + &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   conv_opal_cm3_mol * dis_sed(ised_opal) &
	  & )
	sed_fdis(c,:,it_alk) = sed_fdis(c,:,it_alk) + &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   2.0 * conv_cal_cm3_mol * dis_sed(ised_cal) + &
	  &   2.0 * conv_arg_cm3_mol * dis_sed(ised_arg) - &
	  &   red_Corg_Norg * conv_Corg_cm3_mol * dis_sed(ised_Corg) &
	  & )
	sed_fdis(c,:,it_13C_CO2) = sed_fdis(c,:,it_13C_CO2) + &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   conv_cal_cm3_mol * dis_sed(ised_13C_cal) + &
	  &   conv_arg_cm3_mol * dis_sed(ised_13C_arg) + &
	  &   conv_Corg_cm3_mol * dis_sed(ised_13C_Corg) &
	  & )
	sed_fdis(c,:,it_O2) = sed_fdis(c,:,it_O2) - &
	  & sed_lay_frac(:) * sed_A_cm2 * &
	  & ( &
	  &   red_Corg_O2 * conv_Corg_cm3_mol * dis_sed(ised_Corg) &
	  & )

! ***** (h) calculate surface ocean balance for sedimenting POM-related tracer fluxes
	sed_fdis(c,1,it_CO2)     = sed_fdis(c,1,it_CO2) + &
	  & sed_A_cm2 * conv_Corg_cm3_mol * (new_sed(ised_Corg) - dis_sed(ised_Corg))
	sed_fdis(c,1,it_PO4)     = sed_fdis(c,1,it_PO4) + &
	  & sed_A_cm2 * red_Corg_Porg * conv_Corg_cm3_mol * (new_sed(ised_Corg) - dis_sed(ised_Corg))
	sed_fdis(c,1,it_alk)     = sed_fdis(c,1,it_alk) - &
	  & sed_A_cm2 * red_Corg_Norg * conv_Corg_cm3_mol * (new_sed(ised_Corg) - dis_sed(ised_Corg))
	sed_fdis(c,1,it_13C_CO2) = sed_fdis(c,1,it_13C_CO2) + &
	  & sed_A_cm2 * conv_Corg_cm3_mol * (new_sed(ised_13C_Corg) - dis_sed(ised_13C_Corg))
	sed_fdis(c,1,it_O2)      = sed_fdis(c,1,it_O2) - & 
	  & sed_A_cm2 * red_Corg_O2 * conv_Corg_cm3_mol * (new_sed(ised_Corg) - dis_sed(ised_Corg))

! ***** (i) copy dissolution flux values to output array
	sedfdis(c,m,:) = dis_sed(:)

      END DO! (end of sediment layer ('m') loop)
      
    END DO! (end of water column ('c') loop)

! *** add tracers remineralized in sediment layer to respective ocean boxes ***
    DO it = 1,n_tracer
      tracer(:,:,it) = tracer(:,:,it) + sed_fdis(:,:,it) / phys(:,:,ip_M)
    ENDDO

! *** update value of 'tracer' audit total ***
    dtot(it_PO4)     = dtot(it_PO4) &
      & - SUM(sedfnp(:,:,ised_fnp_Porg)) &
      & + SUM(sed_fdis(:,:,it_PO4))
    dtot(it_CO2)     = dtot(it_CO2) &
      & - SUM(sedfnp(:,:,ised_fnp_cal)) &
      & - SUM(sedfnp(:,:,ised_fnp_arg)) &
      & - SUM(sedfnp(:,:,ised_fnp_Corg)) &
      & + SUM(sed_fdis(:,:,it_CO2))
    dtot(it_Fe)      = dtot(it_Fe) &
      & - SUM(sedfnp(:,:,ised_fnp_Feorg)) &
      & + SUM(sed_fdis(:,:,it_Fe))
    dtot(it_Si)      = dtot(it_Si) &
      & - SUM(sedfnp(:,:,ised_fnp_opal)) &
      & + SUM(sed_fdis(:,:,it_Si))
    dtot(it_13C_CO2) = dtot(it_13C_CO2) &
      & - SUM(sedfnp(:,:,ised_fnp_13C_cal)) &
      & - SUM(sedfnp(:,:,ised_fnp_13C_arg)) &
      & - SUM(sedfnp(:,:,ised_fnp_13C_Corg)) &
      & + SUM(sed_fdis(:,:,it_13C_CO2))
    dtot(it_alk)     = dtot(it_alk) &
      & - 2.0 * SUM(sedfnp(:,:,ised_fnp_cal)) &
      & - 2.0 * SUM(sedfnp(:,:,ised_fnp_arg)) &
      & + SUM(sedfnp(:,:,ised_fnp_Norg)) &
      & + SUM(sed_fdis(:,:,it_alk))
    dtot(it_O2)     = dtot(it_O2) &
      & + SUM(red_Corg_O2 * sedfnp(:,:,ised_fnp_Corg)) &
      & + SUM(sed_fdis(:,:,it_O2))
    dtot(it_18O_O2) = dtot(it_18O_O2) &
      & + SUM(r18O_O2(:,:) * red_Corg_O2 * sedfnp(:,:,ised_fnp_Corg)) &
      & + SUM(sed_fdis(:,:,it_18O_O2))

  END SUBROUTINE couple_sed
! ********************************************************************


! *************************************
! *** UPDATE SEDIMENT 14C INVENTORY ***
! *************************************
  SUBROUTINE update_sed_14C(sed,sedtop)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_tot,n_sed_com_par)::sed
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop

! *** decay 14C within sediments ***
! NOTE: it is easier to decay 14C in ALL sedimentary sub-layers, whether they are filled or not,
!       rather than having to calculate the sediment layer top
    sedtop(:,:,ised_14C_cal) = const_fracdecay_14C * sedtop(:,:,ised_14C_cal)
    sed(:,:,:,ised_14C_cal)  = const_fracdecay_14C * sed(:,:,:,ised_14C_cal)

  END SUBROUTINE update_sed_14C
! *************************************


! ************************************************
! *** SAVE INTEGRAL SEDIMENT CORE DATA TO FILE ***
! ************************************************
  SUBROUTINE save_sed_data(sed,sedtop,sedpar)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_tot,n_sed_com_par)::sed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
! local variables
    INTEGER::c,m,o,oo
    INTEGER::out = 1
    INTEGER::n_sed_lim                                             !
    CHARACTER(LEN=255)::filename                                   !
    CHARACTER(LEN=2)::col_num                                      !
    REAL(KIND=fp_1)::sed_age_lim                                   !
    REAL(KIND=fp_1)::sed_tot_wt                                    ! total mass of solid coponents
    REAL(KIND=fp_1)::sed_tot_vol                                   ! total volume of solid coponents
    INTEGER,DIMENSION(n_col,n_sed_depth)::n_sed_stack_top          ! sediment stack top layer number
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth)::sed_stack_top_th ! sediment stack top layer thickness
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::sed_lim              !
    REAL(KIND=fp_1),DIMENSION(n_col)::lysD_cal                     ! estimate of calcite lysocline depth (m)
    REAL(KIND=fp_1),DIMENSION(n_col)::lysD_arg                     ! estimate of aragonite lysocline depth (m)
    INTEGER::ash_max_o                ! running ash volume maximum sub-layer number
    REAL(KIND=fp_1)::ash_max          ! running ash volume maximum
    REAL(KIND=fp_1)::ash_max_depth    ! running ash volume maximum down-core depth
    REAL(KIND=fp_1)::ash_conv_dbs_age ! convert depth to age using ash stratigraphy
    
! *** initialize variables ***
! allocate array for holding sediment data reordered for writing to file
! NOTE: the array bounds extend from ZERO up to 'n_sedtot' 
!       so that core top layer data can be more easily assimilated
    ALLOCATE(sed_save(n_col,n_sed_depth,0:n_sed_tot,n_sed_com_par),STAT=alloc_error) 
    ALLOCATE(sed_save_age_tot(n_col,n_sed_depth,0:n_sed_tot),STAT=alloc_error)      
    ALLOCATE(sed_save_age_ash(n_col,n_sed_depth,0:n_sed_tot),STAT=alloc_error)     
    ALLOCATE(sed_save_age_14C(n_col,n_sed_depth,0:n_sed_tot),STAT=alloc_error)  
    IF (error /= 0) THEN
! space for the sediment arrays could not be allocated
      PRINT*,"FATAL ERROR [sue_sed.f90, save_sed_data]:"
      PRINT*,"Space for the sediment arrays could not be allocated"
      STOP
    ENDIF
! zero local variables
    sed_save(:,:,:,:)       = 0.0
    sed_save_age_tot(:,:,:) = 0.0
    sed_save_age_ash(:,:,:) = 0.0
    sed_save_age_14C(:,:,:) = 0.0
    
! *** transform sediment array for saving to file ***
! NOTE: the sediment array needs to be re-ordered so that the youngest sediment in the sediment stack 
!       starts with an array index of "1",
!       and the sediment top material is added at index position "0"
! NOTE: sediment composition descriptors ired to %calcite, such as age and pH,
!       need to be normailzed to %calcite
! NOTE: the sediment composition descriptors in the top layer sediments 
!       need to be normailzed to a thickness of 1.0 cm
! NOTE: the sediment composition descriptors in the top (incomplete) sub-layer of the sediment stack 
!       need to be normailzed to a thickness of 1.0 cm
! NOTE: the overall scheme is to loop through each sediment layer, and
!       (a) calculate local constants
!       (b) copy sediment core top layer data to data-file export array
!       (c) copy sediment core stack sub-layer data to data-file export array
!       (d) normailze age and pH values to the layer calcite content
!       (e) normalize ash (volume) content to unit (cm) layer thickness
!       (f) adjust core base data set values to improve MATLAB graphability
!       (g) normailze solid sediment components to a wt fraction (rather than solid volume) basis if required

! loop start

    DO c = 1,n_col
      
      DO m = 1,n_sed_depth

!print*,"a"
! ***** (a) calculate local constants
	n_sed_stack_top(c,m)  = INT(sedpar(c,m,ised_par_top)) + 1
	sed_stack_top_th(c,m) = sedpar(c,m,ised_par_top) - REAL((n_sed_stack_top(c,m) - 1),KIND=fp_1)
	
! ***** (b) copy core top layer data
	sed_save(c,m,0,:) = sedtop(c,m,:)
	
! ***** (c) copy core stack sub-layer data
	DO o = n_sed_stack_top(c,m),1,-1
	  sed_save(c,m,(n_sed_stack_top(c,m) - o + 1),:) = sed(c,m,o,:)
	END DO
	
! ***** (d) normailze age values
	DO o = 0,n_sed_tot
	  IF (sed_save(c,m,o,ised_cal) > 0.0) THEN
	    sed_save(c,m,o,ised_age_cal) = sed_save(c,m,o,ised_age_cal) / sed_save(c,m,o,ised_cal)
	  ELSE
	    sed_save(c,m,o,ised_age_cal) = 0.0
	  ENDIF
	ENDDO

! ***** (e) assign a radiocarbon age
!           NOTE: ensure that age is in (kyr)
	DO o = 0,n_sed_tot
	  IF ((sed_save(c,m,o,ised_14C_cal) > 0.0) .AND. (sed_save(c,m,o,ised_cal) > 0.0)) THEN
	    sed_save_age_14C(c,m,o) = -const_lamda_14C * &
	      & LOG(sed_save(c,m,o,ised_14C_cal) / (standard_14C_12C * sed_save(c,m,o,ised_cal)))
	    sed_save_age_14C(c,m,o) = conv_yr_kyr * sed_save_age_14C(c,m,o)
	  ELSE
	    sed_save_age_14C(c,m,o) = 0.0
	  ENDIF
	ENDDO

! ***** (f) normalize ash (volume) content to unit (cm) layer thickness
	sed_save(c,m,0,ised_ash) = sed_save(c,m,0,ised_ash) / par_sed_top_th
	IF (sed_stack_top_th(c,m) > 0.0) THEN
	  sed_save(c,m,1,ised_ash) = sed_save(c,m,1,ised_ash) / sed_stack_top_th(c,m)
	ENDIF
	DO o = 2,n_sed_tot
	  sed_save(c,m,o,ised_ash) = sed_save(c,m,o,ised_ash) / 1.0
	END DO
	
! ***** (g) adjust core base data set values
!           NOTE: this is to improve the MATALB graphibility of the saved data-set
!           loop through all sediment sub-levels:
!           when the first sediment sub-level is found where the calcite age equals zero,
!           reset the assigned sediment ages further down the core
!           NOTE: start from o = 1 rather than 0
	DO o = 1,n_sed_tot
	  IF (sed_save(c,m,o,ised_age_cal) == 0.0) THEN
	    sed_age_lim = sed_save(c,m,o - 1,ised_age_cal)
	    n_sed_lim = o
	    DO oo = n_sed_lim,n_sed_tot
	      sed_save(c,m,oo,ised_age_cal) = sed_age_lim + 0.001 * REAL((oo - n_sed_lim),KIND=fp_1)
	    END DO
	  ENDIF
	END DO
!           loop through all sediment sub-levels:
!           when the first sediment sub-level is found where vol of refractory material equals zero,
!           reset the assigned sediment refractory contents further down the core to 1.0
!           NOTE: this assumes that there should always be some refractory material in the sediments:
!                 if there is none, it must indicate that this is a previously unused sediment sub-layer
	DO o = 0,n_sed_tot
	  IF (sed_save(c,m,o,ised_refrac) == 0.0) THEN
	    n_sed_lim = o
	    DO oo = n_sed_lim,n_sed_tot
	      sed_save(c,m,oo,ised_refrac) = 1.0
	    END DO
	  ENDIF
	END DO

! ***** (h) normailze solid sediment components to a mass fraction basis (if required)
!           NOTE: as a first step, calculate total mass of solid components in the sediment sub-layer
!           NOTE: ensure that the stable isotopes are treated in the same manner
	IF (sedopt(sopt_sed_save_wtper) == yes) THEN
	  DO o = 0,n_sed_tot
	    sed_tot_wt = &
	      & conv_cal_cm3_g    * sed_save(c,m,o,ised_cal)    + &
	      & conv_arg_cm3_g    * sed_save(c,m,o,ised_arg)    + &
	      & conv_opal_cm3_g   * sed_save(c,m,o,ised_opal)   + &
	      & conv_refrac_cm3_g * sed_save(c,m,o,ised_refrac) + &
	      & conv_FeO_cm3_g    * sed_save(c,m,o,ised_FeO)    + &
	      & conv_Corg_cm3_g   * sed_save(c,m,o,ised_Corg)
	    IF (sed_tot_wt > 0.0) THEN
	      sed_save(c,m,o,ised_cal)       = conv_cal_cm3_g    * sed_save(c,m,o,ised_cal)       / sed_tot_wt
	      sed_save(c,m,o,ised_arg)       = conv_arg_cm3_g    * sed_save(c,m,o,ised_arg)       / sed_tot_wt
	      sed_save(c,m,o,ised_opal)      = conv_opal_cm3_g   * sed_save(c,m,o,ised_opal)      / sed_tot_wt
	      sed_save(c,m,o,ised_refrac)    = conv_refrac_cm3_g * sed_save(c,m,o,ised_refrac)    / sed_tot_wt
	      sed_save(c,m,o,ised_FeO)       = conv_FeO_cm3_g    * sed_save(c,m,o,ised_FeO)       / sed_tot_wt
	      sed_save(c,m,o,ised_Corg)      = conv_Corg_cm3_g   * sed_save(c,m,o,ised_Corg)      / sed_tot_wt
	      sed_save(c,m,o,ised_13C_Corg)  = conv_Corg_cm3_g   * sed_save(c,m,o,ised_13C_Corg)  / sed_tot_wt
	      sed_save(c,m,o,ised_13C_cal)   = conv_cal_cm3_g    * sed_save(c,m,o,ised_13C_cal)   / sed_tot_wt
	      sed_save(c,m,o,ised_13C_arg)   = conv_arg_cm3_g    * sed_save(c,m,o,ised_13C_arg)   / sed_tot_wt
	      sed_save(c,m,o,ised_14C_cal)   = conv_cal_cm3_g    * sed_save(c,m,o,ised_14C_cal)   / sed_tot_wt
	      sed_save(c,m,o,ised_13C_calfp) = conv_cal_cm3_g    * sed_save(c,m,o,ised_13C_calfp) / sed_tot_wt
	      sed_save(c,m,o,ised_13C_calfb) = conv_cal_cm3_g    * sed_save(c,m,o,ised_13C_calfb) / sed_tot_wt
	      sed_save(c,m,o,ised_18O_calfp) = conv_cal_cm3_g    * sed_save(c,m,o,ised_18O_calfp) / sed_tot_wt
	      sed_save(c,m,o,ised_18O_calfb) = conv_cal_cm3_g    * sed_save(c,m,o,ised_18O_calfb) / sed_tot_wt
	    ENDIF
	  END DO
	ELSE
	  DO o = 0,n_sed_tot
	    sed_tot_vol = &
	      & sed_save(c,m,o,ised_cal)    + &
	      & sed_save(c,m,o,ised_arg)    + &
	      & sed_save(c,m,o,ised_opal)   + &
	      & sed_save(c,m,o,ised_refrac) + &
	      & sed_save(c,m,o,ised_FeO)    + &
	      & sed_save(c,m,o,ised_Corg)
	    IF (sed_tot_vol > 0.0) THEN
	      sed_save(c,m,o,ised_cal)       = sed_save(c,m,o,ised_cal)       / sed_tot_vol
	      sed_save(c,m,o,ised_arg)       = sed_save(c,m,o,ised_arg)       / sed_tot_vol
	      sed_save(c,m,o,ised_opal)      = sed_save(c,m,o,ised_opal)      / sed_tot_vol
	      sed_save(c,m,o,ised_refrac)    = sed_save(c,m,o,ised_refrac)    / sed_tot_vol
	      sed_save(c,m,o,ised_FeO)       = sed_save(c,m,o,ised_FeO)       / sed_tot_vol
	      sed_save(c,m,o,ised_Corg)      = sed_save(c,m,o,ised_Corg)      / sed_tot_vol
	      sed_save(c,m,o,ised_13C_Corg)  = sed_save(c,m,o,ised_13C_Corg)  / sed_tot_vol
	      sed_save(c,m,o,ised_13C_cal)   = sed_save(c,m,o,ised_13C_cal)   / sed_tot_vol
	      sed_save(c,m,o,ised_13C_arg)   = sed_save(c,m,o,ised_13C_arg)   / sed_tot_vol
	      sed_save(c,m,o,ised_14C_cal)   = sed_save(c,m,o,ised_14C_cal)   / sed_tot_vol
	      sed_save(c,m,o,ised_13C_calfp) = sed_save(c,m,o,ised_13C_calfp) / sed_tot_vol
	      sed_save(c,m,o,ised_13C_calfb) = sed_save(c,m,o,ised_13C_calfb) / sed_tot_vol
	      sed_save(c,m,o,ised_18O_calfp) = sed_save(c,m,o,ised_18O_calfp) / sed_tot_vol
	      sed_save(c,m,o,ised_18O_calfb) = sed_save(c,m,o,ised_18O_calfb) / sed_tot_vol
	    ENDIF
	  END DO
	ENDIF

! ***** (i) produce stratigraphic marker age scale
!           NOTE: this assumes that the maximum ash volume fraction represents the ash impulse deposition age
!                 and that the sediment ages inbetween this depth and the surface
!                 can be linearly interpolated
!           NOTE: sediment deeper then the ash maximum is aged by linear extrapolation
!           NOTE: first, the ash maximum must be found
!           find ash maximum
	ash_max = 0.0
	DO o = 0,n_sed_tot
	  IF (sed_save(c,m,o,ised_ash) > ash_max) THEN
	    ash_max   = sed_save(c,m,o,ised_ash)
	    ash_max_o = o
	  ENDIF
	END DO
!           calculate ash maximum depth
	SELECT CASE (ash_max_o)
	CASE (0)
	  ash_max_depth = par_sed_top_th / 2.0
	CASE (1)
	  ash_max_depth = par_sed_top_th + sed_stack_top_th(c,m) / 2.0
	CASE default
	  ash_max_depth = par_sed_top_th + sed_stack_top_th(c,m) + REAL((ash_max_o - 2),KIND=fp_1) + 0.5
	END SELECT
!           calculate linear age-depth relation
	ash_conv_dbs_age = REAL(t_ice_init,KIND=fp_1) / ash_max_depth
!           generate age scale
	o = 0
	sed_save_age_ash(c,m,o) = ash_conv_dbs_age * &
	  & (par_sed_top_th / 2.0)
	o = 1
	sed_save_age_ash(c,m,o) = ash_conv_dbs_age * &
	  & (par_sed_top_th + sed_stack_top_th(c,m) / 2.0)
	DO o = 2,n_sed_tot
	  sed_save_age_ash(c,m,o) = ash_conv_dbs_age * &
	    & (par_sed_top_th + sed_stack_top_th(c,m) + REAL((o - 2),KIND=fp_1) + 0.5)
	END DO
!           cap age scale
	DO o = 0,n_sed_tot
	  IF (sed_save_age_ash(c,m,o) >= REAL((t_spin_max + t_ice_init),KIND=fp_1)) THEN
	    sed_save_age_ash(c,m,o) = REAL((t_spin_max + t_ice_init),KIND=fp_1) + 0.001 * REAL(o,KIND=fp_1)
	  ENDIF
	END DO

! 'm' loop end
	
      END DO

! 'c' loop end

    END DO

! *** save sediment tracer (z-axis) data ***
! NOTE: format for sediment data saving is:
!       (1) set filename
!       (2) open filepipe
!       (3) write ocean sediment depth interval mid-points as a file header
!       (4) save data in nested loop, first mixed layer then old sediment data
!       (5) close filepipe

! loop start

    DO c = 1,n_col

! set box number string
      col_num = conv_num_char(c)

! DATA: calcite (wt fraction or volume (cm3))
      filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_cal.res"
      OPEN(unit=out,file=filename,action="write")
      DO o = 0,n_sed_tot
	WRITE(unit=out,FMT='(99F8.5)') &
	  & (sed_save(c,m,o,ised_cal),m=1,n_sed_depth)
      END DO
      CLOSE(unit=out)
! DATA: aragonite (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_arg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,o,ised_arg),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: CaCO3 (wt fraction or volume (cm3))
      filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_CaCO3.res"
      OPEN(unit=out,file=filename,action="write")
      DO o = 0,n_sed_tot
	WRITE(unit=out,FMT='(99F8.5)') &
	  & (sed_save(c,m,o,ised_cal) + sed_save(c,m,o,ised_arg),m=1,n_sed_depth)
      END DO
      CLOSE(unit=out)
! DATA: opal (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_opal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,o,ised_opal),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: Corg (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_Corg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,o,ised_Corg),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: iron oxides (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_FeO.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,o,ised_FeO),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: refractory (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_refrac.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,o,ised_refrac),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: ash (volume (cm3))
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_ash.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,o,ised_ash),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d13Corg
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d13C_Corg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,o,ised_Corg),sed_save(c,m,o,ised_13C_Corg)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d13C (planktonic calcite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d13C_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,o,ised_cal),sed_save(c,m,o,ised_13C_cal)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d13C (planktonic aragonite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d13C_arg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,o,ised_arg),sed_save(c,m,o,ised_13C_arg)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d13C (planktonic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d13C_calfp.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,o,ised_cal),sed_save(c,m,o,ised_13C_calfp)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d13C (benthic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d13C_calfb.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,o,ised_cal),sed_save(c,m,o,ised_13C_calfb)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d18O (planktonic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d18O_calfp.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d18O(sed_save(c,m,o,ised_cal),sed_save(c,m,o,ised_18O_calfp)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! DATA: d18O (benthic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_d18O_calfb.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d18O(sed_save(c,m,o,ised_cal),sed_save(c,m,o,ised_18O_calfb)),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)

! *** save sediment x-/y-axis data ***
! NOTE: format for sediment data saving is:
!       (1) set filename
!       (2) open filepipe
!       (3) write ocean sediment depth interval mid-points as a file header
!       (4) save data in nested loop, first mixed layer then old sediment data
!       (5) close filepipe

! X AXIS DATA: sediment age - calcite
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_age_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (sed_save(c,m,o,ised_age_cal),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: sediment age - 14C (planktonic calcite)
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_age_14C.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (sed_save_age_14C(c,m,o),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: sediment age - ash
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_age_ash.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.3)') &
	& (sed_save_age_ash(c,m,o),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: depth in sediment column
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_dbs.res"
    OPEN(unit=out,file=filename,action="write")
    o = 0
    WRITE(unit=out,FMT='(99F8.2)') &
      & ((par_sed_top_th / 2.0),m=1,n_sed_depth)
    o = 1
    WRITE(unit=out,FMT='(99F8.2)') &
      & (par_sed_top_th + (sed_stack_top_th(c,m) / 2.0),m=1,n_sed_depth)
    DO o = 2,n_sed_tot
      WRITE(unit=out,FMT='(99F8.2)') &
	& (par_sed_top_th + sed_stack_top_th(c,m) + REAL((o - 2),KIND=fp_1) + 0.5,m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: sediment sub-layer thickness
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_layth.res"
    OPEN(unit=out,file=filename,action="write")
    o = 0
    WRITE(unit=out,FMT='(99F8.2)') &
      & (par_sed_top_th,m=1,n_sed_depth)
    o = 1
    WRITE(unit=out,FMT='(99F8.2)') &
      & (sed_stack_top_th(c,m),m=1,n_sed_depth)
    DO o = 2,n_sed_tot
      WRITE(unit=out,FMT='(99F8.2)') &
	& (REAL(1,KIND=fp_1),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)
! Y AXIS DATA: sediment layer depth in water column
    filename = modelname//"_output/"//modelname//"_sed"//col_num//"xx_depth.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sed_tot
      WRITE(unit=out,FMT='(99F8.1)') &
	& (sedpar(c,m,ised_par_D_PIG_mid),m=1,n_sed_depth)
    END DO
    CLOSE(unit=out)

! loop end

  END DO


! *** estimate lysocline depths ***
! NOTE: if the threshold is set to zero, don't estimate or save lysocline depth
  IF (par_sed_lys_thresh > 0.0) THEN
! estimate calcite lysocline depths
    DO c = 1,n_col
      IF (sed_save(c,n_sed_depth,0,ised_cal) > 0.000001) THEN
	lysD_cal(c) = 10000.0
      ELSE
	DO m = n_sed_depth,2,-1
	  IF ((sed_save(c,m - 1,0,ised_cal) > 0.000001) .AND. (sed_save(c,m,0,ised_cal) > 0.000001)) THEN
	    IF ((sed_save(c,m - 1,0,ised_cal) / sed_save(c,m,0,ised_cal)) < &
	      & (1.0 + par_sed_lys_thresh / 100.0)) THEN
	      lysD_cal(c) = sedpar(c,m,ised_par_D_bot)
	      EXIT
	    ELSE
	      
	    ENDIF
	  ELSE
	    lysD_cal(c) = 0.0
	  ENDIF
	END DO
      ENDIF
    END DO
! save estimated calcite lysocline depth
    filename = modelname//"_output/"//modelname//"_sedxxxx_lysD_cal.res"
    OPEN(unit=out,file=filename,action="write")
    WRITE(unit=out,FMT='(99F10.3)') (lysD_cal(c),c = 1,n_col)
    CLOSE(unit=out)
! estimate aragonite lysocline depths
    DO c = 1,n_col
      IF (sed_save(c,n_sed_depth,0,ised_arg) > 0.000001) THEN
	lysD_arg(c) = 10000.0
      ELSE
	DO m = n_sed_depth,2,-1
	  IF ((sed_save(c,m - 1,0,ised_arg) > 0.000001) .AND. (sed_save(c,m,0,ised_arg) > 0.000001)) THEN
	    IF ((sed_save(c,m - 1,0,ised_arg) / sed_save(c,m,0,ised_arg)) < &
	      & (1.0 + par_sed_lys_thresh / 100.0)) THEN
	      lysD_arg(c) = sedpar(c,m,ised_par_D_bot)
	      EXIT
	    ELSE
	      
	    ENDIF
	  ELSE
	    lysD_arg(c) = 0.0
	  ENDIF
	END DO
      ENDIF
    END DO
! save estimated aragonite lysocline depth (if requested)
    filename = modelname//"_output/"//modelname//"_sedxxxx_lysD_arg.res"
    OPEN(unit=out,file=filename,action="write")
    WRITE(unit=out,FMT='(99F10.3)') (lysD_arg(c),c = 1,n_col)
    CLOSE(unit=out)
  END IF

! *** clean up ***
! deallocate local arrays
    DEALLOCATE(sed_save,STAT=dealloc_error)
    DEALLOCATE(sed_save_age_tot,STAT=dealloc_error)

! *** check for problems de-allocating array space ***
    IF (dealloc_error /= 0) THEN
      PRINT*,"FATAL ERROR [sue_sed.f90, save_sed_data]:"
      PRINT*,"Array space could not be de-allocated"
      STOP
    ENDIF
    
  END SUBROUTINE save_sed_data
! ************************************************


! **********************************************
! *** SAVE SEDIMENT SUB-SYSTEM CORE-TOP DATA ***
! **********************************************
  SUBROUTINE save_sed_coretop_data(tsave,sedtop)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::tsave
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop
! local variables
    INTEGER::c,m
    INTEGER::out=1
    CHARACTER(LEN=255)::filename
    CHARACTER(LEN=3)::tsave_string
    REAL(KIND=fp_1)::sed_tot_wt
    REAL(KIND=fp_1)::sed_tot_vol
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sed_save

! *** create filename identifier string ***
    IF (tsave <= 9) THEN
      WRITE(tsave_string(3:3),'(i1)')tsave
      tsave_string(1:2) = "00"
    ELSE IF (tsave <= 99) THEN
      WRITE(tsave_string(2:3),'(i2)')tsave
      tsave_string(1:1) = "0"
    ELSE
      WRITE(tsave_string(1:3),'(i3)')tsave
    ENDIF

! *** initialize local array ***
    sed_save = sedtop
    
! *** normailze solid sediment components to a mass fraction basis (if required)
    DO c = 1,n_col
      DO m = 1,n_sed_depth
	IF (sedopt(sopt_sed_save_wtper) == yes) THEN
	  sed_tot_wt = &
	    & conv_cal_cm3_g    * sed_save(c,m,ised_cal)    + &
	    & conv_arg_cm3_g    * sed_save(c,m,ised_arg)    + &
	    & conv_opal_cm3_g   * sed_save(c,m,ised_opal)   + &
	    & conv_refrac_cm3_g * sed_save(c,m,ised_refrac) + &
	    & conv_FeO_cm3_g    * sed_save(c,m,ised_FeO)    + &
	    & conv_Corg_cm3_g   * sed_save(c,m,ised_Corg)
	  IF (sed_tot_wt > 0.0) THEN
	    sed_save(c,m,ised_cal)       = conv_cal_cm3_g    * sed_save(c,m,ised_cal)       / sed_tot_wt
	    sed_save(c,m,ised_arg)       = conv_arg_cm3_g    * sed_save(c,m,ised_arg)       / sed_tot_wt
	    sed_save(c,m,ised_opal)      = conv_opal_cm3_g   * sed_save(c,m,ised_opal)      / sed_tot_wt
	    sed_save(c,m,ised_refrac)    = conv_refrac_cm3_g * sed_save(c,m,ised_refrac)    / sed_tot_wt
	    sed_save(c,m,ised_FeO)       = conv_FeO_cm3_g    * sed_save(c,m,ised_FeO)       / sed_tot_wt
	    sed_save(c,m,ised_Corg)      = conv_Corg_cm3_g   * sed_save(c,m,ised_Corg)      / sed_tot_wt
	    sed_save(c,m,ised_13C_Corg)  = conv_Corg_cm3_g   * sed_save(c,m,ised_13C_Corg)  / sed_tot_wt
	    sed_save(c,m,ised_13C_cal)   = conv_cal_cm3_g    * sed_save(c,m,ised_13C_cal)   / sed_tot_wt
	    sed_save(c,m,ised_13C_arg)   = conv_arg_cm3_g    * sed_save(c,m,ised_13C_arg)   / sed_tot_wt
	    sed_save(c,m,ised_14C_cal)   = conv_cal_cm3_g    * sed_save(c,m,ised_14C_cal)   / sed_tot_wt
	    sed_save(c,m,ised_13C_calfp) = conv_cal_cm3_g    * sed_save(c,m,ised_13C_calfp) / sed_tot_wt
	    sed_save(c,m,ised_13C_calfb) = conv_cal_cm3_g    * sed_save(c,m,ised_13C_calfb) / sed_tot_wt
	    sed_save(c,m,ised_18O_calfp) = conv_cal_cm3_g    * sed_save(c,m,ised_18O_calfp) / sed_tot_wt
	    sed_save(c,m,ised_18O_calfb) = conv_cal_cm3_g    * sed_save(c,m,ised_18O_calfb) / sed_tot_wt
	  ENDIF
	ELSE
	  sed_tot_vol = &
	    & sed_save(c,m,ised_cal)    + &
	    & sed_save(c,m,ised_arg)    + &
	    & sed_save(c,m,ised_opal)   + &
	    & sed_save(c,m,ised_refrac) + &
	    & sed_save(c,m,ised_FeO)    + &
	    & sed_save(c,m,ised_Corg)
	  IF (sed_tot_vol > 0.0) THEN
	    sed_save(c,m,ised_cal)       = sed_save(c,m,ised_cal)       / sed_tot_vol
	    sed_save(c,m,ised_arg)       = sed_save(c,m,ised_arg)       / sed_tot_vol
	    sed_save(c,m,ised_opal)      = sed_save(c,m,ised_opal)      / sed_tot_vol
	    sed_save(c,m,ised_refrac)    = sed_save(c,m,ised_refrac)    / sed_tot_vol
	    sed_save(c,m,ised_FeO)       = sed_save(c,m,ised_FeO)       / sed_tot_vol
	    sed_save(c,m,ised_Corg)      = sed_save(c,m,ised_Corg)      / sed_tot_vol
	    sed_save(c,m,ised_13C_Corg)  = sed_save(c,m,ised_13C_Corg)  / sed_tot_vol
	    sed_save(c,m,ised_13C_cal)   = sed_save(c,m,ised_13C_cal)   / sed_tot_vol
	    sed_save(c,m,ised_13C_arg)   = sed_save(c,m,ised_13C_arg)   / sed_tot_vol
	    sed_save(c,m,ised_14C_cal)   = sed_save(c,m,ised_14C_cal)   / sed_tot_vol
	    sed_save(c,m,ised_13C_calfp) = sed_save(c,m,ised_13C_calfp) / sed_tot_vol
	    sed_save(c,m,ised_13C_calfb) = sed_save(c,m,ised_13C_calfb) / sed_tot_vol
	    sed_save(c,m,ised_18O_calfp) = sed_save(c,m,ised_18O_calfp) / sed_tot_vol
	    sed_save(c,m,ised_18O_calfb) = sed_save(c,m,ised_18O_calfb) / sed_tot_vol
	  ENDIF
	ENDIF
      END DO
    END DO

! *** save core-top data ***
! calcite (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,ised_cal),c = 1,n_col)
    END DO
    CLOSE(unit=out)
! aragonite (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_arg.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,ised_arg),c = 1,n_col)
    END DO
    CLOSE(unit=out)
! CaCO3 (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_CaCO3.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,ised_cal) + sed_save(c,m,ised_arg),c = 1,n_col)
    END DO
    CLOSE(unit=out)
! opal (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_opal.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,ised_opal),c = 1,n_col)
    END DO
    CLOSE(unit=out)
! refrac (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_refrac.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.5)') &
	& (sed_save(c,m,ised_refrac),c = 1,n_col)
    END DO
    CLOSE(unit=out)
! d13C (planktonic calcite)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_d13C_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,ised_cal),sed_save(c,m,ised_13C_cal)),c=1,n_col)
    END DO
    CLOSE(unit=out)
! d13C (planktonic aragonite)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_d13C_arg.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,ised_arg),sed_save(c,m,ised_13C_arg)),c=1,n_col)
    END DO
    CLOSE(unit=out)
! d13C (planktonic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_d13C_calfp.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.3)') &
      & (calc_d13C(sed_save(c,m,ised_cal),sed_save(c,m,ised_13C_calfp)),c=1,n_col)
    END DO
    CLOSE(unit=out)
! d13C (benthic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_d13C_calfb.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d13C(sed_save(c,m,ised_cal),sed_save(c,m,ised_13C_calfb)),c=1,n_col)
    END DO
    CLOSE(unit=out)
! d18O (planktonic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_d18O_calfp.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d18O(sed_save(c,m,ised_cal),sed_save(c,m,ised_18O_calfp)),c=1,n_col)
    END DO
    CLOSE(unit=out)
! d18O (benthic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_sedxxxx_d18O_calfb.res"
    OPEN(unit=out,file=filename,action="write")
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F8.3)') &
	& (calc_d18O(sed_save(c,m,ised_cal),sed_save(c,m,ised_18O_calfb)),c=1,n_col)
    END DO
    CLOSE(unit=out)

  END SUBROUTINE save_sed_coretop_data
! **********************************************


! **************************************************************
! *** SAVE SEDIMENT SUB-SYSTEM WATER-SEDIMENT RAIN FLUX DATA ***
! **************************************************************
  SUBROUTINE save_sed_fnp(tsave,sedfnp,sedpar)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::tsave
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_fnp)::sedfnp
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
! local variables
    INTEGER::c,m
    INTEGER::out=1
    CHARACTER(LEN=255)::filename
    CHARACTER(LEN=3)::tsave_string
    
! *** create filename identifier string ***
    IF (tsave <= 9) THEN
      WRITE(tsave_string(3:3),'(i1)')tsave
      tsave_string(1:2) = "00"
    ELSE IF (tsave <= 99) THEN
      WRITE(tsave_string(2:3),'(i2)')tsave
      tsave_string(1:1) = "0"
    ELSE
      WRITE(tsave_string(1:3),'(i3)')tsave
    ENDIF

! *** save sediment sub-system depth data ***
! NOTE: units : (m)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_depth.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedpar(c,m,ised_par_D_mid)) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save calcite rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_cal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_cal) * conv_mol_umol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save aragonite rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_arg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_arg) * conv_mol_umol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save CaCO3 rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_CaCO3.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_cal) * conv_mol_umol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) + &
	& (sedfnp(c,m,ised_fnp_arg) * conv_mol_umol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save Corg rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_Corg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_Corg) * conv_mol_umol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save opal rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_opal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_opal) * conv_mol_umol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save Feorg rain rate ***
! NOTE: units : (nmol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_Feorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_Feorg) * conv_mol_nmol) / (sedpar(c,m,ised_par_A) * conv_m2_cm2) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save CaCO3/Corg rain ratio data ***
! NOTE: units : (dimensionless)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_CaCO3toCorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfnp(c,m,ised_fnp_cal) + sedfnp(c,m,ised_fnp_arg)) / (0.000001 + sedfnp(c,m,ised_fnp_Corg)) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save cal/Corg rain ratio data ***
! NOTE: units : (dimensionless)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_caltoCorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& sedfnp(c,m,ised_fnp_cal) / (0.000001 + sedfnp(c,m,ised_fnp_Corg)) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save arg/Corg rain ratio data ***
! NOTE: units : (dimensionless)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_argtoCorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& sedfnp(c,m,ised_fnp_arg) / (0.000001 + sedfnp(c,m,ised_fnp_Corg)) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save Corg remineralization data ***
! NOTE: in units of fraction of np remaining
    filename =modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_remin_Corg.res" 
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.6)') (sedpar(c,m,ised_par_reminf_Corg),c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save opal remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_remin_opal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.6)') (sedpar(c,m,ised_par_reminf_opal),c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save cal remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_remin_cal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.6)') (sedpar(c,m,ised_par_reminf_cal),c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save arg remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedxxxx_remin_arg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.6)') (sedpar(c,m,ised_par_reminf_arg),c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

  END SUBROUTINE save_sed_fnp
! **************************************************************


! *********************************************************************
! *** SAVE SEDIMENT SUB-SYSTEM WATER-SEDIMENT DISSOLUTION FLUX DATA ***
! *********************************************************************
  SUBROUTINE save_sed_fdis(tsave,sedfdis,sedpar)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::tsave
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedfdis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
! local variables
    INTEGER::c,m
    INTEGER::out=1
    CHARACTER(LEN=255)::filename
    CHARACTER(LEN=3)::tsave_string

! *** create filename identifier string ***
    IF (tsave <= 9) THEN
      WRITE(tsave_string(3:3),'(i1)')tsave
      tsave_string(1:2) = "00"
    ELSE IF (tsave <= 99) THEN
      WRITE(tsave_string(2:3),'(i2)')tsave
      tsave_string(1:1) = "0"
    ELSE
      WRITE(tsave_string(1:3),'(i3)')tsave
    ENDIF

! *** save sediment sub-system depth data ***
! NOTE: units : (m)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedxxxx_depth.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedpar(c,m,ised_par_D_mid)) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save calcite dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedxxxx_cal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfdis(c,m,ised_cal) * conv_cal_cm3_mol * conv_mol_umol) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save aragonite dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedxxxx_arg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfdis(c,m,ised_arg) * conv_arg_cm3_mol * conv_mol_umol) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save CaCO3 dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedxxxx_CaCO3.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfdis(c,m,ised_cal) * conv_cal_cm3_mol * conv_mol_umol) + &
	& (sedfdis(c,m,ised_arg) * conv_arg_cm3_mol * conv_mol_umol) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save opal dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedxxxx_opal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO m = 1,n_sed_depth
      WRITE(unit=out,FMT='(99F10.3)') ( &
	& (sedfdis(c,m,ised_opal) * conv_opal_cm3_mol * conv_mol_umol) &
	& ,c = 1,n_col)
    END DO
! close file pipe
    CLOSE(unit=out)
    
  END SUBROUTINE save_sed_fdis
! *********************************************************************


! ********************************************
! *** SAVE SEDIMENT SUB-SYSTEM DIAGNOSTICS ***
! ********************************************
  SUBROUTINE save_sed_diagnostics(sedtop,sedpar,sedtracer,sedfnp,sedfdis)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_tracer)::sedtracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_fnp)::sedfnp
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedfdis
! local variables
    INTEGER::c,m
    INTEGER::out=1
    REAL(KIND=fp_1)::sed_tot_wt
    REAL(KIND=fp_1)::sed_tot_vol
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop_wtpct ! solids as wt%
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sedtop_volpct ! solids as volume%

! *** normailze solid sediment components to a mass fraction basis (if required) ***
    DO c = 1,n_col
      DO m = 1,n_sed_depth
	  sed_tot_wt = &
	    & conv_cal_cm3_g    * sedtop(c,m,ised_cal)    + &
	    & conv_arg_cm3_g    * sedtop(c,m,ised_arg)    + &
	    & conv_opal_cm3_g   * sedtop(c,m,ised_opal)   + &
	    & conv_refrac_cm3_g * sedtop(c,m,ised_refrac) + &
	    & conv_FeO_cm3_g    * sedtop(c,m,ised_FeO)    + &
	    & conv_Corg_cm3_g   * sedtop(c,m,ised_Corg)
	  IF (sed_tot_wt > 0.0) THEN
	    sedtop_wtpct(c,m,ised_cal)       = conv_cal_cm3_g    * sedtop(c,m,ised_cal)       / sed_tot_wt
	    sedtop_wtpct(c,m,ised_arg)       = conv_arg_cm3_g    * sedtop(c,m,ised_arg)       / sed_tot_wt
	    sedtop_wtpct(c,m,ised_opal)      = conv_opal_cm3_g   * sedtop(c,m,ised_opal)      / sed_tot_wt
	    sedtop_wtpct(c,m,ised_refrac)    = conv_refrac_cm3_g * sedtop(c,m,ised_refrac)    / sed_tot_wt
	    sedtop_wtpct(c,m,ised_FeO)       = conv_FeO_cm3_g    * sedtop(c,m,ised_FeO)       / sed_tot_wt
	    sedtop_wtpct(c,m,ised_Corg)      = conv_Corg_cm3_g   * sedtop(c,m,ised_Corg)      / sed_tot_wt
	    sedtop_wtpct(c,m,ised_13C_Corg)  = conv_Corg_cm3_g   * sedtop(c,m,ised_13C_Corg)  / sed_tot_wt
	    sedtop_wtpct(c,m,ised_13C_cal)   = conv_cal_cm3_g    * sedtop(c,m,ised_13C_cal)   / sed_tot_wt
	    sedtop_wtpct(c,m,ised_13C_arg)   = conv_arg_cm3_g    * sedtop(c,m,ised_13C_arg)   / sed_tot_wt
	    sedtop_wtpct(c,m,ised_14C_cal)   = conv_cal_cm3_g    * sedtop(c,m,ised_14C_cal)   / sed_tot_wt
	    sedtop_wtpct(c,m,ised_13C_calfp) = conv_cal_cm3_g    * sedtop(c,m,ised_13C_calfp) / sed_tot_wt
	    sedtop_wtpct(c,m,ised_13C_calfb) = conv_cal_cm3_g    * sedtop(c,m,ised_13C_calfb) / sed_tot_wt
	    sedtop_wtpct(c,m,ised_18O_calfp) = conv_cal_cm3_g    * sedtop(c,m,ised_18O_calfp) / sed_tot_wt
	    sedtop_wtpct(c,m,ised_18O_calfb) = conv_cal_cm3_g    * sedtop(c,m,ised_18O_calfb) / sed_tot_wt
	  ENDIF
	  sed_tot_vol = &
	    & sedtop(c,m,ised_cal)    + &
	    & sedtop(c,m,ised_arg)    + &
	    & sedtop(c,m,ised_opal)   + &
	    & sedtop(c,m,ised_refrac) + &
	    & sedtop(c,m,ised_FeO)    + &
	    & sedtop(c,m,ised_Corg)
	  IF (sed_tot_vol > 0.0) THEN
	    sedtop_volpct(c,m,ised_cal)       = sedtop(c,m,ised_cal)       / sed_tot_vol
	    sedtop_volpct(c,m,ised_arg)       = sedtop(c,m,ised_arg)       / sed_tot_vol
	    sedtop_volpct(c,m,ised_opal)      = sedtop(c,m,ised_opal)      / sed_tot_vol
	    sedtop_volpct(c,m,ised_refrac)    = sedtop(c,m,ised_refrac)    / sed_tot_vol
	    sedtop_volpct(c,m,ised_FeO)       = sedtop(c,m,ised_FeO)       / sed_tot_vol
	    sedtop_volpct(c,m,ised_Corg)      = sedtop(c,m,ised_Corg)      / sed_tot_vol
	    sedtop_volpct(c,m,ised_13C_Corg)  = sedtop(c,m,ised_13C_Corg)  / sed_tot_vol
	    sedtop_volpct(c,m,ised_13C_cal)   = sedtop(c,m,ised_13C_cal)   / sed_tot_vol
	    sedtop_volpct(c,m,ised_13C_arg)   = sedtop(c,m,ised_13C_arg)   / sed_tot_vol
	    sedtop_volpct(c,m,ised_14C_cal)   = sedtop(c,m,ised_14C_cal)   / sed_tot_vol
	    sedtop_volpct(c,m,ised_13C_calfp) = sedtop(c,m,ised_13C_calfp) / sed_tot_vol
	    sedtop_volpct(c,m,ised_13C_calfb) = sedtop(c,m,ised_13C_calfb) / sed_tot_vol
	    sedtop_volpct(c,m,ised_18O_calfp) = sedtop(c,m,ised_18O_calfp) / sed_tot_vol
	    sedtop_volpct(c,m,ised_18O_calfb) = sedtop(c,m,ised_18O_calfb) / sed_tot_vol
	  ENDIF
	    sedtop_wtpct(c,m,:)  = 100.0 * sedtop_wtpct(c,m,:)
	    sedtop_volpct(c,m,:) = 100.0 * sedtop_volpct(c,m,:)
      END DO
    END DO


! *** save sediment sub-system diagnostic data ***
! open file pipe
    OPEN(unit=out,file=modelname//"_output/"//modelname//"_000kaBP_diagnostics_sedxxxx.res",action="WRITE")
! write data
	WRITE(unit=out,FMT='(2A3,A7,A8,A7,5A9,3A9,3A6,5A7,5A7,4A8,4A8)'), &
	  & "c", &
	  & "m", &
	  & "depth", &
	  & "temp", &
	  & "sal", &
	  & "[CO2]", &
	  & "[alk]", &
	  & "[PO4]", &
	  & "[Fe]", &
	  & "[Si]", &
	  & "[CO3]", &
	  & "dCO3ca", &
	  & "dCO3ar", &
	  & "ohmca", &
	  & "ohmar", &
	  & "pH", &
	  & "w%cal", &
	  & "w%arg", &
	  & "w%opal", &
	  & "w%Corg", &
	  & "w%refr", &
	  & "v%cal", &
	  & "v%arg", &
	  & "v%opal", &
	  & "v%Corg", &
	  & "v%refr", &
	  & "sedcal", &
	  & "sedarg", &
	  & "sedopal", &
	  & "sedCorg", &
	  & "discal", &
	  & "disarg", &
	  & "disopal", &
	  & "disCorg"
    DO c = 1,n_col
      DO m = 1,n_sed_depth
	WRITE(unit=out,FMT='(2I3,F7.1,F8.3,F7.3,5F10.3,3F9.3,3F6.3,5F7.3,5F7.3,4F8.3,4F8.3)'), &
	  & c, &
	  & m, &
	  & sedpar(c,m,ised_par_D_mid), &
	  & sedpar(c,m,ised_par_T), &
	  & sedtracer(c,m,it_S), &
	  & sedtracer(c,m,it_CO2) * conv_mol_umol, &
	  & sedtracer(c,m,it_alk) * conv_mol_umol, &
	  & sedtracer(c,m,it_PO4) * conv_mol_umol, &
	  & sedtracer(c,m,it_Fe)  * conv_mol_pmol, &
	  & sedtracer(c,m,it_Si)  * conv_mol_umol, &
	  & sedpar(c,m,ised_par_CO3)     * conv_mol_umol, &
	  & sedpar(c,m,ised_par_dCO3cal) * conv_mol_umol, &
	  & sedpar(c,m,ised_par_dCO3arg) * conv_mol_umol, &
	  & sedpar(c,m,ised_par_ohmcal), &
	  & sedpar(c,m,ised_par_ohmarg), &
	  & sedpar(c,m,ised_par_pH), &
	  & sedtop_wtpct(c,m,ised_cal), &
	  & sedtop_wtpct(c,m,ised_arg), &
	  & sedtop_wtpct(c,m,ised_opal), &
	  & sedtop_wtpct(c,m,ised_Corg), &
	  & sedtop_wtpct(c,m,ised_refrac), &
	  & sedtop_volpct(c,m,ised_cal), &
	  & sedtop_volpct(c,m,ised_arg), &
	  & sedtop_volpct(c,m,ised_opal), &
	  & sedtop_volpct(c,m,ised_Corg), &
	  & sedtop_volpct(c,m,ised_refrac), &
	  & sedfnp(c,m,ised_fnp_cal)  * conv_mol_umol / (sedpar(c,m,ised_par_A) * conv_m2_cm2), &
	  & sedfnp(c,m,ised_fnp_arg)  * conv_mol_umol / (sedpar(c,m,ised_par_A) * conv_m2_cm2), &
	  & sedfnp(c,m,ised_fnp_opal) * conv_mol_umol / (sedpar(c,m,ised_par_A) * conv_m2_cm2), &
	  & sedfnp(c,m,ised_fnp_Corg) * conv_mol_umol / (sedpar(c,m,ised_par_A) * conv_m2_cm2), &
	  & sedfdis(c,m,ised_cal)  * conv_cal_cm3_mol  * conv_mol_umol, &
	  & sedfdis(c,m,ised_arg)  * conv_arg_cm3_mol  * conv_mol_umol, &
	  & sedfdis(c,m,ised_opal) * conv_opal_cm3_mol * conv_mol_umol, &
	  & sedfdis(c,m,ised_Corg) * conv_Corg_cm3_mol * conv_mol_umol
      END DO
    END DO
! close file pipe
    CLOSE(unit=out)
    
  END SUBROUTINE save_sed_diagnostics
! ********************************************


END MODULE sue_sed


