! *************************************************************************************************
! sue_gaia.f90
! Global geophysiological mchanisms
! *************************************************************************************************



MODULE sue_gaia
  

  USE sue_config
  USE sue_lib
  IMPLICIT NONE
  SAVE


CONTAINS


! *************************************************************************************************
! *** couple in gaia meta-system with model *******************************************************
! *************************************************************************************************

! ******************************************************************
! *** couple in gaia meta-system to ocean-atmosphere sub-system ****
! ******************************************************************
  SUBROUTINE couple_gaia(R_dt,oatracer,phys,goatracer,dtot)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::R_dt
    TYPE(ocean_atm_tracer),INTENT(inout)::oatracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    TYPE(ocean_atm_tracer),INTENT(in)::goatracer
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_tracer)::dtot
! local variables
    INTEGER::it,ia
! update ocean tracers
    DO it = 1,n_tracer
      oatracer%ioat_t(:,:,it) = oatracer%ioat_t(:,:,it) + R_dt * goatracer%ioat_t(:,:,it) / phys(:,:,ip_M)
    END DO
! update atmospheric tracers
    DO ia = 1,n_atm
      oatracer%ioat_a(ia) = oatracer%ioat_a(ia) + R_dt * goatracer%ioat_a(ia) * conv_mol_atm
    END DO
! update value of 'tracer' audit total
! NOTE: add atmospheric tracer masses to ocean tracer totals for simplicity
    dtot(it_S)       = dtot(it_S)       + R_dt * SUM(goatracer%ioat_t(:,:,it_S))
    dtot(it_PO4)     = dtot(it_PO4)     + R_dt * SUM(goatracer%ioat_t(:,:,it_PO4))
    dtot(it_alk)     = dtot(it_alk)     + R_dt * SUM(goatracer%ioat_t(:,:,it_alk))
    dtot(it_Si)      = dtot(it_Si)      + R_dt * SUM(goatracer%ioat_t(:,:,it_Si))
    dtot(it_Fe)      = dtot(it_Fe)      + R_dt * SUM(goatracer%ioat_t(:,:,it_Fe))
    dtot(it_CO2)     = dtot(it_CO2)     + R_dt * SUM(goatracer%ioat_t(:,:,it_CO2)) + & 
      & R_dt * goatracer%ioat_a(ia_pCO2)
    dtot(it_13C_CO2) = dtot(it_13C_CO2) + R_dt * SUM(goatracer%ioat_t(:,:,it_13C_CO2)) + &
      & R_dt * goatracer%ioat_a(ia_13C_pCO2)
    dtot(it_O2)      = dtot(it_O2)      + R_dt * SUM(goatracer%ioat_t(:,:,it_O2))  + &
      & R_dt * goatracer%ioat_a(ia_pO2)
    dtot(it_18O_O2)  = dtot(it_18O_O2)  + R_dt * 0.0
  END SUBROUTINE couple_gaia
! ******************************************************************

  
! ******************************************************
! *** riverine input flux due to continental erosion ***
! ******************************************************
! NOTE: fluxes in units of mol yr-1
! NOTE: add fluxes to surface ocean boxes in proportion to their surface area
! (or in proportion to volume if requested)
  FUNCTION calc_fgaia_river(phys,V,A)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_river
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::V
    REAL(KIND=fp_1),INTENT(in)::A
! local variables
    INTEGER::it
! zero result value
    calc_fgaia_river%ioat_t = 0.0
    calc_fgaia_river%ioat_a = 0.0
! return function value
! NOTE: 'par_gaia_frac_riv_sur' denotes the fraction transferred to/from the surface ocean,
!       rather than the whole ocean
! calculate change in oceanic inventory
    DO it = 1,n_tracer
      calc_fgaia_river%ioat_t(:,:,it)     = &
	& par_gaia_riv(it) * (1.0 - par_gaia_frac_riv_sur) * phys(:,:,ip_V) / V
      calc_fgaia_river%ioat_t(:,1,it)     = calc_fgaia_river%ioat_t(:,1,it) + &
	& par_gaia_riv(it) * par_gaia_frac_riv_sur * phys(:,1,ip_A_top) / A
    END DO
  END FUNCTION calc_fgaia_river
! ******************************************************


! *******************************************************************************
! *** volcanic input flux due to volcano outgassing and hydrothermal activity ***
! *******************************************************************************
! NOTE: fluxes in units of mol yr-1
  FUNCTION calc_fgaia_volcan(phys,V,A)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_volcan
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::V
    REAL(KIND=fp_1),INTENT(in)::A
! local variables
    INTEGER::it
! zero result value
    calc_fgaia_volcan%ioat_t = 0.0
    calc_fgaia_volcan%ioat_a = 0.0
! return function value
! NOTE: 'par_gaia_frac_vol_sur' denotes the fraction transferred to/from the surface ocean,
!       rather than the whole ocean
! calculate change in oceanic inventory
    DO it = 1,n_tracer
      calc_fgaia_volcan%ioat_t(:,:,it)     = &
	& par_gaia_vol(it) * (1.0 - par_gaia_frac_vol_sur) * phys(:,:,ip_V) / V
      calc_fgaia_volcan%ioat_t(:,1,it)     = calc_fgaia_volcan%ioat_t(:,1,it) + &
	& par_gaia_vol(it) * par_gaia_frac_vol_sur * phys(:,1,ip_A_top) / A
    END DO
  END FUNCTION calc_fgaia_volcan
! *******************************************************************************


! ****************************************
! *** terrestrial ecosystem input flux ***
! ****************************************
! NOTE: fluxes in units of mol yr-1
  FUNCTION calc_fgaia_terr(fterr,phys,V,A)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_terr
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::fterr
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::V
    REAL(KIND=fp_1),INTENT(in)::A
! local variables
    INTEGER::it,ia
! zero result value
    calc_fgaia_terr%ioat_t = 0.0
    calc_fgaia_terr%ioat_a = 0.0
! calculate transfer of C between terrestrial ecosystems and atmosphere and return function value
! NOTE: fterr is in units of mol kyr-1 (i.e., convert to mol yr-1)
! NOTE: 'par_gaia_frac_terr_atm' denotes the fraction transferred to/from the atmosphere rather than the ocean
! NOTE: 'par_gaia_frac_terr_sur' denotes the (remaining) fraction transferred to/from the surface ocean,
!       rather than the whole ocean
! calculate change in atmospheric inventory
    calc_fgaia_terr%ioat_a(ia_pCO2)        = &
      & par_gaia_frac_terr_atm * fterr(it_CO2)
    calc_fgaia_terr%ioat_a(ia_13C_pCO2)    = &
      & par_gaia_frac_terr_atm * fterr(it_13C_CO2)
! calculate change in atmospheric and oceanic inventory
    DO ia = 1,n_atm
      it = ia
      calc_fgaia_terr%ioat_a(ia)     = &
	& par_gaia_frac_terr_atm * fterr(it)
      calc_fgaia_terr%ioat_t(:,:,it) = &
	& (1 - par_gaia_frac_terr_atm) * fterr(it) * (1.0 - par_gaia_frac_terr_sur) * phys(:,:,ip_V) / V
      calc_fgaia_terr%ioat_t(:,1,it) = calc_fgaia_terr%ioat_t(:,1,it) + &
	& (1 - par_gaia_frac_terr_atm) * fterr(it) * par_gaia_frac_terr_sur * phys(:,1,ip_A_top) / A
    END DO
! convert to correct units
    calc_fgaia_terr%ioat_t = calc_fgaia_terr%ioat_t / conv_kyr_yr
    calc_fgaia_terr%ioat_a = calc_fgaia_terr%ioat_a / conv_kyr_yr
  END FUNCTION calc_fgaia_terr
! ****************************************


! *****************************
! *** neritic sedimets flux ***
! *****************************
! NOTE: fluxes in units of mol yr-1
  FUNCTION calc_fgaia_neri(fneri,phys,V,A)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_neri
! dummy arguments
    REAL(KIND=fp_1),DIMENSION(n_tracer)::fneri
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::V
    REAL(KIND=fp_1),INTENT(in)::A
! local variables
    INTEGER::it
! zero result value
    calc_fgaia_neri%ioat_t = 0.0
    calc_fgaia_neri%ioat_a = 0.0
! calculate transfer of C and alk between coral reef (and netiric) deposits and the ocean,
! and return fuction value
! NOTE: freef is in units of mol kyr-1 (i.e., convert to mol yr-1)
! NOTE: 'par_gaia_frac_reef_sur' denotes the fraction transferred to/from the surface ocean,
!       rather than the whole ocean
! calculate change in oceanic inventory
    DO it = 1,n_tracer
      calc_fgaia_neri%ioat_t(:,:,it)     = &
	& fneri(it) * (1.0 - par_gaia_frac_neri_sur) * phys(:,:,ip_V) / V
      calc_fgaia_neri%ioat_t(:,1,it)     = calc_fgaia_neri%ioat_t(:,1,it) + &
	& fneri(it) * par_gaia_frac_neri_sur * phys(:,1,ip_A_top) / A
    END DO
! convert to correct units
    calc_fgaia_neri%ioat_t = calc_fgaia_neri%ioat_t / conv_kyr_yr
  END FUNCTION calc_fgaia_neri
! *****************************


! *****************************************
! *** atmospheric deposition input flux ***
! *****************************************
! NOTE: fluxes in units of mol yr-1
  FUNCTION calc_fgaia_dust(fdust,fdustFedis,phys)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_dust
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::fdust
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col)::fdustFedis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! zero result value
    calc_fgaia_dust%ioat_t = 0.0
    calc_fgaia_dust%ioat_a = 0.0
! calculate addition of terrigeneous Fe to surface ocean derived from the dust flux input
! by the following steps:
!     (1) calculate total terrigeneous matter input to each surface ocean box (g yr-1)
!     (2) calculate dissolved Fe input to each surface ocean box (mol yr-1)
!     NOTE: fdust is initially in units of (g cm-2 yr-1)
!     NOTE: apply a spatially-variable dust Fe solubility if requested
    IF (bioopt(bopt_uniform_frac_Fe_dis) == yes) THEN
      calc_fgaia_dust%ioat_t(:,1,it_Fe) = conv_m2_cm2 * phys(:,1,ip_A_top) * &
	& conv_Fe_g_mol * par_frac_Fe_dis * par_frac_dust_Fe * fdust(:)
    ELSE
      calc_fgaia_dust%ioat_t(:,1,it_Fe) = conv_m2_cm2 * phys(:,1,ip_A_top) * &
	& conv_Fe_g_mol * fdustFedis(:) * par_frac_dust_Fe * fdust(:)
    ENDIF
! calculate addition of terrigeneous Si to ocean derived from the dust flux input
! NOTE: it will be assumed that all Si input is to the surface ocean
    calc_fgaia_dust%ioat_t(:,1,it_Si) = conv_m2_cm2 * phys(:,1,ip_A_top) * &
	& conv_SiO2_g_mol * par_frac_SiO2_dis * par_frac_dust_SiO2 * fdust(:)
  END FUNCTION calc_fgaia_dust
! *****************************************


! ***********************
! *** misc input flux ***
! ***********************
! NOTE: fluxes in units of mol a-1
  FUNCTION calc_fgaia_misc(phys,V,A)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_misc
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::V
    REAL(KIND=fp_1),INTENT(in)::A
! local variables
    INTEGER::it
! zero result value
    calc_fgaia_misc%ioat_t = 0.0
    calc_fgaia_misc%ioat_a = 0.0
! return function value
! NOTE: 'par_gaia_frac_misc_sur' denotes the fraction transferred to/from the surface ocean,
!       rather than the whole ocean
! calculate change in oceanic inventory
    DO it = 1,n_tracer
      calc_fgaia_misc%ioat_t(:,:,it)     = &
	& par_gaia_misc(it) * (1.0 - par_gaia_frac_misc_sur) * phys(:,:,ip_V) / V
      calc_fgaia_misc%ioat_t(:,1,it)     = calc_fgaia_misc%ioat_t(:,1,it) + &
	& par_gaia_misc(it) * par_gaia_frac_misc_sur * phys(:,1,ip_A_top) / A
    END DO
  END FUNCTION calc_fgaia_misc
! ***********************


! *********************
! *** Si input flux ***
! *********************
! NOTE: fluxes in units of mol a-1
  FUNCTION calc_fgaia_Si(fSi,phys,V,A)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_fgaia_Si
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::fSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::V
    REAL(KIND=fp_1),INTENT(in)::A
! zero result value
    calc_fgaia_Si%ioat_t = 0.0
    calc_fgaia_Si%ioat_a = 0.0
! return function value
! NOTE: 'par_gaia_frac_misc_sur' denotes the fraction transferred to/from the surface ocean,
!       rather than the whole ocean
! calculate change in oceanic inventory
    calc_fgaia_Si%ioat_t(:,:,it_Si)  = &
      & fSi * (1.0 - par_gaia_frac_misc_sur) * phys(:,:,ip_V) / V
    calc_fgaia_Si%ioat_t(:,1,it_Si)  = calc_fgaia_Si%ioat_t(:,1,it_Si) + &
      & fSi * par_gaia_frac_misc_sur * phys(:,1,ip_A_top) / A
    END FUNCTION calc_fgaia_Si
! *********************


! ***********************************************************************************************
! *** calculate fluxes between the ocean-atmosphere-sediment system and the wider environment ***
! ***********************************************************************************************
! NOTE: fluxes in units of mol yr-1
  FUNCTION calc_foat_gaia(fterr,fneri,fdust,fdustFedis,fSi,phys)
    IMPLICIT NONE
! result variable
    TYPE(ocean_atm_tracer)::calc_foat_gaia
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::fterr
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::fneri
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::fdust
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col)::fdustFedis
    REAL(KIND=fp_1),INTENT(in)::fSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    TYPE(ocean_atm_tracer)::fg_vo,fg_ri,fg_du,fg_mi,fg_te,fg_ne,fg_si
    REAL(KIND=fp_1)::tot_V     ! total ocean volume
    REAL(KIND=fp_1)::tot_A_sur ! total ocean surface area
! zero function value
    calc_foat_gaia%ioat_t = 0.0
    calc_foat_gaia%ioat_a = 0.0
! calculate local constants
    tot_V     = SUM(phys(:,:,ip_V))
    tot_A_sur = SUM(phys(:,1,ip_A_top))
! calculate individucal gaian components
    fg_vo = calc_fgaia_volcan(phys,tot_V,tot_A_sur) 
    fg_ri = calc_fgaia_river(phys,tot_V,tot_A_sur)
    fg_mi = calc_fgaia_misc(phys,tot_V,tot_A_sur)   
    fg_te = calc_fgaia_terr(fterr,phys,tot_V,tot_A_sur)
    fg_ne = calc_fgaia_neri(fneri,phys,tot_V,tot_A_sur)  
    fg_du = calc_fgaia_dust(fdust,fdustFedis,phys)
    fg_si = calc_fgaia_Si(fSi,phys,tot_V,tot_A_sur)   
! return function value
     IF (runopt(ropt_gaia) == yes) THEN
       calc_foat_gaia%ioat_t = fg_te%ioat_t + fg_ne%ioat_t + &
	 & fg_vo%ioat_t + fg_ri%ioat_t + fg_du%ioat_t + fg_mi%ioat_t + fg_si%ioat_t
       calc_foat_gaia%ioat_a = fg_te%ioat_a + fg_ne%ioat_a + &
	 & fg_vo%ioat_a + fg_ri%ioat_a + fg_du%ioat_a + fg_mi%ioat_a + fg_si%ioat_a
     ELSE
       calc_foat_gaia%ioat_t = fg_te%ioat_t + fg_ne%ioat_t
       calc_foat_gaia%ioat_a = fg_te%ioat_a + fg_ne%ioat_a
     ENDIF
   END FUNCTION calc_foat_gaia
! ***********************************************************************************************


! **************************************
! *** calculate spin-up gaian fluxes ***
! **************************************
   FUNCTION calc_foat_gaia_spin(fterr,fneri,fdust,fdustFedis,fSi,phys)
     IMPLICIT NONE
! result variable
     TYPE(ocean_atm_tracer)::calc_foat_gaia_spin
! dummy arguments
     REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::fterr
     REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::fneri
     REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::fdust
     REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col)::fdustFedis
     REAL(KIND=fp_1),INTENT(in)::fSi
     REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
     TYPE(ocean_atm_tracer)::fg_vo,fg_ri,fg_du,fg_mi,fg_ne,fg_si
     REAL(KIND=fp_1)::tot_V     ! total ocean volume
     REAL(KIND=fp_1)::tot_A_sur ! total ocean surface area
! zero function value
     calc_foat_gaia_spin%ioat_t = 0.0
     calc_foat_gaia_spin%ioat_a = 0.0
! calculate local constants
     tot_V     = SUM(phys(:,:,ip_V))
     tot_A_sur = SUM(phys(:,1,ip_A_top))
! calculate individucal gaian components
     fg_vo = calc_fgaia_volcan(phys,tot_V,tot_A_sur) 
     fg_ri = calc_fgaia_river(phys,tot_V,tot_A_sur)
     fg_mi = calc_fgaia_misc(phys,tot_V,tot_A_sur)
     fg_ne = calc_fgaia_neri(fneri,phys,tot_V,tot_A_sur)
     fg_du = calc_fgaia_dust(fdust,fdustFedis,phys)
     fg_si = calc_fgaia_Si(fSi,phys,tot_V,tot_A_sur)
! return function value
     IF (runopt(ropt_gaia) == yes) THEN
       calc_foat_gaia_spin%ioat_t = &
	 & fg_vo%ioat_t + fg_ri%ioat_t + fg_du%ioat_t + fg_mi%ioat_t + fg_ne%ioat_t + fg_si%ioat_t
       calc_foat_gaia_spin%ioat_a = &
	 & fg_vo%ioat_a + fg_ri%ioat_a + fg_du%ioat_a + fg_mi%ioat_a + fg_ne%ioat_a + fg_si%ioat_a
     ELSE
       calc_foat_gaia_spin%ioat_t = 0.0
       calc_foat_gaia_spin%ioat_a = 0.0
     ENDIF
   END FUNCTION calc_foat_gaia_spin
! **************************************


! **********************************************
! *** calculate initial spin-up gaian fluxes ***
! **********************************************
   FUNCTION calc_foat_gaia_spin_init(sigfterr,sigfneri,phys)
     IMPLICIT NONE
! result variable
     TYPE(ocean_atm_tracer)::calc_foat_gaia_spin_init
! dummy arguments
     REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr,n_tracer)::sigfterr
     REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr,n_tracer)::sigfneri
     REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
     INTEGER::t_loop
     TYPE(ocean_atm_tracer)::fg_te,fg_ne !
     REAL(KIND=fp_1)::tot_V              ! total ocean volume
! zero function value
     calc_foat_gaia_spin_init%ioat_t = 0.0
     calc_foat_gaia_spin_init%ioat_a = 0.0
! calculate local constants
     tot_V = SUM(phys(:,:,ip_V))
! loop through length of simulation, and 
! (1) calculate gaian components
! (2) increment function value
! NOTE: gaian changes have to be summed back until the simulation start point,
! and the result inverted
! NOTE: fluxes arising from terrestrial C and coral reef CaCO3 changes calculated in this way
! are automatically in units of mol yr-1 which is what is required
! NOTE: organic carbon and CaCO3 are assumed to take place in all ocean boxes in proportion to their volume,
! with no chamges being made to the atmosphere
! NOTE: do not update C and alk inventories due to changes in coral reef storage for the internal sub-model,
!       as there is a increasing background in CaCO3 storage in the signal and the transfer of CaCO3 
!       at the start of spin-up may be so large than the system has difficulty in recovering
     DO t_loop = t_ice_init,t_ice_max,-1
       fg_te%ioat_t(:,:,it_CO2)     = sigfterr(t_loop,it_CO2)     * phys(:,:,ip_V) / tot_V
       fg_te%ioat_t(:,:,it_13C_CO2) = sigfterr(t_loop,it_13C_CO2) * phys(:,:,ip_V) / tot_V
       calc_foat_gaia_spin_init%ioat_t = calc_foat_gaia_spin_init%ioat_t - fg_te%ioat_t
       calc_foat_gaia_spin_init%ioat_a = calc_foat_gaia_spin_init%ioat_a - fg_te%ioat_a
     ENDDO
   END FUNCTION calc_foat_gaia_spin_init
! **********************************************


! ******************************
! *** 'tracer' audit of gaia ***
! ******************************
  FUNCTION calc_oatracer_tot(oatracer,phys)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(n_tracer)::calc_oatracer_tot
! dummy arguments
    TYPE(ocean_atm_tracer),INTENT(in)::oatracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::it,ia
    REAL(KIND=fp_1),DIMENSION(n_tracer)::tot_box
    REAL(KIND=fp_1),DIMENSION(n_tracer)::tot_atm
    TYPE(ocean_atm_tracer)::oatracer_DOCremin
! zero variables
    tot_box=0.0
    tot_atm=0.0
    calc_oatracer_tot = 0.0
! initialize local tracer variable
    oatracer_DOCremin%ioat_t = oatracer%ioat_t
    oatracer_DOCremin%ioat_a = oatracer%ioat_a
! remineralize DOC over and above the inventory for the purposes of the audit
    oatracer_DOCremin%ioat_t(:,:,it_DOC)     = 0.0
    oatracer_DOCremin%ioat_t(:,:,it_13C_DOC) = 0.0
    oatracer_DOCremin%ioat_t(:,:,it_DOFe)    = 0.0
    oatracer_DOCremin%ioat_t(:,:,it_CO2)     = oatracer_DOCremin%ioat_t(:,:,it_CO2) + &
      & oatracer%ioat_t(:,:,it_DOC)
    oatracer_DOCremin%ioat_t(:,:,it_13C_CO2) = oatracer_DOCremin%ioat_t(:,:,it_13C_CO2) + &
      & oatracer%ioat_t(:,:,it_13C_DOC)
    oatracer_DOCremin%ioat_t(:,:,it_PO4)     = oatracer_DOCremin%ioat_t(:,:,it_PO4) + &
      & red_Corg_Porg * oatracer%ioat_t(:,:,it_DOC)
    oatracer_DOCremin%ioat_t(:,:,it_alk)     = oatracer_DOCremin%ioat_t(:,:,it_alk) - &
      & red_Corg_Norg * oatracer%ioat_t(:,:,it_DOC)
    oatracer_DOCremin%ioat_t(:,:,it_O2)      = oatracer_DOCremin%ioat_t(:,:,it_O2) - &
      & red_Corg_O2 * oatracer%ioat_t(:,:,it_DOC)
    oatracer_DOCremin%ioat_t(:,:,it_Fe)      = oatracer_DOCremin%ioat_t(:,:,it_Fe) + &
      & oatracer%ioat_t(:,:,it_DOFe)
! sum ocean components
! NOTE 1: ignore any particulate material that has not been reminerilized or sedimented
! NOTE 2: accounting for changes in 18O is omitted for now, as d18O of the ocean is artificially forced
    DO it = 1,n_tracer
      tot_box(it) = SUM(oatracer_DOCremin%ioat_t(:,:,it) * phys(:,:,ip_M))
    END DO
! sum atmospheric components
    tot_atm(it_CO2)     = oatracer_DOCremin%ioat_a(ia_pCO2)     * conv_atm_mol
    tot_atm(it_13C_CO2) = oatracer_DOCremin%ioat_a(ia_13C_pCO2) * conv_atm_mol
    tot_atm(it_O2)      = oatracer_DOCremin%ioat_a(ia_pO2)      * conv_atm_mol
    tot_atm(it_18O_O2)  = oatracer_DOCremin%ioat_a(ia_18O_pO2)  * conv_atm_mol
! add components and return function value
    calc_oatracer_tot = tot_box + tot_atm
  END FUNCTION calc_oatracer_tot
! ******************************


! ***********************************************************************************************************
! *** audit 'tracer' (sum tracer values in all components of the system to ensure that mass is conserved) ***
! ***********************************************************************************************************
  SUBROUTINE audit_gaia(dtot,totinit,oatracer,phys,dtotper)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::dtot
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::totinit
    TYPE(ocean_atm_tracer),INTENT(in)::oatracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_tracer)::dtotper
! local variables
    INTEGER::it
    REAL(KIND=fp_1),DIMENSION(n_tracer)::tot_new=0.0
! make new geochemical inventory of ocean-atmosphere system
    tot_new = calc_oatracer_tot(oatracer,phys)
! calculate percentage change in total system geochemical inventory
    dtotper = 100.0 * (tot_new - (totinit + dtot)) / (totinit + 1.0)
! check that the system has not drifted 'unreasonably'
! if so: issue warning
    DO it = 1,n_tracer
      IF ((ABS(dtotper(it)) >= par_dtot_per_threshold) .AND. &
	& (it /= it_18O_O2) .AND. (it /= it_T) .AND. (it /= it_S)) THEN
	IF (runopt(ropt_warn) == yes) THEN
	  PRINT*,"WARNING: the inventory of tracer  <",it,"> has significantly drifted"
	ENDIF
	program_error = yes
	warning(iwarn_traceraudit) = yes
      ENDIF
    END DO
  END SUBROUTINE audit_gaia
! ***********************************************************************************************************


! *************************************************************************************************
! *** PALEO GAIAN FORCINGS CALCULATIONS ***********************************************************
! *************************************************************************************************


! *******************************************************************************************
! *** estimate carbon transfer between terrestrial ecosystems and ocean-atmosphere system ***
! *******************************************************************************************
  FUNCTION calc_sig_fterr(terr)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_tracer)::calc_sig_fterr
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr,n_tracer)::terr
! local variables
    INTEGER::t
! zero result variable
    calc_sig_fterr = 0.0
! calculate successive changes in carbon stored in the terrestrial biosphere
! (increases are positive)
    DO t = t_ice_init,(t_ice_max + 1),dt_ice
      calc_sig_fterr(t,:) = terr(t - 1,:) - terr(t,:)
    END DO
! invert signal, to give corresponding C changes in ocean-atmosphere system
    calc_sig_fterr(:,:) = -calc_sig_fterr(:,:)
  END FUNCTION calc_sig_fterr
! *******************************************************************************************
  
  
! *************************************************************
! *** estimate exchange between neritic sediments and ocean ***
! *************************************************************
  FUNCTION calc_sig_fneri(neri)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_tracer)::calc_sig_fneri
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr,n_tracer)::neri
! local variables
    INTEGER::t
! zero result variable
    calc_sig_fneri(:,:) = 0.0
! calculate successive changes neritic sediments storage
! (increases are positive)
    DO t = t_ice_init,(t_ice_max + 1),dt_ice
      calc_sig_fneri(t,:) = neri(t - 1,:) - neri(t,:)
    END DO
! invert signal, to give corresponding tracer changes in ocean-atmosphere system
    calc_sig_fneri(:,:) = -calc_sig_fneri(:,:)
  END FUNCTION calc_sig_fneri
! *************************************************************


! **********************************************************************************************************
! *** estimate CaCO3 transfer between shallow carbonates and ocean - after Mundhoven and Francois [1996] ***
! **********************************************************************************************************
  SUBROUTINE calc_sig_neri_CaCO3(signeri,bathdata,sigsealevel,PIGphys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(0:n_kyr,n_tracer)::signeri
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bath_depth,n_bath_par)::bathdata
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr)::sigsealevel
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::PIGphys
! local variables
! NOTE: "depth"      <=> neretic ocean floor depth below sea level (m)
!       "CaCO3_reef" <=> neretic coral reef CaCO3 storage
!       "CaCO3_neri" <=> netiric sediment (non-reefal) CaCO3 storage
! NOTE: use allocatable variables because of the large array sizes
    INTEGER::c
    INTEGER::t,o
    INTEGER::out=1
    REAL(KIND=fp_1),DIMENSION(0:n_kyr)::dsealeveldt                   ! rate of change of sealevel
    REAL(KIND=fp_1),DIMENSION(n_col,par_n_bath_depth_neri)::bath_area ! sea floor area 
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::depth   
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::CaCO3_reefal
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::CaCO3_nonreefal
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::buildup_reefal
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::buildup_nonreefal
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::erosion_reefal
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::erosion_nonreefal

! *** allocate size of local neritic sediment variable arrays ***
    ALLOCATE(depth(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(CaCO3_reefal(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(CaCO3_nonreefal(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(buildup_reefal(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(buildup_nonreefal(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(erosion_reefal(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(erosion_nonreefal(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    
! *** check for problems allocating array space ***
    IF (error /= 0) THEN
      PRINT*,"FATAL ERROR [sue_gaia.f90, calc_sig_CaCO3carb_MF]:"
      PRINT*,"Array space could not be allocated"
      STOP
    ENDIF

! *** initialize local variable ***
    dsealeveldt(:)         = 0.0
    bath_area(:,:)         = 0.0
    depth(:,:,:)           = 0.0
    CaCO3_reefal(:,:,:)      = 0.0
    CaCO3_nonreefal(:,:,:)   = 0.0
    buildup_reefal(:,:,:)    = 0.0
    buildup_nonreefal(:,:,:) = 0.0
    erosion_reefal(:,:,:)    = 0.0
    erosion_nonreefal(:,:,:) = 0.0

! *** calculate rate of sealevel rise and water depth ***
! rate of sealevel rise
! NOTE: in units of (m kyr-1)
! NOTE: start from time = (n_kyr - 1)
    DO t = (n_kyr - 1),0,-1
      dsealeveldt(t) = (sigsealevel(t) - sigsealevel(t + 1)) / 1.0
    END DO
! water depth
! NOTE: use only the upper portion of the total ocean depth profile
    DO t = n_kyr,0,-1
      depth(t,:,1:par_n_bath_depth_neri) = bathdata(:,1:par_n_bath_depth_neri,ibath_D) + sigsealevel(t)
    ENDDO

! *** calculate ocean floor area in 1 m depth increments ***
! NOTE: use only the upper portion of the total ocean depth profile
    DO o = 1,par_n_bath_depth_neri
      bath_area(:,o) = PIGphys(:,1,ip_A_top) * bathdata(:,o,ibath_Afrac)
    END DO

! *** calculate change in reef and netiric CaCO3 storage ***
! NOTE: CaCO3 storage in (mol)
! NOTE: changes in CaCO3 storage in (mol kyr-1)
! NOTE: start from time = (n_kyr - 1)
! NOTE: calculate increase due to coral reef and netiric CaCO3 accumulation first,
!       and then losses due to erosion
! NOTE: assume that coral only grows in water depths shallower than "par_gaia_MF_maxDreef"
! NOTE: assume that neritic carbonate accumulation only occurs in water depths 
!       less than "par_gaia_maxDneri" below sealevel
!       - typically this depth value would be set to the depth of the ocean mixed layer,
!       within which no sediment accumulation is allowed.
! NOTE: total CaCO3 erosion is calculated => this must be partitioned into reefal and non-reefal CaCO3
!       in proportion to the amounts of stored reefal and non-reefal CaCO3
    DO t = (n_kyr - 1),0,-1
      DO c = 1,n_col
	DO o = 1,par_n_bath_depth_neri
! reefal neritic CaCO3
	  IF ((depth(t,c,o) > 0.0) .AND. (depth(t,c,o) <= par_gaia_neri_CaCO3_reefal_maxD)) THEN
	    buildup_reefal(t,c,o) = par_gaia_neri_CaCO3_reefal_racc * bath_area(c,o) * &
	      & calc_CaCO3reef_MF_precip(dsealeveldt(t))
	  ELSE
	    buildup_reefal(t,c,o) = 0.0
	  ENDIF
	  IF (depth(t,c,o) < 0.0) THEN
	    erosion_reefal(t,c,o) = par_gaia_neri_CaCO3_reefal_rero * bath_area(c,o)
	  ELSE
	    erosion_reefal(t,c,o) = 0.0
	  ENDIF
	  IF (erosion_reefal(t,c,o) > CaCO3_reefal(t + 1,c,o)) THEN
	    erosion_reefal(t,c,o) = CaCO3_reefal(t + 1,c,o)
	  ENDIF
	  CaCO3_reefal(t,c,o) = CaCO3_reefal(t + 1,c,o) + &
	    & (buildup_reefal(t,c,o) - erosion_reefal(t,c,o))
! non-reefal neritic CaCO3
	  IF ((depth(t,c,o) > 0.0) .AND. (depth(t,c,o) <= par_gaia_neri_CaCO3_nonreefal_maxD)) THEN
	    buildup_nonreefal(t,c,o) = par_gaia_neri_CaCO3_nonreefal_racc * bath_area(c,o)
	  ELSE
	    buildup_nonreefal(t,c,o) = 0.0
	  ENDIF
	  IF (depth(t,c,o) < 0.0) THEN
	    erosion_nonreefal(t,c,o) = par_gaia_neri_CaCO3_nonreefal_rero * bath_area(c,o)
	  ELSE
	    erosion_nonreefal(t,c,o) = 0.0
	  ENDIF
	  IF (erosion_nonreefal(t,c,o) > CaCO3_nonreefal(t + 1,c,o)) THEN
	    erosion_nonreefal(t,c,o) = CaCO3_nonreefal(t + 1,c,o)
	  ENDIF
	  CaCO3_nonreefal(t,c,o) = CaCO3_nonreefal(t + 1,c,o) + &
	    & (buildup_nonreefal(t,c,o) - erosion_nonreefal(t,c,o))
	END DO
      END DO
    END DO

! *** calculate total CaCO3 storage and return function value ***
    DO t = 0,n_kyr
      signeri(t,it_CO2)     = SUM(CaCO3_reefal(t,:,:)) + SUM(CaCO3_nonreefal(t,:,:))
      signeri(t,it_13C_CO2) = calc_frac_13C(par_gaia_13C_neri) * signeri(t,it_CO2)
      signeri(t,it_alk)     = 2.0 * signeri(t,it_CO2)
    END DO

! *** save raw data ***
! convert units (mol to 10^15 mol)
    CaCO3_reefal(:,:,:)      = CaCO3_reefal(:,:,:)   / 1.0E+15
    CaCO3_nonreefal(:,:,:)   = CaCO3_nonreefal(:,:,:)   / 1.0E+15
    buildup_reefal(:,:,:)    = buildup_reefal(:,:,:) / 1.0E+15
    buildup_nonreefal(:,:,:) = buildup_nonreefal(:,:,:) / 1.0E+15
    erosion_reefal(:,:,:)    = erosion_reefal(:,:,:) / 1.0E+15
    erosion_nonreefal(:,:,:) = erosion_nonreefal(:,:,:) / 1.0E+15
! save reefal CaCO3 storage
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_reefal.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(CaCO3_reefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save non-reefal CaCO3 storage
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_nonreefal.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(CaCO3_nonreefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save total neritic CaCO3 storage
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_tot.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(CaCO3_reefal(t,:,:)) + SUM(CaCO3_nonreefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save reefal CaCO3 buildup
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_reefal_accumul.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(buildup_reefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save non-reefal CaCO3 buildup
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_nonreefal_accumul.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(buildup_nonreefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save total CaCO3 buildup
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_tot_accumul.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(buildup_reefal(t,:,:) + buildup_nonreefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save reefal CaCO3 erosion
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_reefal_erosion.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(erosion_reefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save non-reefal CaCO3 erosion
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_nonreefal_erosion.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(erosion_nonreefal(t,:,:))
    END DO
    CLOSE(unit=out)
! save total CaCO3 erosion
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_CaCO3_tot_erosion.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(erosion_reefal(t,:,:) + erosion_nonreefal(t,:,:))
    END DO
    CLOSE(unit=out)

! *** deallocate array space ***
    DEALLOCATE(depth,STAT=dealloc_error)
    DEALLOCATE(CaCO3_reefal,STAT=dealloc_error)
    DEALLOCATE(CaCO3_nonreefal,STAT=dealloc_error)
    DEALLOCATE(buildup_reefal,STAT=dealloc_error)
    DEALLOCATE(buildup_nonreefal,STAT=dealloc_error)
    DEALLOCATE(erosion_reefal,STAT=dealloc_error)
    DEALLOCATE(erosion_nonreefal,STAT=dealloc_error)
    
! *** check for problems de-allocating array space ***
    IF (dealloc_error /= 0) THEN
      PRINT*,"FATAL ERROR [sue_gaia.f90, calc_sig_neri_CaCO3]:"
      PRINT*,"Array space could not be de-allocated"
      STOP
    ENDIF

  END SUBROUTINE calc_sig_neri_CaCO3
! **********************************************************************************************************


! ***********************************************************************************
! *** calculate coral reef growth limitation factor [Munhoven end Francois, 1996] ***
! ***********************************************************************************
  FUNCTION calc_CaCO3reef_MF_precip(dsealeveldt)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_CaCO3reef_MF_precip
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::dsealeveldt
! return function value
    IF (gaiaopt(gopt_neri_CaCO3_reefal_dsealevel) == yes) THEN
      IF (dsealeveldt <= -2.5) THEN
	calc_CaCO3reef_MF_precip = 0.0
      ELSEIF (dsealeveldt < 2.5) THEN
	calc_CaCO3reef_MF_precip = (2.5 + dsealeveldt) / 5.0
      ELSEIF (dsealeveldt < 10.0) THEN
	calc_CaCO3reef_MF_precip = 1.0
      ELSEIF (dsealeveldt < 15.0) THEN
	calc_CaCO3reef_MF_precip = (15.0 - dsealeveldt) / 5.0
      ELSEIF (dsealeveldt >= 15.0) THEN
	calc_CaCO3reef_MF_precip = 0.0
      ENDIF
    ELSE
      calc_CaCO3reef_MF_precip = 1.0
    END IF
  END FUNCTION calc_CaCO3reef_MF_precip
! ***********************************************************************************


! *******************************************************************************
! *** estimate dissolved silica transfer between shallow carbonates and ocean ***
! *******************************************************************************
  SUBROUTINE calc_sig_neri_opal(signeri,bathdata,sigsealevel,PIGphys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(0:n_kyr,n_tracer)::signeri
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bath_depth,n_bath_par)::bathdata
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr)::sigsealevel
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::PIGphys
! local variables
! NOTE: "depth"     <=> neretic ocean floor depth below sea level (m)
!       "opal_neri" <=> neretic sediment opal storage
! NOTE: use allocatable variables because of the large array sizes
    INTEGER::c
    INTEGER::t,o
    INTEGER::out=1
    REAL(KIND=fp_1)::erosion_tot     !      
    REAL(KIND=fp_1)::opal_tot        !       
    REAL(KIND=fp_1)::A_maxDneri_tot  ! cumulative ocean floor area to "maxDneri" (m2)
    REAL(KIND=fp_1)::opal_raccumul   ! accumulation rate of neritic sedimentary opal (mol m-2 kyr-1)
    REAL(KIND=fp_1),DIMENSION(n_col,par_n_bath_depth_neri)::bath_area ! sea floor area 
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::depth   
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::opal_neri  
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::buildup_neri 
    REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::erosion_neri

! *** allocate size of local neritic sediment variable arrays ***
    ALLOCATE(depth(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(opal_neri(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(buildup_neri(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)
    ALLOCATE(erosion_neri(0:n_kyr,n_col,par_n_bath_depth_neri),STAT=error)

! *** check for problems allocating array space ***
    IF (error /= 0) THEN
      PRINT*,"FATAL ERROR [sue_gaia.f90, calc_sig_opal]:"
      PRINT*,"Array space could not be allocated"
      STOP
    ENDIF

! *** initialize local variable ***
    A_maxDneri_tot      = 0.0
    bath_area(:,:)      = 0.0
    depth(:,:,:)        = 0.0
    opal_neri(:,:,:)    = 0.0
    buildup_neri(:,:,:) = 0.0
    erosion_neri(:,:,:) = 0.0

! *** calculate water depth ***
! NOTE: use only the upper portion of the total ocean depth profile
    DO t = n_kyr,0,-1
      depth(t,:,1:par_n_bath_depth_neri) = bathdata(:,1:par_n_bath_depth_neri,ibath_D) + sigsealevel(t)
    ENDDO

! *** calculate ocean floor area in 1 m depth increments ***
! NOTE: use only the upper portion of the total ocean depth profile
    DO o = 1,par_n_bath_depth_neri
      bath_area(:,o) = PIGphys(:,1,ip_A_top) * bathdata(:,o,ibath_Afrac)
    END DO

! *** calculate change in netiric opal storage ***
! NOTE: opal storage in (mol)
! NOTE: changes in opal storage in (mol kyr-1)
! NOTE: start from time = (n_kyr - 1)
! NOTE: calculate increase due to netiric opal accumulation first,
!       and then losses due to erosion
! NOTE: assume that neritic opal accumulation only occurs in water depths 
!       less than "par_gaia_maxDneri" below sealevel
!       - typically this depth value would be set to the depth of the ocean mixed layer,
!       within which no sediment accumulation is allowed.
    DO t = (n_kyr - 1),0,-1
      DO c = 1,n_col
	DO o = 1,par_n_bath_depth_neri
	  IF ((depth(t,c,o) > 0.0) .AND. (depth(t,c,o) <= par_gaia_neri_opal_maxD)) THEN
	    buildup_neri(t,c,o) = par_gaia_neri_opal_racc * bath_area(c,o)
	  ELSE
	    buildup_neri(t,c,o) = 0.0
	  ENDIF
	  IF (depth(t,c,o) < 0.0) THEN
	    erosion_neri(t,c,o) = par_gaia_neri_opal_rero * bath_area(c,o)
	  ELSE
	    erosion_neri(t,c,o) = 0.0
	  ENDIF
	  IF (erosion_neri(t,c,o) > opal_neri(t + 1,c,o)) THEN
	    erosion_neri(t,c,o) = opal_neri(t + 1,c,o)
	  ENDIF
	  opal_neri(t,c,o) = opal_neri(t + 1,c,o) + &
	    & (buildup_neri(t,c,o) - erosion_neri(t,c,o))
	END DO
      END DO
    END DO

! *** calculate total opal storage and return function value ***
    DO t = n_kyr,0,-1
      signeri(t,it_Si) = SUM(opal_neri(t,:,:))
    END DO

! *** save raw data ***
! convert units (mol to 10^15 mol)
    opal_neri(:,:,:)    = opal_neri(:,:,:)    / 1.0E+15
    buildup_neri(:,:,:) = buildup_neri(:,:,:) / 1.0E+15
    erosion_neri(:,:,:) = erosion_neri(:,:,:) / 1.0E+15
! save neritic opal storage
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_opal_tot.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(opal_neri(t,:,:))
    END DO
    CLOSE(unit=out)
! save neritic opal buildup
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_opal_buildup.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(buildup_neri(t,:,:))
    END DO
    CLOSE(unit=out)
! save neritic opal erosion
    OPEN(unit=out,file=modelname//"_output/"//"suexxxx_sig_neri_opal_erosion.res",action="write")
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I5,F10.3)') t,SUM(erosion_neri(t,:,:))
    END DO
    CLOSE(unit=out)

! *** deallocate array space ***
    DEALLOCATE(depth,STAT=dealloc_error)
    DEALLOCATE(opal_neri,STAT=dealloc_error)
    DEALLOCATE(buildup_neri,STAT=dealloc_error)
    DEALLOCATE(erosion_neri,STAT=dealloc_error)
    
! *** check for problems de-allocating array space ***
    IF (dealloc_error /= 0) THEN
      PRINT*,"FATAL ERROR [sue_gaia.f90, calc_sig_opal]:"
      PRINT*,"Array space could not be de-allocated"
      STOP
    ENDIF
    
  END SUBROUTINE calc_sig_neri_opal
! *******************************************************************************


! *************************************************************************************************


END MODULE sue_gaia
