! *************************************************************************************************
! sue_sed_lib.f90
! Sediment system library routines
! *************************************************************************************************



MODULE sue_sed_lib


  USE sue_config
  USE sue_lib
  USE sue_co2calc
  USE sue_sed_CaCO3
  USE sue_sed_opal
  IMPLICIT NONE
  SAVE
  
  
CONTAINS


! ***************************************************************
! *** calculate sediment CaCO3 diagenesis via a look-up table ***
! ***************************************************************
  SUBROUTINE calc_sed_dis_CaCO3_lookup(seddis,sednew,sedtop,sedpar,sedtracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sed_com_par)::seddis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_com_par)::sednew
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_com_par)::sedtop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::sedtracer
! local variables
    REAL(KIND=fp_1)::D                 ! depth in ocean (m)
    REAL(KIND=fp_1)::dCO3_cal,dCO3_arg ! over-saturation (mol [CO3--] kg-1)
    REAL(KIND=fp_1)::frac_cal_top      ! dry mass fraction of calcite (in top sediment)
    REAL(KIND=fp_1)::frac_arg_top      ! dry mass fraction of aragonite (in top sediment)
    REAL(KIND=fp_1)::conc_O2           ! concentration of dissolved O2 (mol kg-1)
    REAL(KIND=fp_1)::fCorg             ! Corg rain rate (mol cm-2 yr-1)
    REAL(KIND=fp_1)::fCorg_cal         ! Corg rain rate (associated with calcite dissolution)
    REAL(KIND=fp_1)::fCorg_arg         ! Corg rain rate (associated with aragonite dissolution)
    REAL(KIND=fp_1)::sed_wt_top        ! sediment mass (in top sediment)
    REAL(KIND=fp_1)::frac_caldis_new   ! fraction of calcite dissolved from new sediment
    REAL(KIND=fp_1)::frac_caldis_top   ! fraction of calcite dissolved from top sediment
    REAL(KIND=fp_1)::frac_caldisi_new  ! fraction of calcite dissolved from new sediment (interface)
    REAL(KIND=fp_1)::frac_caldisi_top  ! fraction of calcite dissolved from top sediment (interface)
    REAL(KIND=fp_1)::frac_caldish_new  ! fraction of calcite dissolved from new sediment (homogeneous)
    REAL(KIND=fp_1)::frac_caldish_top  ! fraction of calcite dissolved from top sediment (homogeneous)
    REAL(KIND=fp_1)::frac_argdis_new   ! fraction of aragonite dissolved from new sediment
    REAL(KIND=fp_1)::frac_argdis_top   ! fraction of aragonite dissolved from top sediment
    REAL(KIND=fp_1)::frac_argdisi_new  ! fraction of aragonite dissolved from new sediment (interface)
    REAL(KIND=fp_1)::frac_argdisi_top  ! fraction of aragonite dissolved from top sediment (interface)
    REAL(KIND=fp_1)::frac_argdish_new  ! fraction of aragonite dissolved from new sediment (homogeneous)
    REAL(KIND=fp_1)::frac_argdish_top  ! fraction of aragonite dissolved from top sediment (homogeneous)
    REAL(KIND=fp_1)::dis_cal           ! raw calcite dissolution flux from lookup table
    REAL(KIND=fp_1)::dis_arg           ! raw aragonite dissolution flux from lookup table
    REAL(KIND=fp_1)::dis_cal_und       ! undersaturation-driven calcite dissolution flux from lookup table
    REAL(KIND=fp_1)::dis_arg_und       ! undersaturation-driven aragonite dissolution flux from lookup table
    REAL(KIND=fp_1)::rdis_cal          ! calcite interface/homogeneous dissolution ratio
    REAL(KIND=fp_1)::rdis_arg          ! aragonite interface/homogeneous dissolution ratio

! *** calculate sediment diagenesis defining variables ***
! NOTE: the units of the Corg flux must be changed from (cm3 cm-2 yr-1) to (mol cm-2 yr-1),
!       in order to calculate CaCO3 diagenesis
! NOTE: cap the Corg flux at a values of 50 (umol cm-2 yr-1) (or 'lookup_fCorg_max'),
!       since the lookup table is only generated over that range,
!       and enough [O2] may not in actual fact be available to oxidize such a flux in the sediments fully
! depth in ocean
    D = sedpar(ised_par_D_mid)
! set saturation state wrt calcite and aragonite
    dCO3_cal = sedpar(ised_par_dCO3cal)
    dCO3_arg = sedpar(ised_par_dCO3arg)
! set bottom-water [O2]
    conc_O2 = 200.0E-06
! mass fraction (dry) of calcite and aragonite in sediments
    sed_wt_top = conv_cal_cm3_g    * sedtop(ised_cal)    + &
      &          conv_arg_cm3_g    * sedtop(ised_arg)    + &
      &          conv_opal_cm3_g   * sedtop(ised_opal)   + &
      &          conv_refrac_cm3_g * sedtop(ised_refrac) + &
      &          conv_FeO_cm3_g    * sedtop(ised_FeO)    + &
      &          conv_Corg_cm3_g   * sedtop(ised_Corg)
    frac_cal_top = conv_cal_cm3_g * sedtop(ised_cal) / sed_wt_top
    frac_arg_top = conv_arg_cm3_g * sedtop(ised_arg) / sed_wt_top
! Corg rain rate
    fCorg = par_sed_diagenfrac_Corg * conv_Corg_cm3_mol * sednew(ised_Corg)
    IF (fCorg > lookup_fCorg_max) fCorg = lookup_fCorg_max
    IF (sedopt(sopt_sed_diagen_respir) == no) fCorg = 0.0
    fCorg_cal = (sedtop(ised_cal) / (sedtop(ised_cal) + sedtop(ised_arg) + 0.000001)) * fCorg
    fCorg_arg = (sedtop(ised_arg) / (sedtop(ised_cal) + sedtop(ised_arg) + 0.000001)) * fCorg

! *** calculate the calcite dissolution flux ***
! NOTE: the dissolution flux is calculated in units of (mol cm-2 yr);
!       this must be converted back to cm3 cm-2 yr
! NOTE: the procedure for calculating 'interface' dissolution of calcite
!       from new sedimenting and top sediment material is:
!       (1) IF the estimated dissolution flux < 0.0, reset to 0.0
!           (this is possible, becuase of the use of extrapolation for values falling outside of 
!           the look-up table bounds)
!       (2) ELSEIF the estimated dissolution flux is less than the new material flux,
!           then take all calcite required for the dissolution flux from the new material 
!       (3) ELSEIF the estimated dissolution flux is greater or equal to the combined inventories of 
!           new material and top material,
!           then cap the dissolution flux at this total
!       (4) ELSE, take all the new material, 
!           with the rest coming from the top sediment
! NOTE: each time calculate the fraction of dissolved calcite taken from the new sedimentary calcite,
!       and also the fraction of dissolved calcite taken from the top sedimentary calcite,
!       so that the dissolution of stable isotopes can be simplified
! NOTE: care must be taken to ensure that no divide-by-zero errors occur
!       (i.e., test for zero calciate and aragonite contents of new and top sediment)
! (a) calculate potential calcite dissolution flux
    dis_cal = interp_4D(lookup_sed_dis_cal,D,dCO3_cal,frac_cal_top,fCorg_cal, &
      & lookup_D_max,lookup_dCO3_max,lookup_frac_max,lookup_fCorg_max, &
      & lookup_i_D_min,lookup_i_D_max, &
      & lookup_i_dCO3_min,lookup_i_dCO3_max, &
      & lookup_i_frac_min,lookup_i_frac_max, &
      & lookup_i_fCorg_min,lookup_i_fCorg_max)
    dis_cal = conv_cal_mol_cm3 * dis_cal
! (b) calculate actual dissolution flux
    IF (dis_cal <= 0.0) THEN
      seddis(ised_cal) = 0.0
    ELSEIF (dis_cal >= (sednew(ised_cal) + sedtop(ised_cal))) THEN
      seddis(ised_cal) = sednew(ised_cal) + sedtop(ised_cal)
    ELSE
      seddis(ised_cal) = dis_cal
    ENDIF
! (c) calculate interface dissolution flux - top and new sediment fractions
    IF (dis_cal <= 0.0) THEN
      frac_caldisi_new = 0.0
      frac_caldisi_top = 0.0
    ELSEIF (dis_cal <= sednew(ised_cal)) THEN
      IF (sednew(ised_cal) > 0.0) THEN
	frac_caldisi_new = seddis(ised_cal) / sednew(ised_cal)
      ELSE
	frac_caldisi_new = 0.0
      ENDIF
      frac_caldisi_top = 0.0
    ELSEIF (dis_cal >= (sednew(ised_cal) + sedtop(ised_cal))) THEN
      frac_caldisi_new = 1.0
      frac_caldisi_top = 1.0
    ELSE
      frac_caldisi_new = 1.0
      IF (sedtop(ised_cal) > 0.0) THEN
	frac_caldisi_top = (seddis(ised_cal) - sednew(ised_cal)) / sedtop(ised_cal)
      ELSE
	frac_caldisi_top = 0.0
      ENDIF
    ENDIF
! (d) calculate homogeneous dissolution flux - top and new sediment fractions
    IF (dis_cal <= 0.0) THEN
      frac_caldish_new = 0.0
      frac_caldish_top = 0.0
    ELSEIF (dis_cal >= (sednew(ised_cal) + sedtop(ised_cal))) THEN
      frac_caldish_new = 1.0
      frac_caldish_top = 1.0
    ELSE
      frac_caldish_new = dis_cal / (sednew(ised_cal) + sedtop(ised_cal))
      frac_caldish_top = dis_cal / (sednew(ised_cal) + sedtop(ised_cal))
    ENDIF
! (e) calculate weighted dissolution flux - top and new sediment fractions
    IF (sedopt(sopt_sed_rdis_fixed) == yes) THEN
      frac_caldis_new = par_sed_fraci * frac_caldisi_new + (1.0 - par_sed_fraci) * frac_caldish_new
      frac_caldis_top = par_sed_fraci * frac_caldisi_top + (1.0 - par_sed_fraci) * frac_caldish_top
    ELSE
!     NOTE: calculated from ratio of dissolution to sedimentation flux
      IF (seddis(ised_cal) > sednew(ised_cal)) THEN
	rdis_cal = 1.0
      ELSEIF (sednew(ised_cal) > 0.0) THEN
	rdis_cal = (seddis(ised_cal) / sednew(ised_cal))**2.0
      ELSE
	rdis_cal = 0.0
      ENDIF
      frac_caldis_new = rdis_cal * frac_caldisi_new + &
	& (1.0 - rdis_cal) * (par_sed_fraci * frac_caldisi_new + (1.0 - par_sed_fraci) * frac_caldish_new)
      frac_caldis_top = rdis_cal * frac_caldisi_top + &
	& (1.0 - rdis_cal) * (par_sed_fraci * frac_caldisi_top + (1.0 - par_sed_fraci) * frac_caldish_top)
    ENDIF

! *** calculate the aragonite dissolution flux ***
! NOTE: the dissolution flux is calculated in units of (mol cm-2 yr);
!       this must be converted back to cm3 cm-2 yr
! NOTE: for description, see calcite dissolution calculation above
! (a) calculate potential aragonite dissolution flux
    dis_arg = interp_4D(lookup_sed_dis_arg,D,dCO3_arg,frac_arg_top,fCorg_arg, &
      & lookup_D_max,lookup_dCO3_max,lookup_frac_max,lookup_fCorg_max, &
      & lookup_i_D_min,lookup_i_D_max, &
      & lookup_i_dCO3_min,lookup_i_dCO3_max, &
      & lookup_i_frac_min,lookup_i_frac_max, &
      & lookup_i_fCorg_min,lookup_i_fCorg_max)
    dis_arg = conv_arg_mol_cm3 * dis_arg
! (b) calculate actual dissolution flux
    IF (dis_arg <= 0.0) THEN
      seddis(ised_arg) = 0.0
    ELSEIF (dis_arg >= (sednew(ised_arg) + sedtop(ised_arg))) THEN
      seddis(ised_arg) = sednew(ised_arg) + sedtop(ised_arg)
    ELSE
      seddis(ised_arg) = dis_arg
    ENDIF
! (c) calculate interface dissolution flux - top and new sediment fractions
    IF (dis_arg <= 0.0) THEN
      frac_argdisi_new = 0.0
      frac_argdisi_top = 0.0
    ELSEIF (dis_arg <= sednew(ised_arg)) THEN
      IF (sednew(ised_arg) > 0.0) THEN
	frac_argdisi_new = seddis(ised_arg) / sednew(ised_arg)
      ELSE
	frac_argdisi_new = 0.0
      ENDIF
      frac_argdisi_top = 0.0
    ELSEIF (dis_arg >= (sednew(ised_arg) + sedtop(ised_arg))) THEN
      frac_argdisi_new = 1.0
      frac_argdisi_top = 1.0
    ELSE
      frac_argdisi_new = 1.0
      IF (sedtop(ised_arg) > 0.0) THEN
	frac_argdisi_top = (seddis(ised_arg) - sednew(ised_arg)) / sedtop(ised_arg)
      ELSE
	frac_argdisi_top = 0.0
      ENDIF
    ENDIF
! (d) calculate homogeneous dissolution flux - top and new sediment fractions
    IF (dis_arg <= 0.0) THEN
      frac_argdish_new = 0.0
      frac_argdish_top = 0.0
    ELSEIF (dis_arg >= (sednew(ised_arg) + sedtop(ised_arg))) THEN
      frac_argdish_new = 1.0
      frac_argdish_top = 1.0
    ELSE
      frac_argdish_new = dis_arg / (sednew(ised_arg) + sedtop(ised_arg))
      frac_argdish_top = dis_arg / (sednew(ised_arg) + sedtop(ised_arg))
    ENDIF
! (e) calculate wieghted dissolution flux - top and new sediment fractions
    IF (sedopt(sopt_sed_rdis_fixed) == yes) THEN
      frac_argdis_new = par_sed_fraci * frac_argdisi_new + (1.0 - par_sed_fraci) * frac_argdish_new
      frac_argdis_top = par_sed_fraci * frac_argdisi_top + (1.0 - par_sed_fraci) * frac_argdish_top
    ELSE
!     NOTE: calculated from ratio of dissolution to sedimentation flux
      IF (seddis(ised_arg) > sednew(ised_arg)) THEN
	rdis_arg = 1.0
      ELSEIF (sednew(ised_arg) > 0.0) THEN
	rdis_arg = (seddis(ised_arg) / sednew(ised_arg))**2.0
      ELSE
	rdis_arg = 0.0
      ENDIF
      frac_argdis_new = rdis_arg * frac_argdisi_new + &
	& (1.0 - rdis_arg) * (par_sed_fraci * frac_argdisi_new + (1.0 - par_sed_fraci) * frac_argdish_new)
      frac_argdis_top = rdis_arg * frac_argdisi_top + &
	& (1.0 - rdis_arg) * (par_sed_fraci * frac_argdisi_top + (1.0 - par_sed_fraci) * frac_argdish_top)
    ENDIF

! *** calculate isotope dissolution fluxes ***
! 'bulk' isotope tracers
    seddis(ised_13C_cal) = frac_caldis_new * sednew(ised_13C_cal) + &
      & frac_caldis_top * sedtop(ised_13C_cal)
    seddis(ised_13C_arg) = frac_argdis_new * sednew(ised_13C_arg) + &
      & frac_argdis_top * sedtop(ised_13C_arg)
    seddis(ised_14C_cal) = frac_caldis_new * sednew(ised_14C_cal) + &
      & frac_caldis_top * sedtop(ised_14C_cal)
! 'proxy' isotope tracers
    seddis(ised_13C_calfp) = frac_caldis_new * sednew(ised_13C_calfp) + &
      & frac_caldis_top * sedtop(ised_13C_calfp)
    seddis(ised_13C_calfb) = frac_caldis_new * sednew(ised_13C_calfb) + &
      & frac_caldis_top * sedtop(ised_13C_calfb)
    seddis(ised_18O_calfp) = frac_caldis_new * sednew(ised_18O_calfp) + &
      & frac_caldis_top * sedtop(ised_18O_calfp)
    seddis(ised_18O_calfb) = frac_caldis_new * sednew(ised_18O_calfb) + &
      & frac_caldis_top * sedtop(ised_18O_calfb)

! *** calculate age dissolution fluxes ***
    seddis(ised_age_cal)  = frac_caldis_new * sednew(ised_age_cal) + &
      & frac_caldis_top * sedtop(ised_age_cal)

! *** check lookup table integrety (if requested) ***
    IF (runopt(ropt_debug_sed_lib) == yes) THEN
      CALL chk_calc_sed_dis_CaCO3_lookup(frac_cal_top,fCorg,conc_O2,dis_cal,sedpar,sedtracer)
    ENDIF

  END SUBROUTINE calc_sed_dis_CaCO3_lookup
! ***************************************************************


! **************************************************************
! *** calculate sediment opal diagenesis via a look-up table ***
! **************************************************************
  SUBROUTINE calc_sed_dis_opal_lookup(seddis,sednew,sedtop,sedpar,sedtracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sed_com_par)::seddis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_com_par)::sednew
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_com_par)::sedtop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::sedtracer
! local variables
    REAL(KIND=fp_1)::frac_opal    ! volume fraction of opal in the sediment (cm3 cm-3 solids)
    REAL(KIND=fp_1)::conc_Si      ! bottom water [Si] (mol kg-1)
    REAL(KIND=fp_1)::T            ! sediment surface temeprature (K)
    REAL(KIND=fp_1)::KSi          ! estimated opal dissolution constant
    REAL(KIND=fp_1)::opaltorefrac ! %refrac/%opal
    REAL(KIND=fp_1)::f_opal       ! opal flux to sediments (umol cm-2 yr-1)
    REAL(KIND=fp_1)::dis_opal     ! raw opal dissolution flux from lookup table
    REAL(KIND=fp_1)::sed_vol_top  ! sediment solids volume (in top sediment)

! *** calculate sediment diagenesis defining variables ***
! sediment surface temperature (K)
    T = sedpar(ised_par_T)
! bottom-water [Si] (mol kg-1)
    conc_Si = sedtracer(it_Si)
! volume fraction of opal in sediments
    sed_vol_top = sedtop(ised_cal)    + &
      &           sedtop(ised_arg)    + &
      &           sedtop(ised_opal)   + &
      &           sedtop(ised_refrac) + &
      &           sedtop(ised_FeO)    + &
      &           sedtop(ised_Corg)
    frac_opal = sedtop(ised_opal) / sed_vol_top
! make sasymptotic [Si] dependent on the ratio of sediment (top) refrac to opal (wt%/wt%) (if selected)
! NOTE: cap ratio at 'par_sed_opal_Sitoopalmax'
    IF (sedopt(sopt_sed_diagen_AltoasymSi) == yes) THEN
      IF ((conv_refrac_cm3_g * sedtop(ised_refrac)) > &
	& (par_sed_opal_Sitoopalmax * (conv_opal_cm3_g * sedtop(ised_opal)))) THEN
	opaltorefrac = par_sed_opal_Sitoopalmax
      ELSE
	opaltorefrac = (conv_refrac_cm3_g * sedtop(ised_refrac)) / (conv_opal_cm3_g * sedtop(ised_opal))
      END IF
    ELSE
      opaltorefrac = 0.0
    ENDIF
! base opal dissolution rate constant
    IF (sedopt(sopt_sed_diagen_AltoKSi) == yes) THEN
!      KSi = par_sed_opal_KSi0 + (0.08 + 0.14 * 1.0 / (0.12 + opaltorefrac)) / conv_yr_s
!      KSi = (0.030 + 0.175 * 1.0 / (0.040 + opaltorefrac)**0.625) / conv_yr_s
      KSi = (0.0500 + 0.0550 * 1.0 / (0.0164 + opaltorefrac)**0.7500) / conv_yr_s
    ELSE
      KSi = par_sed_opal_KSi0
    ENDIF

! *** calculate the opal dissolution flux ***
! NOTE: the dissolution flux is calculated in units of (mol cm-2 yr);
!       this must be converted back to cm3 cm-2 yr
! NOTE: the procedure for calculating 'interface' dissolution of opal
!       from new sedimenting and top sediment material is:
!       (1) IF the estimated dissolution flux < 0.0, reset to 0.0
!           (this is possible, becuase of the use of extrapolation for values falling outside of 
!           the look-up table bounds)
!       (2) ELSEIF the estimated dissolution flux is less than the new material flux (less a token fraction),
!           then take all opal required for the dissolution flux from the new material 
!           (less a token fraction)
!       (3) ELSEIF the estimated dissolution flux is greater or equal to the combined inventories of 
!           new material and top material,
!           then cap the dissolution flux at this total
!       (4) ELSE, take all the new material, 
!           with the rest coming from the top sediment
! (a) calculate potential opal dissolution flux
    dis_opal = interp_5D(lookup_sed_dis_opal,frac_opal,conc_Si,T,KSi,opaltorefrac, &
      & lookup_opalpc_max,lookup_concSi_max,lookup_T_max,lookup_KSi0_max,lookup_opaltorefrac_max, &
      & lookup_i_opalpc_min,lookup_i_opalpc_max, &
      & lookup_i_concSi_min,lookup_i_concSi_max, &
      & lookup_i_T_min,lookup_i_T_max, &
      & lookup_i_KSi0_min,lookup_i_KSi0_max, &
      & lookup_i_opaltorefrac_min,lookup_i_opaltorefrac_max)
    dis_opal = conv_opal_mol_cm3 * dis_opal
! (b) calculate actual dissolution flux
    IF (dis_opal <= 0.0) THEN
      seddis(ised_opal) = 0.0
    ELSEIF (dis_opal >= (sednew(ised_opal) + sedtop(ised_opal))) THEN
      seddis(ised_opal) = sednew(ised_opal) + sedtop(ised_opal)
    ELSE
      seddis(ised_opal) = dis_opal
    ENDIF

! *** check lookup table integrety (if requested) ***
    IF (runopt(ropt_debug_sed_lib) == yes) THEN
      CALL chk_calc_sed_dis_opal_lookup(frac_opal,conc_Si,T,KSi,opaltorefrac,dis_opal)
    ENDIF

  END SUBROUTINE calc_sed_dis_opal_lookup
! ***************************************************************


! *****************************************************
! *** calculate sediment interface remineralization ***
! *****************************************************
  SUBROUTINE calc_sed_dis_CaCO3_interf(dissed,newsed,sedtop,sedpar,Dcalsat,Dargsat,sedporostop)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sed_com_par)::dissed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_com_par)::newsed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_com_par)::sedtop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in)::Dcalsat
    REAL(KIND=fp_1),INTENT(in)::Dargsat
    REAL(KIND=fp_1),INTENT(in)::sedporostop
! local variables
    REAL(KIND=fp_1)::ohmcal_bot     ! estimated OHM(cal) at bottom depth of the sedimentary layer (calcite)
    REAL(KIND=fp_1)::ohmcal_av      ! estimated average OHM(cal) over undersaturated sediment (calcite)
    REAL(KIND=fp_1)::ohmarg_bot     ! estimated OHM(cal) at bottom depth of the sedimentary layer (aragonite)
    REAL(KIND=fp_1)::ohmarg_av      ! estimated average OHM(cal) over undersaturated sediment (aragonite)
    REAL(KIND=fp_1)::Afrac_undersat ! area fraction undersaturated
    REAL(KIND=fp_1)::dis_frac_cal   ! dissolution fraction (calcite)
    REAL(KIND=fp_1)::dis_frac_arg   ! dissolution fraction (aragonite)
    REAL(KIND=fp_1)::new_sed_th     ! thickness of new sediment
    REAL(KIND=fp_1)::rem_sed_th     ! thickness of sediment required to complete dissolution layer
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::dis_layer ! dissolution layer

! *** construct dissolution layer ***
! calculate thickness of new sediment
! NOTE: use the porosity value for the top sediment layer
    new_sed_th = (newsed(ised_cal) + newsed(ised_arg) + newsed(ised_opal) + &
      & newsed(ised_refrac) + newsed(ised_FeO) + newsed(ised_Corg)) / (1.0 - sedporostop)
! calculate required thickness of top sediment to form a 'par_sed_interf_th' cm thick interface layer
! NOTE: check whether the thickness of new sediment is greater than the required interface layer thickness
    IF (new_sed_th > par_sed_interf_th) THEN
      IF (runopt(ropt_warn) == yes) THEN
	PRINT*,"WARNING: thickness of sedimenting material is greater than (cm): ",par_sed_interf_th
      ENDIF
      program_error = yes
      warning(iwarn_newsedth) = yes
      rem_sed_th = 0.0
    ELSE
      rem_sed_th = par_sed_interf_th - new_sed_th
    ENDIF
! calculate the sediment material in the interface layer
! NOTE: the material required from the top layer is simply calculated as the proportion of the 
!       total top layer thickness that the required thickness represents, 
!       multiplied by the total sediment in the top layer
    dis_layer(:) = newsed(:) + (rem_sed_th / par_sed_top_th) * sedtop(:)

! *** calculate fraction of calcite that will dissolve in the interface layer ***
! NOTE: this current version which uses mid-depths of the sedimentary depth layers,
! is based on the estimated calcite compensation depth, rather than an ohm_cal = 1.0,
! which means that there will be some degree or error introduced (as the calsatD is only approximated)
    IF (Dcalsat >= sedpar(ised_par_D_bot)) THEN
! if the calcite saturation depth is below the bottom of the sedimentary depth layer,
! then there is super-saturation w.r.t. calcite throughout the depth interval => no dissolution
      dis_frac_cal = 0.0
    ELSE
      IF (Dcalsat <= sedpar(ised_par_D_top) .and. sedpar(ised_par_ohmcal) <= 1.0) THEN
! - if the calcite saturation depth is above the top of the sedimentary depth layer,
! then there is under-saturation w.r.t. calcite throughout the depth interval => full dissolution
! NOTE: also ensure that ohm_cal <= 1.0 so that numerical errors do not arrise
! (ohm_cal can under some circumstances be > 1.0 due to errors in estimating Dcalsat)
! NOTE: the factor of '0.01' is to convet the dissolution rate constant from % to a fraction
	dis_frac_cal = 0.01 * conv_yr_d * par_caldis_k * (1.0 - sedpar(ised_par_ohmcal))**par_caldis_exp
! cap returned value at '0.999999' (i.e., vitrually 100% dissolution)
! NOTE: a very small fraction of CaCO3 is left undissolved to allow the tracking of sediment layer age
	IF (dis_frac_cal > 0.999999) dis_frac_cal = 0.999999
      ELSE
! - the calcite saturation depth must fall within the top and bottom of the sedimentary depth layer,
! therefore carry out the following steps:
! (i)   calculate the area fraction of sediment which is undersaturated ('Afrac_undersat')
! (ii)  estimate OHM(cal) at the bottom of the sedimentary layer (assume linear relationship)
! (iii) estimate average OHM(cal) over the undersaturation sediment (i.e., between Dcalsat and D_bot)
! (iv)  ensure that the average OHM(cal) value is <= 1.0 to avoid numerical errors (cap at 1.0 if not)
! (v)   calculate dissolution
! (vi)  cap returned value, taking into account area fraction undersaturated,
! i.e., fractional dissolution is capped at 1.0 for only this fraction
	Afrac_undersat = (sedpar(ised_par_D_bot) - Dcalsat) / &
	  & (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
	ohmcal_bot     = 1.0 + (sedpar(ised_par_ohmcal) - 1.0) * &
	  & (sedpar(ised_par_D_bot) - Dcalsat) / (sedpar(ised_par_D_mid) - Dcalsat)
	ohmcal_av      = (1.0 + ohmcal_bot) / 2.0
	IF (ohmcal_av > 1.0) ohmcal_av = 1.0
	dis_frac_cal = 0.01 * conv_yr_d *  par_caldis_k * (1.0 - ohmcal_av)**par_caldis_exp
	IF (dis_frac_cal > (0.999999 * Afrac_undersat)) dis_frac_cal = 0.999999 * Afrac_undersat
      ENDIF
    ENDIF

! *** calculate fraction of aragonite that will dissolve in the interface layer ***
! NOTE: this current version which uses mid-depths of the sedimentary depth layers,
! is based on the estimated aragonite compensation depth, rather than an ohm_arg = 1.0,
! which means that there will be some degree or error introduced (as the argsatD is only approximated)
    IF (Dargsat >= sedpar(ised_par_D_bot)) THEN
! if the aragonite saturation depth is below the bottom of the sedimentary depth layer,
! then there is super-saturation w.r.t. aragonite throughout the depth interval => no dissolution
      dis_frac_arg = 0.0
    ELSE
      IF (Dargsat <= sedpar(ised_par_D_top) .and. sedpar(ised_par_ohmarg) <= 1.0) THEN
! - if the aragonite saturation depth is above the top of the sedimentary depth layer,
! then there is under-saturation w.r.t. aragonite throughout the depth interval => full dissolution
! NOTE: also ensure that ohm_arg <= 1.0 so that numerical errors do not arrise
! (ohm_arg can under some circumstances be > 1.0 due to errors in estimating Dargsat)
! NOTE: the factor of '0.01' is to convet the dissolution rate constant from % to a fraction
	dis_frac_arg = 0.01 * conv_yr_d * par_argdis_k * (1.0 - sedpar(ised_par_ohmarg))**par_argdis_exp
! cap returned value at '1.0' (i.e., 100% dissolution)
! NOTE: no aragonite need be left undissolved, because sediment age etc. are tied to %calcite
	IF (dis_frac_arg > 1.0) dis_frac_arg = 1.0
      ELSE
! - the aragonite saturation depth must fall within the top and bottom of the sedimentary depth layer,
! therefore carry out the following steps:
! (i)   calculate the area fraction of sediment which is undersaturated ('Afrac_undersat')
! (ii)  estimate OHM(arg) at the bottom of the sedimentary layer (assume linear relationship)
! (iii) estimate average OHM(arg) over the undersaturation sediment (i.e., between Dargsat and D_bot)
! (iv)  ensure that the average OHM(arg) value is <= 1.0 to avoid numerical errors (cap at 1.0 if not)
! (v)   calculate dissolution
! (vi)  cap returned value, taking into account area fraction undersaturated,
! i.e., fractional dissolution is capped at 1.0 for only this fraction
	Afrac_undersat = (sedpar(ised_par_D_bot) - Dargsat) / &
	  & (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
	ohmarg_bot     = 1.0 + (sedpar(ised_par_ohmarg) - 1.0) * &
	  & (sedpar(ised_par_D_bot) - Dargsat) / (sedpar(ised_par_D_mid) - Dargsat)
	ohmarg_av      = (1.0 + ohmarg_bot) / 2.0
	IF (ohmarg_av > 1.0) ohmarg_av = 1.0
	dis_frac_arg = 0.01 * conv_yr_d * par_argdis_k * (1.0 - ohmarg_av)**par_argdis_exp
	IF (dis_frac_arg > (1.0 * Afrac_undersat)) dis_frac_arg = 1.0 * Afrac_undersat
      ENDIF
    ENDIF

! *** calculate sediment dissolution ***
! primary dissolution fluxes
    dissed(ised_cal)  = dis_frac_cal * dis_layer(ised_cal)
    dissed(ised_arg)  = dis_frac_arg * dis_layer(ised_arg)
! secondary dissolution fluxes
! NOTE: check that the calcite or aragonite contents of the dissolution layer are not zero,
!       in order to avoid divide-by-zero problems
    IF (dis_layer(ised_cal) > 0.0) THEN
      dissed(ised_13C_cal)   = dis_layer(ised_13C_cal)   * (dissed(ised_cal) / dis_layer(ised_cal))
      dissed(ised_13C_calfp) = dis_layer(ised_13C_calfp) * (dissed(ised_cal) / dis_layer(ised_cal))
      dissed(ised_13C_calfb) = dis_layer(ised_13C_calfb) * (dissed(ised_cal) / dis_layer(ised_cal))
      dissed(ised_18O_calfp) = dis_layer(ised_18O_calfp) * (dissed(ised_cal) / dis_layer(ised_cal))
      dissed(ised_18O_calfb) = dis_layer(ised_18O_calfb) * (dissed(ised_cal) / dis_layer(ised_cal))
    ELSE
      dissed(ised_13C_cal)   = 0.0
      dissed(ised_13C_calfp) = 0.0
      dissed(ised_13C_calfb) = 0.0
      dissed(ised_18O_calfp) = 0.0
      dissed(ised_18O_calfb) = 0.0
    ENDIF
    IF (dis_layer(ised_arg) > 0.0) THEN
      dissed(ised_13C_arg) = dis_layer(ised_13C_arg) * (dissed(ised_arg) / dis_layer(ised_arg))
    ELSE
      dissed(ised_13C_arg) = 0.0
    ENDIF

  END SUBROUTINE calc_sed_dis_CaCO3_interf
! ********************************************************


! **************************************
! *** MIX SEDIMENT STACK SUB_LAYERS  ***
! **************************************
  SUBROUTINE sed_mix(n_sed_mix_com_min,sed,sedtop,sedpar,sedmixk,sedtopth,sedstacktopth,sedporos,sedporostop)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::n_sed_mix_com_min
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(0:n_sed_mix,1:n_sed_mix_com_min)::sed
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(1:n_sed_mix_com_min)::sedtop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_sed_mix)::sedmixk
    REAL(KIND=fp_1),INTENT(in)::sedtopth
    REAL(KIND=fp_1),INTENT(in)::sedstacktopth
    REAL(KIND=fp_1),INTENT(in)::sedporos
    REAL(KIND=fp_1),INTENT(in)::sedporostop
! local variables
    INTEGER::d
    REAL(KIND=fp_1)::r_sed_por                                 ! porosity ratio correction factor
    REAL(KIND=fp_1)::sed_stack_top_th                          !
    REAL(KIND=fp_1)::sed_stack_bot_th                          !
    REAL(KIND=fp_1),DIMENSION(0:n_sed_mix)::mix                ! adapted sediment mixing rate profile
    REAL(KIND=fp_1),DIMENSION(1:n_sed_mix_com_min)::exe              ! exchange sedimentary material
    REAL(KIND=fp_1),DIMENSION(0:n_sed_mix,1:n_sed_mix_com_min)::dsed ! sediment composition change
    REAL(KIND=fp_1),DIMENSION(1:n_sed_mix_com_min)::dsedtop          ! sediment top composition change
    
! *** mix the sediment stack sub-layer within the mixing depth ***

! NOTE: restrictions on the dimensioning of arrays in f90 requires that 
!       the partial sediment stack passed into this procedure has 
!       an index of 'n_sed_mix' for the top (incomplete) sub-layer, and '0' for the bottom
! NOTE: the sediment mixing rate profile has a corresponing ordering, with the maximum mixing rate 
!       (i.e., nearest the sediment surface), with the highest ('n_sed_mix') index
! NOTE: sedimentary layers are sequentially mixed downwards in pairs
! NOTE: the sediment stack must be mixed in with the top layer, in addition to the sediment stack 
!       being mixed internally
! NOTE: the mixing rate between the sediment stack and the top layer takes its value from 
!       the 'sedmixk' array at index=0 
!       (care must be taken as this is the opposite end of the array from the next mixing rate down)
! NOTE: for the mixing between the top sediment layer and the sediment stack,
!       because the porosity of these two sediment zones may be different,
!       the (solids) volume of top layer sedimentary material is normalized by the ratio of
!       the top layer to the sediment stack porosity    

! (1) initialize (zero) sediment compositional change variable
    dsed(:,:)  = 0.0
    dsedtop(:) = 0.0
! (2) calculate thickness of top (incomplete) sub-layer, 
!     together with thickness of bottom sub-layer as if the sediment depth was fixed at n_sed_mix cm
    sed_stack_top_th = sedstacktopth
    sed_stack_bot_th = 1.0 - sed_stack_top_th
! (3) calculate exchange fraction between each pair of mixed layer sub-layers
    d = n_sed_mix
    mix(d) = sedmixk(n_sed_mix)
    DO d = (n_sed_mix - 1),1,-1
      mix(d) = (1.0 - sed_stack_top_th) * sedmixk(d + 1) + sed_stack_top_th * sedmixk(d)
    END DO
    d = 0
    mix(d) = sedmixk(1)
! (4) calculate top (incomplete) sub-layer mixing exchange
    d = n_sed_mix
    exe(:) = mix(d) * ((sed_stack_top_th * sed(d - 1,:)) - sed(d,:))
    dsed(d,:)     = dsed(d,:)     + exe(:)
    dsed(d - 1,:) = dsed(d - 1,:) - exe(:)
! (5) calculate complete sub-layer mixing exchanges
    DO d = (n_sed_mix - 1),2,-1
      exe(:) = mix(d) * (sed(d - 1,:) - sed(d,:))
      dsed(d,:)     = dsed(d,:)     + exe(:)
      dsed(d - 1,:) = dsed(d - 1,:) - exe(:)
    END DO
! (6) calculate bottom (treated as incomplete) sub-layer mixing exchange
    d = 1
    exe(:) = mix(d) * ((sed_stack_bot_th * sed(d - 1,:)) - (sed_stack_bot_th * sed(d,:)))
    dsed(d,:)     = dsed(d,:)     + exe(:)
    dsed(d - 1,:) = dsed(d - 1,:) - exe(:)
! (7) calculate mixing exchange with sediment top ('well-mixed') layer
    r_sed_por = (1.0 - sedporos) / (1.0 - sedporostop)
    d = n_sed_mix
    exe(:) = sedmixk(0) * &
      & ((sed_stack_top_th * r_sed_por * sedtop(:) / sedtopth) - sed(d,:))
    dsed(d,:)  = dsed(d,:)  + exe(:)
    dsedtop(:) = dsedtop(:) - exe(:)
    d = (n_sed_mix - 1)
    exe(:) = sedmixk(0) * &
      & ((sed_stack_bot_th * r_sed_por * sedtop(:) / sedtopth) - (sed_stack_bot_th * sed(d,:)))
    dsed(d,:)  = dsed(d,:)  + exe(:)
    dsedtop(:) = dsedtop(:) - exe(:)
! (8) update mixed sub-layer values
    sed(:,:)  = sed(:,:) + dsed(:,:)
    sedtop(:) = sedtop(:) + dsedtop(:)
    
  END SUBROUTINE sed_mix
! **************************************


! **************************************************************************
! *** check calculation of CaCO3 sediment diagenesis via a look-up table ***
! **************************************************************************
  SUBROUTINE chk_calc_sed_dis_CaCO3_lookup(fraccal,fCorg,o2,discal,sedpar,sedtracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::fraccal ! dry mass fraction
    REAL(KIND=fp_1),INTENT(in)::fCorg   ! Corg rain rate (mol cm-2 yr-1)
    REAL(KIND=fp_1),INTENT(in)::o2      ! [O2]
    REAL(KIND=fp_1),INTENT(in)::discal  ! 
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_tracer)::sedtracer
! local variables
    REAL(KIND=fp_1)::S
    REAL(KIND=fp_1)::T
    REAL(KIND=fp_1)::P
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca
! assumed state values
    REAL(KIND=fp_1)::temp    = 4.0
    REAL(KIND=fp_1)::sal     = 35.0
    REAL(KIND=fp_1)::raincal = 18.E-6
    REAL(KIND=fp_1)::raincly = 18.E-5
    REAL(KIND=fp_1)::tco2    = 2235E-6
! input variables
    REAL(KIND=fp_1)::dco3
    REAL(KIND=fp_1)::water_z,rainorg,calpc
    REAL(KIND=fp_1)::alk
    REAL(KIND=fp_1)::co2,co3,hco3,calsat,calohm
! result variables
    REAL(KIND=fp_1)::ttrorg,ttrcal,ttrtc,ttr13tc,ttr14tc,ttral
    REAL(KIND=fp_1)::diftc,difal
    REAL(KIND=fp_1)::dis_cal_1,dis_cal_2
! 
    REAL(KIND=fp_1)::const_025 = 0.25
    REAL(KIND=fp_1)::const_010 = 0.10
    REAL(KIND=fp_1)::const_020 = 0.20
    REAL(KIND=fp_1)::const_005 = 0.05
    
! *** initialize calculation ***
! set local constants
    water_z = sedpar(ised_par_D_mid)
    rainorg = fCorg
    calpc   = fraccal
    S       = sal
    T       = temp + const_zeroc
    P       = water_z / 10.0
    dco3    = sedpar(ised_par_dCO3cal)
! restrict input variable ranges due to diagenesis model stability issues
    IF (rainorg > 50.0E-06) rainorg = 50.0E-06
    
! 
    CALL co2const(S,T,P,k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)

! calculate calcite dissolution flux using a fixed value fo alk and estimating tco2 (as per look-up table)
    CALL findalk_dCO3_cal(dco3,tco2,water_z,temp,sal,alk, &
      & k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    CALL sedflx_cal(temp,sal,water_z,alk,tco2,o2,raincal,rainorg,raincly, &
      & const_025,const_010,const_020,const_005,co3,dco3, &
      & ttrorg,ttrcal,ttrtc,ttr13tc,ttr14tc,ttral,calpc,calohm,diftc,difal, &
      & k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    dis_cal_1 = ttrcal

! calculate calcite dissolution flux using actual values for alk and co2
    tco2 = sedtracer(it_CO2)
    alk  = sedtracer(it_alk)
    CALL calc_carb_cal_archer(temp,sal,water_z,alk,tco2,co2,hco3,co3,calsat,calohm, &
      & k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    CALL sedflx_cal(temp,sal,water_z,alk,tco2,o2,raincal,rainorg,raincly, &
      & const_025,const_010,const_020,const_005,co3,dco3, &
      & ttrorg,ttrcal,ttrtc,ttr13tc,ttr14tc,ttral,calpc,calohm,diftc,difal, &
      & k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    dis_cal_2 = ttrcal

! contrast look-up table estimated and calculated values
! NOTE: for a valid comparison 'dis_cal' units must be converted from (g cm-2 yr-1) to (umol cm-2 yr-1)
! NOTE: the other calcite dissolution flux estimates must be converted from (mol cm-2 yr) to (umol cm-2 yr-1)
    PRINT'(6F9.1)',water_z,1.0E+06 * dco3,100.0 * calpc,1.0E+06 * rainorg,1.0E+06 * tco2,1.0E+06 * alk
    PRINT*,"look-up table estimation                      : ",1.0E+06 * discal / conv_cal_mol_cm3
    PRINT*,"calculated value (fixed tCO2 + estimated alk) : ",1.0E+06 * dis_cal_1
    PRINT*,"calculated value (actual tCO2 and alk)        : ",1.0E+06 * dis_cal_2
    PRINT*,"---"

  END SUBROUTINE chk_calc_sed_dis_CaCO3_lookup
! **************************************************************************


! *************************************************************************
! *** check calculation of opal sediment diagenesis via a look-up table ***
! *************************************************************************
  SUBROUTINE chk_calc_sed_dis_opal_lookup(fracopal,concSi,T,KSi0,opaltorefrac,disopal)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::fracopal     ! volume fraction of opal in the sediment (cm3 cm-3 solids)
    REAL(KIND=fp_1),INTENT(in)::concSi       ! bottom water [Si] (mol kg-1)
    REAL(KIND=fp_1),INTENT(in)::T            ! sediment surface temeprature (K)
    REAL(KIND=fp_1),INTENT(in)::KSi0         ! estimated opal dissolution constant
    REAL(KIND=fp_1),INTENT(in)::opaltorefrac ! %refrac/%opal
    REAL(KIND=fp_1),INTENT(in)::disopal      ! raw opal dissolution flux from lookup table
! local variables
    INTEGER::t_loop_opal
    REAL(KIND=fp_1)::opalpc
    REAL(KIND=fp_1)::conc_Si_eq
    REAL(KIND=fp_1)::fSi
    REAL(KIND=fp_1)::fSi_new
    REAL(KIND=fp_1)::fSi_old
    REAL(KIND=fp_1)::f_opal_dis
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::z
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::sed_pore
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::sed_conc_Si
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::DSi
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::KSi

! *** filter input data ***
! restrict input variable ranges due to diagenesis model stability issues
    IF (fracopal < 0.02) THEN
      opalpc = 0.02
    ELSE
      opalpc = fracopal
    ENDIF

! *** initialize model parameters ***
! sediment sub-layers
    opal_delz(:) = 0.2
    opal_delz(1) = 0.0
    opal_delz(opal_zimax + 1:opal_nzmax) = 1.0
! zero variables
    conc_Si_eq = 0.0
    fSi_new = 0.0
    fSi_old = 0.0
! calculate sub-layer mid-depth profile
    z(:) = calc_z()
! calculate porosity profile
    sed_pore(:) = calc_pore(z(:))

! *** calculate opal dissolution flux ***
! initialize sediment system
    CALL init_sedflx_Si(concSi,sed_conc_Si(:))
! calculate [Si] diffusivity profile
    DSi(:) = calc_DSi(T,sed_pore(:))
! calculate equilibrium [Si]
    conc_Si_eq = calc_conc_Si_eq(T)
! convert equilibriun [Si] to sediment asymptotic [Si] according sediment detrital content
    conc_Si_eq = conv_si_eqlibtoasympt(opaltorefrac) * conc_Si_eq
! calculate silica dissolution flux
    DO t_loop_opal = 1,opal_t_loop_max
! store previous value of the Si dissolution flux
      fSi_old = fSi_new
! calculate Si dissolution rate constant profile
!      KSi(:) = calc_KSi(KSi0,conc_Si_eq,sed_conc_Si(:))
      KSi(:) = calc_KSi_alt(z(:),KSi0,T,conc_Si_eq,sed_conc_Si(:))
! update [Si] profile
      CALL runge_kutta_4_opal(opal_t_step,opalpc,sed_conc_Si,sed_pore,DSi,KSi)
! test Si sediment interface flux
      fSi_new = DSi(2) * conv_cm3_kg * (sed_conc_Si(2) - sed_conc_Si(1)) / (0.5 * opal_delz(2))
      IF (ABS((fSi_new - fSi_old) / fSi_new) <= opal_df_thesh) THEN
	f_opal_dis = fSi_new * conv_mol_umol * conv_yr_s
	EXIT
      ELSE
	IF (t_loop_opal >= opal_t_loop_max) THEN
	  PRINT*,""
	  PRINT*,"No stable solution possible"	
	  PRINT*,""
	  EXIT
	ENDIF
      END IF
    ENDDO

! *** test results ***
! contrast look-up table estimated and calculated values
! NOTE: for a valid comparison 'dis_cal' units must be converted from (g cm-2 yr-1) to (umol cm-2 yr-1)
! NOTE: the other calcite dissolution flux estimates must be converted from (mol cm-2 yr) to (umol cm-2 yr-1)
    PRINT'(6F10.3)',fracopal * 100.0,opalpc * 100.0,concSi * conv_mol_umol,T,KSi0 * conv_yr_s,opaltorefrac
    PRINT'(A30,F10.3)',"total simulation time (yrs) : ",t_loop_opal * (opal_t_step / conv_yr_s)
    PRINT'(A30,F10.3)',"look-up table estimation    : ",1.0E+06 * disopal * conv_opal_cm3_mol
    PRINT'(A30,F10.3)',"calculated value            : ",f_opal_dis
    PRINT*,"---"
    
  END SUBROUTINE chk_calc_sed_dis_opal_lookup
! *************************************************************************


END MODULE sue_sed_lib


